/** (c) Brendan Macmillan 2001, licensed under GNU's GPL 2.
		(If your project is not GPL, then a fee is due - see license on website)
    Website: http://www.csse.monash.edu.au/~bren/JSX
    Website: http://freshmeat.net/projects/jsx
    List (can read without subscribing, need to join to post):
      http://groups.yahoo.com/group/JSX-ideas/messages
		Commercial licensing enquiries only:
			bren@mail.csse.monash.edu.au **/

/* Example usage code:

import JSX.*;
class DTDTest {
  static public void main(String[] args) throws Exception {
    new ObjOut(                                             //we are writing
      new Config().setDTDGenerator(                         //Config object
        new DTDGenerator(new java.io.FileWriter("my.dtd"))  //set Writer
      )
    ).writeObject(new DTDTest());     	// <-------- put your object here.
  }
}
*/


/** NOTE: each ref-type within the class is recorded.  Note that this recording
	* also includes any subtypes eg:  (a | suba | b | subb1 | subb2)*
	**/

package JSX;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;


//Problem: "DTD" confuses convention of class's first letter lowecase...
public class DTDGenerator {
	//static final boolean DEBUG = true;
	static final boolean DEBUG = false;

	PrintWriter out;
	public DTDGenerator(Writer writer) {
		this.out = new PrintWriter(writer);
	}


/**
	* populate().
	* ----------
	* Called from putAlias() in XMLSerialize.
	* It must be called before the alias is depopulated (eg if alias-ref is off)
	**/
	HashSet classList = new HashSet();
	void populate(Object obj) {	//what's a better name?
		classList.add(obj.getClass());
	}


/** 
	* generate().
	* -----------
	* Programmer: called from end of commonSer() in XMLSerialize.
	* Note that it must be called after the end of serialization, and not
	* after a custom serialization call.  This is why we don't call it from
	* ObjOut.writeObjectOverride() nor XMLSerialize.serialize().
	*
	* API dicussion:
	* Every object graph configures us, populating the class
	* list, but the user needs to explicitly call writeDTD.  Thus, we can
	* collect over several object graphs, and it is clearer than writing both
	* XML and DTD at the same time (hidden and unexpected).
	* We could have a switch to control this behaviour (background vs explicit),
	* but better to choose just one or the other.
	**/

	HashSet entitiesDone = new HashSet();
	void generate() {
		if (DEBUG) System.err.println(classList);
		classList.add(NULL);	//special cases
		classList.add(ALIASREF);
		Object[] c = (Object[]) classList.toArray(new Object[0]);
		classList.clear();	//to prevent garbled results, on subsequent calls
		entitiesDone.clear();
		writeDTD(c);
		//out.close();
	}


/** KEY METHOD **/
	public void writeDTD(Object[] classList) {
		for (int i=0; i<classList.length; i++) {
		//LOGIC
			SepFields r;
			Object obj = classList[i];
			if (obj instanceof Class) {
				Class clazz = (Class) obj;
				r = custom(clazz);
				if (r==null) r = sepFields(clazz);
				//add "JSX format" fields
				r.primList.add(XMLSerialize.ALIAS_ID_TOKEN);
			} else if (obj instanceof SyntheticClass) {
				r = ((SyntheticClass) obj).sepFields;	//neither can have ID
			} else throw new InternalError("expected Class or SyntheticClass");

			r.primList.add(XMLSerialize.NAME_TOKEN);	//problem: not *required*

			ArrayList allRefList = getSubclasses(r.refList, classList);

		//PRESENTATION	
			String className;
			if (obj instanceof Class) {	//use polymorphism, in a wrapper
				Class clazz = (Class) obj;
		 		className = getClassName(clazz);
			} else if (obj instanceof SyntheticClass) {
				className = ((SyntheticClass) obj).className;
			} else throw new InternalError("expected Class or SyntheticClass");
			writeElement(className, allRefList);	//ESCAPE used here, too
			writeAttList(className, r.primList);
		}
		out.flush();
	}



	static final SyntheticClass NULL;
	static final SyntheticClass ALIASREF;
	static {
		//NULL
		NULL = new SyntheticClass(XMLSerialize.NULL_TOKEN);
		//ALIASREF
		ALIASREF = new SyntheticClass(XMLSerialize.ALIAS_TAG_TOKEN);
		ALIASREF.sepFields.primList.add(XMLSerialize.ALIAS_ATTR_TOKEN);
	}

	String getClassName(Class clazz) {
		String className;
		if (clazz.isArray()) {
			if (clazz.getComponentType()==byte.class)
				className = XMLSerialize.BINARY_DATA_TOKEN;
			else
				className = XMLSerialize.getArrayType(clazz);
		}
		else className = clazz.getName();
		return ParseUtilities.escapeDollar(className);	//escape both
	}

	SepFields sepFields(Class clazz) {
		//"ref" includes all reference types: classes, arrays and interfaces
		Field[] f = clazz.getDeclaredFields();	//need INHERITED ONES
		ArrayList primCache = new ArrayList();
		ArrayList refCache = new ArrayList();
		for (int i=0; i<f.length; i++) {
			Field field = f[i];
			Class type = field.getType();
			if (type.isPrimitive() || type==String.class) primCache.add(field);
			else refCache.add(field);
		}
		return new SepFields(refCache, primCache);
	}

	SepFields custom(Class clazz) {
		SepFields r = null;
		if (clazz.isArray()) {
			r = new SepFields();
			r.primList.add(XMLSerialize.LENGTH_TOKEN);	//just need the String
			Class component = clazz.getComponentType(); //array "subclass" handled?
			if (!component.isPrimitive())
				r.refList.add(clazz.getComponentType());	//Strings are treated as ref
			else if (component==byte.class) {
				r.primList.add(XMLSerialize.VALUE_TOKEN);	//same as wrapper
			} else {	//array of primitives: problem
				//for (int i=0; ....) {
				//	r.primList.add("a"+i);	//but... depends on specific length!!
				//array of primitives requires *runtime* length number of attr
				//}
			}
		} else if (clazz==Boolean.class ||			//all wrappers are the same
				clazz==Byte.class ||
				clazz==Short.class ||
				clazz==Integer.class ||
				clazz==Long.class ||
				clazz==Float.class ||
				clazz==Double.class ||
				clazz==String.class ||			//char data - may differ
				clazz==Character.class) {		//char data - may differ
			r = new SepFields();
			r.primList.add(XMLSerialize.VALUE_TOKEN);	//just need the String
				//would be nicer if Class was the same...
		} else if (clazz==Vector.class) {
			r = new SepFields();
			r.refList.add(Object.class);		//need the class (for subclasses)
		} else if (clazz==Hashtable.class) {	//not even numbers; but little point
			r = new SepFields();
			r.refList.add(Object.class);		//need the class (for subclasses)
		} else if (clazz==Class.class) {
			r = new SepFields();
			r.primList.add(XMLSerialize.CLASSNAME_TOKEN);	//just need the String
		}
		return r;
	}


//LOGIC CODE
//----------
	//input:  list of Fields
	//output: list of Strings
	ArrayList getSubclasses(ArrayList refList, Object[] classList) {
		if (DEBUG) System.err.println("Starting Subclasses");
		ArrayList result = new ArrayList();
		for (Iterator i=refList.listIterator(); i.hasNext(); ) {
			Class type;
			Object ref = i.next();
			if (ref instanceof Field)	//use polymorphism instead
				type = ((Field)ref).getType();
			else if (ref instanceof Class)
				type = (Class) ref;
			else throw new InternalError("Expected a Field or Class; got a "+ref.getClass());
		if (DEBUG) System.err.println("\tField of type: "+type);

	result.add(type);	//no presentation code here

	//ENTITY def
		if (!entitiesDone.contains(type)) {
			entitiesDone.add(type);
			out.print("<!ENTITY % "+getClassName(type)+" \"(");
				boolean firstOne = true;	
				for (int j=0; j<classList.length; j++) {
					Object obj = classList[j];
					if (obj instanceof Class) {	//use polymorphism, in a wrapper
						if (type.isAssignableFrom((Class)obj)) {
							//superclass          subclass
							//result.add(obj);	//no presentation code here
							if (firstOne) firstOne = false;
							else out.print("|");
							out.print(getClassName((Class)obj));
						}
					} else if (obj instanceof SyntheticClass) {
						//result.add(obj);	//null and aliasref always assignable
						if (firstOne) firstOne = false;
						else out.print("|");
						out.print(((SyntheticClass)obj).className);
					} else throw new InternalError("expected Class or SyntheticClass");
				}
			out.println(")\">");
			if (DEBUG) System.err.println("\t\tresult so far: "+result);
			}
		}


		return result;
	}



//PRESENTATION CODE
//-----------------
	//REF: present ref fields as Element content. eg:
	//  <!ELEMENT clazz (x|y|z)*>
	void writeElement(String elementName, ArrayList refList) {
		out.print("<!ELEMENT "+elementName+" ");
		// S -> e				  //if empty
		//   S -> ( F E  	//Start
		//     F -> | F		//keep on adding
		//     F -> e			//end of list
		//   E -> ) 		  //End
		//if (refList.size()==0) out.print("EMPTY");
		boolean firstOne = true;
		for (Iterator i=refList.listIterator(); i.hasNext(); ) {
			if (firstOne) {
				firstOne = false;
				out.print("(");	//now we know non-empty
			} else {
				out.print("|");
			}
			Object obj = i.next();
			String className;		//exact same code as above
			if (obj instanceof Class) {	//use polymorphism, in a wrapper
				Class clazz = (Class) obj;
		 		className = getClassName(clazz);
			} else if (obj instanceof SyntheticClass) {
				className = ((SyntheticClass) obj).className;
			} else throw new InternalError("expected Class or SyntheticClass");
			out.print("%"+className+";");	//entities...
		}
		//NB: (x|y|z)* syntax: we parse by knowing the first one
		if (firstOne) out.print("EMPTY");	//if empty
		else out.print(")*");
		out.println(">");	//close "<!ELEMENT "
	}

	//PRIM: present primitive fields as an attribute list. eg:
	//  <!ATTLIST clazz
	//    i CDATA #IMPLIED
	//    j CDATA #IMPLIED
	//    k CDATA #IMPLIED
	//	>
	void writeAttList(String className, ArrayList attList) {
		boolean firstPrim = true;
		if (DEBUG) System.err.println(attList);
		for (Iterator i=attList.listIterator(); i.hasNext(); ) {
			if (firstPrim) {
				out.println("<!ATTLIST " + className);	//ln eases layout
				firstPrim = false;	//in case it has none
			}
			Object f = i.next();
			if (f instanceof Field)
				out.println("  " + ((Field)f).getName() + " CDATA #IMPLIED");
			else if (f instanceof String)
				out.println("  " + f + " CDATA #IMPLIED");

		}
		if (!firstPrim) {	//only if it had at least one
											//what about JSX stuff, like obj-name and alias-ID?
			out.println(">");
		}
	}



/** some rudimentary test code **/
	int a;
	int b;
	int c;
	Object o;
	public static void main(String[] args) throws Exception {
		Class[] c = {String.class, DTDGenerator.class, XMLSerialize.class};
		DTDGenerator generator=new DTDGenerator(new OutputStreamWriter(System.out));
		generator.writeDTD(c);
	}

}


	class SyntheticClass {
		String className;
		SepFields sepFields = new SepFields();
		SyntheticClass(String className) {
			this.className = className;
		}
	}

	//for return values and class content
	class SepFields {
		ArrayList refList;
		ArrayList primList;
		SepFields() {
			this.refList = new ArrayList();
			this.primList = new ArrayList();
		}
		SepFields(ArrayList refList, ArrayList primList) {
			this.refList = refList;
			this.primList = primList;
		}
	}

/** FUTURE COOL.
	* Use entities for subclasses, instead of listing them all explicitly.
	* More readable (one entity per ref field) and shorter DTD.
	* It shifts work to the validator
	* In implementation, repeated content disappears.
	* And we can do (&Object; &Object;)* for Hashtable easily.
	**/

/** IMPLEMENTATION/ARCHECTURE: to do.
	* Use polymorphism transparently for SyntheticClass and Field.
	*	 - enable specification of optional or not (eg alias-ID)
	**/

/** MINOR BUGS: to do.
	* Strings can appear as "aliases" as an attribute.
	* Switch for "aliasID": whether String occur as attributes or tags.
	**/

/** MINOR FEATURES: to do.
	*	change JSX format: "ArrayOf-binary-data", not "ArrayOf-ArrayOf-byte"
	**/

/** Limitations.
	*	Custom serialized ref types.
			Soln: use "Object" catch-all, when writeObject
	*	Custom serialized prim types.
			Soln: JSX format to have a tag per primitive
	* Arrays of primitives.
			Soln: JSX format to have a tag per primitive

	* NB: "ref" = reference type, including objects, arrays and interfaces.
	* NB: "prim" = primtive types, including Strings in some cases.
	**/
