import java.util.*;
import java.io.*;
import JSX.*;

public class Loader
{
    public Loader()
    {

    }
  
    // Method to load data from a given XML document containing
    // serialization data.
    // Inputs: String filename - takes the name of the XML document
    public Vector loadFromXML(String filename)
    {
	try
	{
	    // Create a vector to hold the empires
	    Vector v;
	    // Set up object to handle input stream from XML file
	    ObjIn in = new ObjIn(new FileReader(filename));
	    // load data from XML file and cast to vector
	    v = (Vector) in.readObject();	
	    //return the empires vector
	    return v;
	}
	catch(Exception e)
	{
	    Vector err = new Vector();
	    System.out.println("Error in loading Empires XML document");
	    return err;
	}
    }

     
    // Method to load empires and set their initial unit allocations.
    public Vector initEmpires(/*Map map*/)
    {
	try
	{
	    // Create vectors to hold the empires and units
	    Vector empires = loadFromXML("empires.xml");
	    Vector initialForces = loadFromXML("initialforces.xml");
	    Vector spaceSources = (Vector)loadFromXML("spaceforces.xml");
	    Vector groundSources = (Vector)loadFromXML("groundforces.xml");
	    
	    // iterate through empires
	    for(int empiresIter = 0; empiresIter < empires.size(); empiresIter++)
		{
		    Empire empire = (Empire) empires.elementAt(empiresIter);
		    System.out.println("Empire: "+empire.toString());

		    // Vector for Planets
		    Vector planets = (Vector) empire.getPlanets();

		    // Iterate through all planets in the current empire
		    for(int planetsIter = 0; planetsIter < planets.size(); planetsIter++)
		    {
			// current planet to add forces to
			Planet curPlanet = (Planet)planets.elementAt(planetsIter);
			// current planet being checked
			Planet onPlanet;
			// initial planet data
			Vector initialPlanets;
			System.out.println("  Getting Forces for "+curPlanet.getName());
			
			// Create initial SPACE unit for current planet
			String spaceUnitName = curPlanet.getName() + " Space Fleet 1";
			Unit spaceUnit = new Unit(spaceUnitName, "Neutral");
			
			// Create initial GROUND unit for current planet
			String groundUnitName = curPlanet.getName() + " Ground Army 1";
			Unit groundUnit = new Unit(groundUnitName, "Neutral");

			// Iterate through all planets in initial data list 
			// (find the current planet's initial forces)
			for(int initialIter = 0; initialIter < initialForces.size(); initialIter++)
			{
			    initialPlanets = (Vector) initialForces.elementAt(initialIter);
			    String planetName = (String)initialPlanets.elementAt(0);
			    System.out.println("    Checking planet "+planetName);
			    
			    // planet found in initial data.  Load units appropriately
			    if(planetName.equals(curPlanet.getName()) == true)
			    {
				// test message
				System.out.println("      Found planet "+planetName);
				// load space unit list
				Vector spaceForces = (Vector) initialPlanets.elementAt(1);
				// load ground unit list
				Vector groundForces = (Vector) initialPlanets.elementAt(2);
				// test messages
				System.out.println(spaceForces.toString());
				System.out.println(groundForces.toString());
				// clone space forces into unit
				// start by looking through all required forces
				for(int spaceForcesIter = 0; spaceForcesIter < spaceForces.size(); spaceForcesIter++)
				{
				    // get the force name
				    String spaceForce = (String)spaceForces.elementAt(spaceForcesIter);

				    // look through all source info
				    for(int spaceSourcesIter = 0; spaceSourcesIter < spaceSources.size(); spaceSourcesIter++)
				    {
					// get the source
					SpaceForce spaceSource = (SpaceForce) spaceSources.elementAt(spaceSourcesIter);
					// get the source name
					String sourceName = spaceSource.getName();
					if(sourceName.equals(spaceForce) == true)
					{
					    System.out.println("        Found source data for "+sourceName);
					    SpaceForce force = (SpaceForce) spaceSource.clone();
					    System.out.println("        " + force.toString());

					    // ### This is where we'd add the location information
					   
					    // finally add something to something else
					    // add force to spaceUnit (of the current planet)
					    // 
					    spaceUnit.addForce(force);
					    // stop looking.  We found the source data
					    break;
					}
				    }
				}
				
				// clone GROUND forces into unit
				// start by looking through all required forces
				for(int groundForcesIter = 0; groundForcesIter < groundForces.size(); groundForcesIter++)
				{
				    // get the force name
				    String groundForce = (String)groundForces.elementAt(groundForcesIter);

				    // look through all source info
				    for(int groundSourcesIter = 0; groundSourcesIter < groundSources.size(); groundSourcesIter++)
				    {
					// get the source
					GroundForce groundSource = (GroundForce) groundSources.elementAt(groundSourcesIter);
					// get the source name
					String sourceName = groundSource.getName();

					if(sourceName.equals(groundForce) == true)
					{
					    System.out.println("        Found source data for "+sourceName);
					    GroundForce force = (GroundForce) groundSource.clone();
					    System.out.println("        " + force.toString());

					    // ### This is where we'd add the location information
					   
					    // Add force to groundUnit (of the current planet)
					    groundUnit.addForce(force);
					    // stop looking.  We found the source data for this ground units
					    break;
					}
				    }
				}
				// stop looking.  we're done with this planet
				break;
			    }
			}
			
			// actually add the units to the current empire
			empire.addSpaceUnit(spaceUnit);
			empire.addGroundUnit(spaceUnit);			
		    }
		}

	    
	    // return the loaded empires vector
	    return empires;
	}
	catch(Exception e)
	{
	    Vector err = new Vector();
	    System.out.println("Error in loading Empires XML document");
	    return err;
	}
    }
}
