import java.util.*;
import java.io.*;


public class AIIntelligenceFile implements Serializable
{

	TreeMap<String, AIPlanetIntelligence> intelligenceFile;

	String playerName;

	Vector<Empire> allEmpires;

	Map map;

	public AIIntelligenceFile(String name, Vector empires, Map map)
	{
		playerName = name;

		this.map = map;

		allEmpires = new Vector<Empire>();

		intelligenceFile = new TreeMap<String, AIPlanetIntelligence>();

		for (int i = 0; i < empires.size(); i++)
		{
			Empire e = (Empire) empires.elementAt(i);

			for (int j = 0; j < e.getPlanets().size(); j++)
			{
				add((Planet)e.getPlanets().elementAt(j));
			}

			allEmpires.addElement(e.clone());
		}
	}

	public void add(Planet p)
	{
		AIPlanetIntelligence intel = new AIPlanetIntelligence(p, map.getTurn(), playerName);

		if (intelligenceFile.containsKey(p.getName()))
		{
			intelligenceFile.remove(p.getName());
		}

		intelligenceFile.put(p.getName(), intel);
	}

	public void updateUnits(Planet p, int turn, Map map, Vector neutralSpace, Vector neutralGround)
	{
		AIPlanetIntelligence intel = get(p.getName());

		if (intel != null)
		{
			intel.reliability = turn;

			intel.units = new Vector<Unit>();

			Location location = p.getLocation();

			for (int i = 0; i < map.getPlayers().size(); i++)
			{
				Player player = (Player) map.getPlayers().elementAt(i);

				if (!player.getName().equals(playerName))
				{
					for (int t = 0; t < player.getEmpire().getSpaceUnits().size(); t++)
					{
						Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(t);

						if (!sUnit.isEmpty() && sUnit.getLocation().getPositionX() == location.getPositionX() && sUnit.getLocation().getPositionY() == location.getPositionY())
						{
							intel.units.addElement(sUnit.clone());
						}
					}
					for (int t = 0; t < player.getEmpire().getGroundUnits().size(); t++)
					{
						Unit gUnit = (Unit) player.getEmpire().getGroundUnits().elementAt(t);

						if (!gUnit.isEmpty() && !gUnit.getLoaded() && gUnit.getLocation().getPositionX() == location.getPositionX() && gUnit.getLocation().getPositionY() == location.getPositionY())
						{
							intel.units.addElement(gUnit.clone());
						}
					}
				}
			}
			for (int t = 0; t < neutralSpace.size(); t++)
			{
				Unit sUnit = (Unit) neutralSpace.elementAt(t);

				if (!sUnit.isEmpty() && sUnit.getLocation().getPositionX() == location.getPositionX() && sUnit.getLocation().getPositionY() == location.getPositionY())
				{
					intel.units.addElement(sUnit.clone());
				}
			}
			for (int t = 0; t < neutralGround.size(); t++)
			{
				Unit gUnit = (Unit) neutralGround.elementAt(t);

				if (!gUnit.isEmpty() && !gUnit.getLoaded() && gUnit.getLocation().getPositionX() == location.getPositionX() && gUnit.getLocation().getPositionY() == location.getPositionY())
				{
					intel.units.addElement(gUnit.clone());
				}
			}

		}
	}


	public AIPlanetIntelligence get(String key)
	{
		return intelligenceFile.get(key);
	}

	public String toString()
	{
		return intelligenceFile.toString();
	}


	// Update all entries in the intelligence file with the aggression felt against that
	// specific player.
	public void updateAggression(String player, double aggressionChange)
	{
		if (!player.equals(playerName))
		{
			Collection<AIPlanetIntelligence> intel = intelligenceFile.values();

			Iterator<AIPlanetIntelligence> iterator = intel.iterator();

			while (iterator.hasNext())
			{
				AIPlanetIntelligence file = iterator.next();

				if (file.planet.getOwner().equals(player))
				{
					file.aggression += aggressionChange;

					if (file.aggression < 0)
					{
						file.aggression = 0;
					}
				}
			}
		}
	}


	public void turnRefresh(Map map)
	{
		for (int p = 0; p < map.getPlayers().size(); p++)
		{
			Empire empire = ((Player)map.getPlayers().elementAt(p)).getEmpire();

			for (int i = 0; i < empire.getPlanets().size(); i++)
			{
				Planet planet = (Planet) empire.getPlanets().elementAt(i);

				AIPlanetIntelligence intel = intelligenceFile.get(planet.getName());

				intel.planet.setOwner(planet.getOwner());
			}
		}

		Collection<AIPlanetIntelligence> intel = intelligenceFile.values();

		Iterator<AIPlanetIntelligence> iterator = intel.iterator();

		while (iterator.hasNext())
		{
			AIPlanetIntelligence file = iterator.next();
			file.aggression -= 0.2f;

			if (file.aggression < 0)
			{
				file.aggression = 0;
			}
		}
	}

	// Returns the known victory points of a player as a ratio.
	public double getVictoryPoints(Player player, Map map)
	{
		double result = 0;

		String race = player.getEmpire().getRace();

		for (int i = 0; i < allEmpires.size(); i++)
		{
			Empire empire = allEmpires.elementAt(i);

			if (empire.getRace().equals(race))
			{
				empire.getPlanets().clear();

				Collection<AIPlanetIntelligence> intel = intelligenceFile.values();

				Iterator<AIPlanetIntelligence> iterator = intel.iterator();

				while (iterator.hasNext())
				{
					AIPlanetIntelligence file = iterator.next();
					if (file.planet.getOwner().equals(player.getName()))
					{
						empire.getPlanets().addElement(file.planet);
					}
				}

				empire.calculateVictoryPoints(map);

				result = (empire.getVictoryPoints()+0.0)/empire.getMaxVictoryPoints();

				break;
			}
		}

		return result;
	}

	// Updates the empire listing based on new intelligence.
	public void updateEmpire(Empire empire)
	{
		for (int i  = 0; i < allEmpires.size(); i++)
		{
			if (empire.getRace().equals(allEmpires.elementAt(i).getRace()))
			{
				allEmpires.setElementAt(empire.clone(), i);
				break;
			}
		}
	}

	public int getTech(Player player)
	{
		for (int i = 0; i < allEmpires.size(); i++)
		{
			Empire empire = allEmpires.elementAt(i);

			if (empire.getRace().equals(player.getEmpire().getRace()))
			{
				return empire.getTech();
			}
		}
		return 0;
	}

}