import java.util.*;
import java.io.*;

public class AIPlanetIntelligence implements Serializable
{
	public Planet planet;

	public float aggression;

	public int distanceToThreats;
	public Vector<Unit> closestEnemyUnits;

	public int reliability;

	public int mood;
	public static final int DEFENSIVE = 0;
	public static final int NEUTRAL = 1;

	public Vector<Unit> units;

	public AIPlanetIntelligence(Planet p, int turn, String playerName)
	{
		planet = p.clone();

		units = new Vector<Unit>();

		Location location = planet.getLocation();

		for (int i = 0; i < location.getUnits().size(); i++)
		{
			Unit unit = (Unit) location.getUnits().elementAt(i);
			if (!unit.getOwner().equals(playerName))
			{
				units.addElement(unit.clone());
			}
		}

		reliability = turn;

		aggression = 0;
		distanceToThreats = 0;
		mood = NEUTRAL;

		closestEnemyUnits = new Vector<Unit>();
	}

	public String toString()
	{
		String result = "Planet:\n" + planet.toString() + "\n";
		for (int i = 0; i < units.size(); i++)
		{
			result += "\n" + units.elementAt(i).getForces();
		}
		result += "\n" + "Reliability: " + reliability + "\nAggresion: " + aggression+"\n";
		result += "Mood: " + mood + "\n";
		return result;
	}

	// Calculate the production importance based on the distance from enemies, and the distance
	// from the current target location.
	public double getProductionImportance(Location targetLocation, Map map, AIIntelligenceFile intelligenceFile)
	{
		double result = 1.0;

		result -= distanceToThreats/3.0;

		double maxThreat = getMaximumThreateningAggression(map, intelligenceFile);

		maxThreat = Math.min(maxThreat/5, 1.0);

		result += maxThreat * 0.5;

		if (result < 0.25)
		{
			result = 0.25;
		}

		if (targetLocation != null)
		{
			// Find distance to the target location.
			int distanceToTarget = map.distance(targetLocation, planet.getLocation());

			distanceToTarget = (distanceToTarget-1)/2;

			result += (3 - distanceToTarget)/6.0;
		}

		/*
		if (mood == DEFENSIVE)
		{
			result += 0.2;
		}
		*/

		//System.out.println(planet + " Production Importance: " + result);
		return result;
	}

	// Finds the maximum aggression among the nearest enemy forces to a specific planet.
	public double getMaximumThreateningAggression(Map map, AIIntelligenceFile intelligenceFile)
	{
		double result = 0;

		for (int i = 0; i < closestEnemyUnits.size(); i++)
		{
			Unit enemyUnit = closestEnemyUnits.elementAt(i);

			for (int t = 0; t < map.getPlayers().size(); t++)
			{
				Player p = (Player) map.getPlayers().elementAt(t);

				if (p.getName().equals(enemyUnit.getOwner()))
				{
					if (p.getEmpire().getPlanets().size() > 0)
					{
						AIPlanetIntelligence temp = intelligenceFile.get(((Planet)p.getEmpire().getPlanets().elementAt(0)).getName());

						if (temp.aggression > result)
						{
							result = temp.aggression;
						}
					}
					break;
				}
			}
		}

		return result;
	}


	// Checks whether the specified player is the only one with the closest units to this planet.
	public boolean isSoleAggressor(String aggressorName)
	{
		boolean result = true;

		for (int i = 0; i < closestEnemyUnits.size(); i++)
		{
			Unit enemyUnit = closestEnemyUnits.elementAt(i);

			if (!enemyUnit.getOwner().equals(aggressorName))
			{
				result = false;
			}

			if (enemyUnit.getLocation().getPlanets().size() == 0)
			{
				Random random = new Random(System.currentTimeMillis());

				if (random.nextInt(100) < 50)
				{
					result = false;
				}
			}
		}
		return result;
	}


}