import java.util.*;
import java.io.*;

public class AIUnit implements Serializable
{
	// The destination that this AIUnit is moving towards.
	// Will be left as null if the unit is not intended to move.
	public Location destination;

	// The unit that is part of this AI unit.
	public Unit unit;

	// The last location that this force came from.
	Location lastLocation;

	Random random;

	public AIUnit(Unit u, Location targetLocation)
	{
		unit = u;
		destination = targetLocation;
		random = new Random(System.currentTimeMillis());
		lastLocation = null;
	}

	// Moves this AIUnit towards its destination along the fastest route, and prepares them
	// for battle against any resistance in their path.
	public void move(Map map, Vector<Player> otherPlayers)
	{
		// Move towards destination, if we have one.
		if (destination != null && unit != null && !unit.isEmpty())
		{
			unit.calculateMinMove();

			Location currentLocation = unit.getLocation();

			int z = 0;
			for (int i = 0; i < unit.getMinMove(); z++ )
			{
				if (currentLocation.getPositionX() == destination.getPositionX() && currentLocation.getPositionY() == destination.getPositionY())
				{
					break;
				}

				// If we are in the asteroid belt, we can come out on any side of it.
				if (currentLocation.getAsteroids())
				{
					Vector<Location> adjacentLocations = map.getAsteroidAdjacent();

					int minDistance = 1000;
					Location closestLocation = null;

					for (int t = 0; t < adjacentLocations.size(); t++)
					{
						Location loc = adjacentLocations.elementAt(t);

						if (map.distance(loc, destination) < minDistance)
						{
							minDistance = map.distance(loc, destination);
							closestLocation = loc;
						}
					}
					currentLocation = closestLocation;
					i++;
				}
				else
				{
					int selection = random.nextInt(2);

					if ((selection == 0 && destination.getPositionX() - currentLocation.getPositionX() != 0) || destination.getPositionY() - currentLocation.getPositionY() == 0)
					{
						Location loc = map.getLocationAt(currentLocation.getPositionY(), currentLocation.getPositionX()-1);

						if (currentLocation.getPositionX() - destination.getPositionX() < 0)
						{
							loc = map.getLocationAt(currentLocation.getPositionY(), currentLocation.getPositionX()+1);
						}
						if (loc != null && loc.getValid() && (lastLocation == null || loc.getPositionY() != lastLocation.getPositionY() || loc.getPositionX() != lastLocation.getPositionX()))
						{
							boolean enemyPresence = false;

							if ((i > 0) && (i >= unit.getMinMove()-1) && (loc.getPositionX() != destination.getPositionX() || loc.getPositionY() != destination.getPositionY()))
							{
								if (loc.getPlanets().size() > 0)
								{
									if (!((Planet)loc.getPlanets().elementAt(0)).getOwner().equals(unit.getOwner()))
									{
										enemyPresence = true;
										//System.out.println("Afraid of enemy forces at " + loc);
									}
								}
							}

							if (!enemyPresence)
							{
								//System.out.println(unit.getName() + " has moved on X axis from " + currentLocation + " to " + loc);
								currentLocation = loc;
								i++;

								if (currentLocation.getAsteroids())
								{
									i++;
								}
							}
							else
							{
								break;
							}
						}
					}
					else
					{
						Location loc = map.getLocationAt(currentLocation.getPositionY()-1, currentLocation.getPositionX());

						if (currentLocation.getPositionY() - destination.getPositionY() < 0)
						{
							loc = map.getLocationAt(currentLocation.getPositionY()+1, currentLocation.getPositionX());
						}
						if (loc != null && loc.getValid() && (lastLocation == null || loc.getPositionY() != lastLocation.getPositionY() || loc.getPositionX() != lastLocation.getPositionX()))
						{
							boolean enemyPresence = false;

							if ((i > 0) && (i >= unit.getMinMove()-1) && (loc.getPositionX() != destination.getPositionX() || loc.getPositionY() != destination.getPositionY()))
							{
								if (loc.getPlanets().size() > 0)
								{
									if (!((Planet)loc.getPlanets().elementAt(0)).getOwner().equals(unit.getOwner()))
									{
										enemyPresence = true;
										//System.out.println("Afraid of enemy forces at " + loc);
									}
								}
							}

							if (!enemyPresence)
							{

								//System.out.println(unit.getName() + " has moved on Y axis from " + currentLocation + " to " + loc);
								currentLocation = loc;
								i++;
								if (currentLocation.getAsteroids())
								{
									i++;
								}
							}
							else
							{
								break;
							}
						}
					}
				}

				if (z > 1000)
				{
					//System.out.println(unit.getName() + " is stuck!");

					if (currentLocation.getPositionX() == destination.getPositionX())
					{
						int selection = random.nextInt(2);

						if (selection == 0)
						{
							Location loc = map.getLocationAt(currentLocation.getPositionY(), currentLocation.getPositionX()-1);
							if (loc != null && loc.getValid() && (lastLocation == null || loc.getPositionY() != lastLocation.getPositionY() || loc.getPositionX() != lastLocation.getPositionX()))
							{
								currentLocation = loc;
								i++;
								z = 0;
								if (currentLocation.getAsteroids())
								{
									i++;
								}
							}

						}
						else
						{
							Location loc = map.getLocationAt(currentLocation.getPositionY(), currentLocation.getPositionX()+1);
							if (loc != null && loc.getValid() && (lastLocation == null || loc.getPositionY() != lastLocation.getPositionY() || loc.getPositionX() != lastLocation.getPositionX()))
							{
								currentLocation = loc;
								i++;
								z = 0;
								if (currentLocation.getAsteroids())
								{
									i++;
								}
							}
						}
					}
					else if (currentLocation.getPositionY() == destination.getPositionY())
					{
						int selection = random.nextInt(2);

						if (selection == 0)
						{
							Location loc = map.getLocationAt(currentLocation.getPositionY()-1, currentLocation.getPositionX());
							if (loc != null && loc.getValid() && (lastLocation == null || loc.getPositionY() != lastLocation.getPositionY() || loc.getPositionX() != lastLocation.getPositionX()))
							{
								currentLocation = loc;
								i++;
								z = 0;
								if (currentLocation.getAsteroids())
								{
									i++;
								}

							}

						}
						else
						{
							Location loc = map.getLocationAt(currentLocation.getPositionY()+1, currentLocation.getPositionX());
							if (loc != null && loc.getValid() && (lastLocation == null || loc.getPositionY() != lastLocation.getPositionY() || loc.getPositionX() != lastLocation.getPositionX()))
							{
								currentLocation = loc;
								i++;
								z = 0;
								if (currentLocation.getAsteroids())
								{
									i++;
								}
							}
						}
					}
					if (z > 1000)
					{
						break;
					}
				}
			}
			setNewLocation(map, currentLocation);
		}
	}

	private void setNewLocation(Map map, Location newLocation)
	{
		Location l  = unit.getLocation();

		int distance = map.distance(l, newLocation);

		// We are moving out of the belt.
		if (l.getAsteroids())
		{
			distance = 1;
		}

		// Check if we just moved onto the asteroid belt.
		if (newLocation.getAsteroids())
		{
			distance += 1;
		}

		unit.setMinMove(distance);

		l.removeUnit(unit);
		unit.setLocation(newLocation);
		newLocation.addUnit(unit);
		lastLocation = l;

		for (int i = 0; i < unit.getForces().size(); i++)
		{
			SpaceForce sf = (SpaceForce) unit.getForces().elementAt(i);

			//System.out.println(unit.getOwner() + "'s unit " + unit.getName() + " with a " + sf + " at position " + newLocation);

			for (int j = 0; j < sf.getStorage().size(); j++)
			{
				Unit gUnit = (Unit) sf.getStorage().elementAt(j);

				Location gL = gUnit.getLocation();
				gL.removeUnit(gUnit);
				gUnit.setLocation(newLocation);
				newLocation.addUnit(unit);

				//System.out.println(gUnit.getOwner() + "'s unit " +gUnit.getName() + " at "+ newLocation);
			}
		}

		if (destination != null && destination.getPositionX() == newLocation.getPositionX() && destination.getPositionY() == newLocation.getPositionY())
		{
			groundInvasion(map, destination);
		}
	}

	// Rolls back a unit's movement to its previous location.
	public void rollBack()
	{
		Location l = unit.getLocation();
		l.removeUnit(unit);
		unit.setLocation(lastLocation);
		lastLocation.addUnit(unit);

		for (int i = 0; i < unit.getForces().size(); i++)
		{
			SpaceForce sf = (SpaceForce) unit.getForces().elementAt(i);

			//System.out.println(unit.getOwner() + "'s unit " + unit.getName() + " with a " + sf + " at position " + newLocation);

			for (int j = 0; j < sf.getStorage().size(); j++)
			{
				Unit gUnit = (Unit) sf.getStorage().elementAt(j);

				Location gL = gUnit.getLocation();
				gL.removeUnit(gUnit);
				gUnit.setLocation(lastLocation);
				lastLocation.addUnit(unit);
				//System.out.println(gUnit.getOwner() + "'s unit " +gUnit.getName() + " at "+ newLocation);
			}
		}
	}



	private void groundInvasion(Map map, Location location)
	{
		location = map.getLocationAt(location.getPositionY(), location.getPositionX());

		if (location.getPlanets().size() > 0)
		{
			Planet plan = (Planet) location.getPlanets().elementAt(0);

			for (int i = 0; i < unit.getForces().size(); i++)
			{
				SpaceForce sf = (SpaceForce) unit.getForces().elementAt(i);

				Vector storage = sf.getStorage();

				for (int j = 0; j < storage.size(); j++)
				{
					Unit groundUnit  = (Unit) storage.elementAt(j);

					if (plan.getOwner().equals(unit.getOwner()))
					{
						storage.removeElementAt(j);
						groundUnit.setLoaded(false);
						j--;
					}
					else
					{
						//System.out.println("Landing on " + plan);
						groundUnit.setGroundInvade(true);
					}
				}
			}
		}
	}
}
