import java.util.*;
import java.io.*;

public class Battle implements Serializable
{

	private Location location;
	private boolean space;
	private Vector combatants;
	private int battleCount;  // Used to keep track of how many combatants have acted this round.


	public Battle(Location locatio, boolean spac)
	{
		location = locatio;
		space = spac;
		combatants = new Vector();
		battleCount = 0;
	}

	public Vector getCombatants()
	{
		return combatants;
	}

	public Location getLocation()
	{
		return location;
	}

	public boolean getSpace()
	{
		return space;
	}

	public void addCombatant(Combatant c)
	{
		combatants.addElement(c);
	}

	public void removeCombatant(Combatant c)
	{
		combatants.removeElement(c);
	}

	public void setCombatants(Vector combat)
	{
		combatants = combat;
	}

	public boolean over()
	{
		return combatants.size() < 2;
	}

	public int getBattleCount()
	{
		return battleCount;
	}

	public boolean stillContested()
	{
		int count = 0;

		for (int i = 0; i < combatants.size(); i++)
		{
			Combatant c = (Combatant) combatants.elementAt(i);

			if (c.getAllForces().size() > 0)
			{
				count++;
			}
		}

		if (count > 1)
		{
			return true;
		}
		return false;
	}

	public void incrementBattleCount()
	{
		int neutrals = 0;

		for (int i = 0; i < combatants.size(); i++)
		{
			Combatant c = (Combatant) combatants.elementAt(i);

			if (c.getName().equals("Neutral") || c.getName().startsWith("AI "))
			{
				neutrals++;
			}
		}

		if (battleCount > combatants.size() - neutrals)
		{
			battleCount = 0;
		}
		battleCount++;
	}

	public boolean isCombatant(String playerName)
	{
		boolean result = false;

		for (int i = 0; i < combatants.size(); i++)
		{
			Combatant c = (Combatant) combatants.elementAt(i);
			if (c.getName().equals(playerName))
			{
				result = true;
			}
		}

		return result;
	}

	public void addUnit(Unit u)
	{
	    	boolean done = false;
		for (int i = 0; i < combatants.size(); i++)
		{
			Combatant c = (Combatant) combatants.elementAt(i);
			if (c.getName().equals(u.getOwner()))
			{
			    	done = true;
				c.addUnit(u);
			}
		}
		if (!done)
		{
			Combatant newC = new Combatant(u.getOwner());
			newC.addUnit(u);
			combatants.addElement(newC);
		}

	}

	public Vector getUnits()
	{
		Vector result = new Vector();

		for (int i = 0; i < combatants.size(); i++)
		{
			Combatant c = (Combatant) combatants.elementAt(i);

			for (int t = 0; t < c.getUnits().size(); t++)
			{
				Unit u = (Unit) c.getUnits().elementAt(t);

				result.addElement(u);
			}
		}

		return result;
	}

	// Returns the name of the battle's winner.
	public String getWinner()
	{
		String result = "";

		for (int i = 0; i < combatants.size(); i++)
		{
			Combatant c = (Combatant) combatants.elementAt(i);

			if (c.getSize() > 0)
			{
				result = c.getName();
			}
		}

		return result;
	}

	// Checks if there is still damage remaining to be allocated in this battle.
	public boolean moreDamageToAllocate()
	{
		for (int i = 0; i < combatants.size(); i++)
		{
			Combatant c = (Combatant) combatants.elementAt(i);

			if (c.getDamageToAllocate() > 0)
			{
				return true;
			}
		}
		return false;
	}


	// Returns a functional name for a battle.  Used for defensive redeploy, so it includes a planet's name or
	// just a moniker as a space battle.
	public String toString()
	{
		String result = "";

		if (location.getPlanets().size() > 0)
		{
			Planet plan = (Planet) location.getPlanets().elementAt(0);

			result += "Over " + plan.getName();
		}
		else
		{
			result += "In Deep Space";
		}

		return result;
	}


}

