import java.util.*;
import java.io.*;

public class Combatant implements Serializable
{

	private String name;
	private Vector units;
	private int damage;
	private int totalDamage;
	private int damageTaken;
	private int damageToAllocate;

	// A prioritized list of targets for this combatant's damage.
	private Vector<Combatant> targets;

	public Combatant(String nam)
	{
		name = nam;
		units = new Vector();
		damage = -1;
		totalDamage = 0;
		damageTaken = 0;
		damageToAllocate = 0;

		targets = new Vector<Combatant>();
	}

	public Vector getUnits()
	{
		return units;
	}

	public String getName()
	{
		return name;
	}

	public String toString()
	{
		return name;
	}

	public int getTotalDamage()
	{
		return totalDamage;
	}

	public int getDamageTaken()
	{
		return damageTaken;
	}

	public void setDamageTaken(int damT)
	{
		damageTaken = damT;
	}

	public int getArmor()
	{
		int result = 0;

		for (int i = 0; i < units.size(); i++)
		{
			Unit u = (Unit) units.elementAt(i);
			for (int k = 0; k < u.getForces().size(); k++)
			{
				Force f = (Force) u.getForces().elementAt(k);
				result += f.getCurrentArmor();
			}

		}

		return result;
	}

	public String getDamageRange()
	{
		int min = 0;
		int max = 0;

		for (int i = 0; i < units.size(); i++)
		{
			Unit u = (Unit) units.elementAt(i);
			for (int k = 0; k < u.getForces().size(); k++)
			{
				Force f = (Force) u.getForces().elementAt(k);

				if (f.getCommitted())
				{
					min += f.getMinAttack();
					max += f.getMaxAttack();
				}
			}

		}

		return min + " - " + max;
	}

	public Vector<Combatant> getTargets()
	{
		return targets;
	}

	public void setTargets(Vector<Combatant> setting)
	{
		targets = setting;
	}

	public void addUnit(Unit u)
	{
		units.addElement(u);
	}

	public int getSize()
	{
		int size = 0;

		for (int i = 0; i < units.size(); i++)
		{
		    	Unit u = (Unit) units.elementAt(i);
			size += u.getForces().size();
		}

	    return size;
	}

	public Vector getAllForces()
	{
		Vector results = new Vector();

		for (int i = 0; i < units.size(); i++)
		{
			Unit u = (Unit) units.elementAt(i);
			for (int k = 0; k < u.getForces().size(); k++)
			{
				Force f = (Force) u.getForces().elementAt(k);
				results.addElement(f);
			}
		}

		return results;
	}

	public void removeForce(Force f)
	{
		for (int i = 0; i < units.size(); i++)
		{
			Unit u = (Unit) units.elementAt(i);
			if (u.getForces().contains(f))
			{
				u.getForces().removeElement(f);
			}
		}
	}

	public int getDamage()
	{
		return damage;
	}

	public void setDamage(int dam)
	{
		damage = dam;
		totalDamage += damage;
	}

	public int getDamageToAllocate()
	{
		return damageToAllocate;
	}

	public void setDamageToAllocate(int dam)
	{
		damageToAllocate = dam;
	}

	public void restoreAllForces()
	{
		Vector f = getAllForces();

		for (int i = 0; i < f.size(); i++)
		{
			Force force = (Force) f.elementAt(i);
			force.restoreArmor();
		}
	}


	// This method checks to see if there are any committed forces remaining in the
	// battle, and returns the result as a boolean.  If false, then uncommitted forces
	// have become valid targets.

	public boolean remainingCommitted()
	{
		boolean result = false;

		Vector forces = getAllForces();

		for (int i = 0; i < forces.size(); i++)
		{
			Force f = (Force) forces.elementAt(i);

			if (f.getCommitted())
			{
				result = true;
				i = forces.size();
			}
		}

		return result;
	}


	// Ths method displays all information about the combatant as a single string.

	public String display()
	{
		int minDamage = 0;
		int maxDamage = 0;
		int armor = 0;

		String result = "\n";

		result += name + "\n-------------\n\n";

		for (int i = 0; i < units.size(); i++)
		{
			Unit u = (Unit) units.elementAt(i);

			result += u.getName() + "\n";

			for (int t = 0; t < u.getForces().size(); t++)
			{
				Force f = (Force) u.getForces().elementAt(t);

				result += "    " + (t+1) + ". " + f.getName() + "\n";

				minDamage += f.getMinAttack();
				maxDamage += f.getMaxAttack();
				armor += f.getArmor();
			}

			result += "\n";
		}

		result += "Attack Power: " + minDamage + "-" + maxDamage + "\nTotal Armor: " + armor;

		result += "\n\n";

		return result;
	}

}
