import java.util.*;
import java.io.*;

public class Empire implements Serializable
{

   	private String name;
	private String race;
	private String description;
	private String blurb;

	private int money;
	private int tech;
	private int espionage;
	private int maxEspionage;
	private int counterEspionage;
	private int maintenance;

	private int food;
	private int minerals;
	private int gas;

	// Current victory points, points needed for a win.

	private int victoryPoints;
	private int maxVictoryPoints;

	private Vector planets;
	private Vector spaceUnits;
	private Vector groundUnits;


	public Empire(String rac, int mone, int tec, int espionag, int maxEspionag, int counterEspionag, int maintenanc, int foo, int mineral, int ga, int maxVictory)
	{
		planets = new Vector();
		spaceUnits = new Vector();
		groundUnits = new Vector();

		race = rac;
		money = mone;
		tech = tec;
		espionage = espionag;
		maxEspionage = espionag;
		counterEspionage = counterEspionag;
		maintenance = maintenanc;

		food = foo;
		minerals = mineral;
		gas = ga;

		victoryPoints = 0;
		maxVictoryPoints = maxVictory;
	}

	// Clones this Empire object and returns the clone.
	public Empire clone()
	{
		Empire clone = new Empire(this.race, this.money, this.tech, this.espionage, this.maxEspionage, this.counterEspionage, this.maintenance, this.food, this.minerals, this.gas, this.maxVictoryPoints);
		clone.setName(this.name);
		clone.setVictoryPoints(this.victoryPoints);

		for (int i =0; i < planets.size(); i++)
		{
			Planet planet = (Planet) planets.elementAt(i);

			clone.addPlanet(planet.clone());
		}

		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit unit = (Unit) spaceUnits.elementAt(i);

			clone.addSpaceUnit(unit.clone());
		}

		for (int i = 0; i < groundUnits.size(); i++)
		{
			Unit unit = (Unit) groundUnits.elementAt(i);

			clone.addGroundUnit(unit.clone());
		}

		return clone;
	}

	public void setName(String nam)
	{
		name = nam;
	}

	public void setDescription(String describe)
	{
		description = describe;
	}

	public void addSpaceUnit(Unit u)
	{
		spaceUnits.addElement(u);
	}

	public void addGroundUnit(Unit u)
	{
		groundUnits.addElement(u);
	}

	public void addPlanet(Planet p)
	{
		planets.addElement(p);
	}

	public void setMoney(int mon)
	{
		money = mon;
	}

	public void calculateMaintenance()
	{
	    	maintenance = 0;
		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit u = (Unit) spaceUnits.elementAt(i);
			for (int k = 0; k < u.getForces().size(); k++)
			{
				Force f = (Force) u.getForces().elementAt(k);
				maintenance += f.getMaintenance();
			}
		}
		for (int i = 0; i < groundUnits.size(); i++)
		{
			Unit u = (Unit) groundUnits.elementAt(i);

			// Ground units on Warchor are exempt from paying maintenance.
			if (race.equals("Warchor") && (!u.isEmpty()) && ((Location)u.getLocation()).getPlanets().size() > 0 && ((Planet)((Location)u.getLocation()).getPlanets().elementAt(0)).getName().equals("Warchor"))
			{
				//System.out.println("No maintenance for ground forces on Warchor!");
			}

			else
			{
				for (int k = 0; k < u.getForces().size(); k++)
				{
					Force f = (Force) u.getForces().elementAt(k);
					maintenance += f.getMaintenance();
				}
			}
		}
	}

	public void setEspionage(int esp)
	{
		espionage = esp;
	}

	public void setCounterEspionage(int counterEsp)
	{
		counterEspionage = counterEsp;
	}

	public void setMaxEspionage(int max)
	{
		maxEspionage = max;
	}

	public void setTech(int tec)
	{
		tech = tec;
	}

	public void setFood(int foo)
	{
		food = foo;
	}

	public void setMinerals(int mine)
	{
		minerals = mine;
	}

	public void setPlanets(Vector plans)
	{
		planets = plans;
	}

	public void setGas(int ga)
	{
		gas = ga;
	}

	public String getName()
	{
	    	String returning = name;

		if (returning != null)
		{
			if (returning.length() > 13)
			{
				returning = returning.substring(0,13) + "..";
			}

			return returning;
		}
		else
		{
			return race;
		}
	}

	public String getRace()
	{
		return race;
	}

	public String toString()
	{
		return race;
	}

	public int getMaxEspionage()
	{
		return maxEspionage;
	}

	public Vector getPlanets()
	{
		return planets;
	}

	public Vector getSpaceUnits()
	{
	 	return spaceUnits;
	}

	public void setSpaceUnits(Vector v)
	{
		spaceUnits = v;
	}

	public Vector getGroundUnits()
	{
		return groundUnits;
	}

	public void setGroundUnits(Vector v)
	{
		groundUnits = v;
	}

	public int getMoney()
	{
		return money;
	}

	public int getTech()
	{
		return tech;
	}

	public int getMaintenance()
	{
		return maintenance;
	}

	public int getIncome()
	{
	    	int totalEconomy = 0;
		for (int i = 0; i < planets.size(); i++)
		{
			Planet p = (Planet) planets.elementAt(i);
			totalEconomy += p.getEconomy();
		}

		return totalEconomy-maintenance;
	}

	public int getFood()
	{
		return food;
	}

	public int getMinerals()
	{
		return minerals;
	}

	public int getGas()
	{
		return gas;
	}

	public void collect()
	{
		int totalEconomy = 0;
		int totalFood = 0;
		int totalMinerals = 0;
		int totalGas = 0;

		for (int i = 0; i < planets.size(); i++)
		{
			Planet p = (Planet) planets.elementAt(i);
			totalEconomy += p.getEconomy();

			// apply tech modifiers!

			totalFood += p.getFood();

			// Hydroponics at Tech 30.
			if (tech > 29)
			{
				// Add 1 food if planet produces any.
				if (p.getFood() > 0)
				{
					totalFood += 1;
				}
				//totalFood += (p.getFood() * 0.5);
			}

			totalMinerals += p.getMinerals();

			// Ore Refining at tech 40.
			if (tech > 39)
			{
				if (p.getMinerals() > 0)
				{
					totalMinerals += 1;
				}
				//totalMinerals += (p.getMinerals() * 0.5);
			}

			totalGas += p.getGas();

			// Gas refining at Tech 35.
			if (tech > 34)
			{
				if (p.getGas() > 0)
				{
					totalGas += 1;
				}
				//totalGas += (p.getGas() * 0.5);
			}

			p.restoreIndustry();
		}

		boolean drak = false;

		// Check if the player has a Drakkhaan in the asteroid belt.
		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit u = (Unit) spaceUnits.elementAt(i);

			if (!u.isEmpty())
			{
				Location loc = u.getLocation();

				if (loc.getAsteroids())
				{
					// Check for Drakkhaan
					for (int t = 0; t < u.getForces().size(); t++)
					{
						Force f = (Force) u.getForces().elementAt(t);

						if (f.getName().equals("Drakkhaan"))
						{
							drak = true;
							System.out.println("The belt is safe with a Drakkhaan!");
						}
					}
				}
			}
		}

		// Check for asteroid strikes.
		if (!drak)
		{
			for (int i = 0; i < spaceUnits.size(); i++)
			{
				Unit u = (Unit) spaceUnits.elementAt(i);

				if (!u.isEmpty())
				{
					Location loc = u.getLocation();

					if (loc.getAsteroids())
					{
						// 1/10 chance of asteroid strike.
						for (int t = 0; t < u.getForces().size(); t++)
						{
							SpaceForce sf = (SpaceForce) u.getForces().elementAt(t);

							// Determine if there is a strike.
							Random r = new Random();
							int strike = (r.nextInt(9));

							System.out.println("Checking " + sf.getName() + " against asteroids! " + strike);

							if (strike < 1)
							{
								// Force is destroyed.

								// Destroy any forces onboard.

								for (int k = 0; k < sf.getStorage().size(); k++)
								{
									Unit utemp = (Unit) sf.getStorage().elementAt(k);

									for (int z = 0; z < groundUnits.size(); z++)
									{
										Unit utemp2 = (Unit) groundUnits.elementAt(z);
										if (utemp.getName().equals(utemp2.getName()))
										{
											groundUnits.removeElementAt(z);
											z = groundUnits.size();
										}
									}

									utemp.setLoaded(false);
								}

								u.getForces().removeElementAt(t);
								t--;
								System.out.println(sf.getName() + " has been destroyed by asteroids! " + strike);
							}
						}
					}
				}
			}
		}

		// Check for asteroid belt.
		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit u = (Unit) spaceUnits.elementAt(i);

			if (!u.isEmpty())
			{
				Location loc = u.getLocation();

				if (loc.getAsteroids())
				{
					totalMinerals += 3;
					i = spaceUnits.size();

					// Ore Refining at tech 40.
					if (tech > 39)
					{
						totalMinerals += 1;//2;
					}
				}
			}
		}

		calculateMaintenance();

		money += totalEconomy - maintenance;
		if (money < 0)
		{
			money = -1;
		}
		food += totalFood;
		minerals += totalMinerals;
		gas += totalGas;

		restoreRepair();
	}

	public void setOwner(String owner)
	{
		for (int i = 0; i < planets.size(); i++)
		{
			Planet p = (Planet) planets.elementAt(i);
			p.setOwner(owner);
		}
		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit u = (Unit) spaceUnits.elementAt(i);
			u.setOwner(owner);
		}
		for (int i = 0; i < groundUnits.size(); i++)
		{
			Unit u = (Unit) groundUnits.elementAt(i);
			u.setOwner(owner);
		}
	}

	public int getFoodTurn()
	{
		int totalFood = 0;
		for (int i = 0; i < planets.size(); i++)
		{
			Planet p = (Planet) planets.elementAt(i);

			totalFood += p.getFood();

			// apply tech modifiers! - Hydroponics at 30 Tech

			if (tech > 29)
			{
				if (p.getFood() > 0)
				{
					totalFood += 1;
				}
				//totalFood += p.getFood() * 0.5;
			}
		}
		return totalFood;
	}

	public int getMineralsTurn()
	{
		int totalMinerals = 0;
		for (int i = 0; i < planets.size(); i++)
		{
			Planet p = (Planet) planets.elementAt(i);

			// apply tech modifiers!

			totalMinerals += p.getMinerals();

			// Ore Refining at tech 40.
			if (tech > 39)
			{
				if (p.getMinerals() > 0)
				{
					totalMinerals += 1;
				}
				//totalMinerals += p.getMinerals() * 0.5;
			}
		}

		// Check for the asteroid belt.
		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit u = (Unit) spaceUnits.elementAt(i);

			if (!u.isEmpty())
			{

				Location loc = u.getLocation();

				if (loc.getAsteroids())
				{
					totalMinerals += 3;
					i = spaceUnits.size();

					// Ore Refining at tech 40.
					if (tech > 39)
					{
						totalMinerals += 1;//2;
					}
				}
			}
		}
		return totalMinerals;
	}

	public int getGasTurn()
	{
		int totalGas = 0;
		for (int i = 0; i < planets.size(); i++)
		{
			Planet p = (Planet) planets.elementAt(i);

			// apply tech modifiers!

			totalGas += p.getGas();

			// Gas Refining at tech 35.
			if (tech > 34)
			{
				if (p.getGas() > 0)
				{
					totalGas += 1;
				}
				//totalGas += p.getGas() * 0.5;
			}
		}
		return totalGas;
	}

	public int getEspionage()
	{
		return espionage;
	}

	public int getCounterEspionage()
	{
		return counterEspionage;
	}

	public String getDescription()
	{
	    int i;
	    String result = race;

	    result += "\n\nTech: " + tech + "\tMoney: " + money;
	    result += "\nEspionage: " + espionage + "\tCounter-Esp: " + counterEspionage;
	    result += "\nFood: " + food + "\tMinerals: " + minerals + "\nGas: " + gas;
	    result += "\tMaintenance: " + maintenance;

	    result += "\n\nPlanets\n\n";
	    for (i = 0; i < planets.size(); i++)
	    {
	    	Planet p = (Planet) planets.elementAt(i);
		result += p + "\nEconomy: " + p.getEconomy() + "\nIndustry: " + p.getCurrentIndustry() + "/" + p.getIndustry();
		result += "\nRes/Turn: " + p.getFood() + " F, " + p.getMinerals() + " M, " + p.getGas() + " G";
		result += "\n\n";

		if (i < spaceUnits.size())
		{
			Unit u = (Unit) spaceUnits.elementAt(i);
			result += "Space: " +u.toString() +" - " + u.getLocation() + ", Loaded: " + u.getLoaded() + "\n";

			Vector forces = u.getForces();
			for (int t = 0; t < forces.size(); t++)
			{
				SpaceForce sf = (SpaceForce) forces.elementAt(t);
				result += sf.getName() + "\n";
			}
			result+="\n";

		}

		if (i < groundUnits.size())
		{
			Unit u = (Unit) groundUnits.elementAt(i);
			result += "Ground: " +u.toString() +" - " + u.getLocation() + ", Loaded: " + u.getLoaded() + "\n";

			Vector forces = u.getForces();
			for (int t = 0; t < forces.size(); t++)
			{
				GroundForce gf = (GroundForce) forces.elementAt(t);
				result += gf.getName() + "\n";
			}
			result+="\n";
		}
	    }

	    for (int z = i; z < spaceUnits.size(); z++)
	    {
		Unit u = (Unit) spaceUnits.elementAt(z);

		if (!u.isEmpty())
		{
			result += "Space: " +u.toString() +" - " + u.getLocation() + ", Loaded: " + u.getLoaded() + "\n";

			Vector forces = u.getForces();
			for (int t = 0; t < forces.size(); t++)
			{
				SpaceForce sf = (SpaceForce) forces.elementAt(t);
				result += sf.getName() + "\n";
			}
			result+="\n";
		}
	    }

	    for (int z = i; z < groundUnits.size(); z++)
	    {
		Unit u = (Unit) groundUnits.elementAt(z);

		if (!u.isEmpty())
		{
			result += "Ground: " +u.toString() +" - " + u.getLocation() + ", Loaded: " + u.getLoaded() + "\n";

			Vector forces = u.getForces();
			for (int t = 0; t < forces.size(); t++)
			{
				GroundForce gf = (GroundForce) forces.elementAt(t);
				result += gf.getName() + "\n";
			}
			result+="\n";
		}
	    }

	    return result;
	}

	public void setMaxVictoryPoints(int max)
	{
		maxVictoryPoints = max;
	}

	public int getVictoryPoints()
	{
		return victoryPoints;
	}

	public void setVictoryPoints(int victory)
	{
		victoryPoints = victory;
	}

	public int getMaxVictoryPoints()
	{
		return maxVictoryPoints;
	}

	public String getBlurb()
	{
		blurb = "";

		if (race.equals("Dortal"))
		{
			blurb += "Dortal - 6/20 Victory Points\n-------------------------------------\n\nDortal is a planet of barbarians and smugglers. They nearly had wooden ships as their space force, but the idea was squashed.\n\nDortal makes good resources and has decent counter espionage. Their economy and industry begin low. Barbarians are a cheap high damage ground unit with poor armor.";
		}
		if (race.equals("Chalchak"))
		{
			blurb += "Chalchak - 4/20 Victory Points\n-------------------------------------\n\nChalchak is a planet of smugglers and dinosaurs. A jungle world, they have high food resources.\n\nChalchak has decent resources and good espionage. Their industry begins low. Star Pteranodons are a cheap decent damage space unit with decent armor.";
		}
		if (race.equals("Zelzakia"))
		{
			blurb += "Zelzakia - 5/25 Victory Points\n-------------------------------------\n\nZelzakians are the penultimate race of the galaxy, created by all 6 of the gods. They are powerful, and will fight to (re)build their empire.\n\nZelzakia has poor resources and poor espionage/counter espionage. Their economy is decent, as is their industry. Zelzakian superwarriors are a ground unit that have decent armor, and very high damage potential.";
		}
		if (race.equals("Terroria"))
		{
			blurb += "Terroria - 4/15 Victory Points\n-------------------------------------\n\nTerroria is a city planet of crime and gang wars. A violent race, the Terrorians are however cunning and evil.\n\nTerroria's resources are the worst in the game, although they have very good espionage. Economy is good, while industry begins low. Terrorians are a cheap ground unit that have bad armor, but the ability to do high damage.";
		}
		if (race.equals("Sphere"))
		{
			blurb += "Sphere - 7/26 Victory Points\n-------------------------------------\n\nSpherans are haughty and proud, created by no god. They wish to prove once and for all that they are the dominant race (the others dont yet realize their own inferiority).\n\nSphere makes good resources and has decent espionage/counter espionage. Their economy and industry both start high. Drakkhaans have the ability to navigate the asteroid belt without destruction, giving Sphere safe mineral mining. Spheran Warlords have very good armor, high damage potential, but cost a lot of maintenance.";
		}
		if (race.equals("Electra"))
		{
			blurb += "Electra - 12/28 Victory Points\n-------------------------------------\n\nThe Electran Empire is usually at war with Sphere. They are smarmy, and consider themselves a paragon of culture.\n\nThe Electran Empire makes very good resources and has high economy. Their industry is extremely low but they have good counter espionage. Electra begins with the planets Denchul and Tortalus, the latter which allows them to buy tech at half the price. Emperor's Guard are a high-tech ground unit with decent armor and good damage.";
		}
		if (race.equals("Armoria"))
		{
			blurb += "Armoria - 6/18 Victory Points\n-------------------------------------\n\nThe Armorians are a race of good and noble warriors, created by the highest god of light, Millius. They fight against the Warchons and the Doom Priests, servants of Dagobros.\n\nArmoria starts with good resources, but has low per turn production. Their economy is good and their industry is decent. They have ok counter espionage. Armorians are a moderately priced ground unit that have good armor and the ability to do high damage.";

		}
		if (race.equals("Deksiil"))
		{
			blurb += "Deksiil - 8/24 Victory Points\n-------------------------------------\n\nDeksiil is a strong military planet, probably the one most like our Earth. The Deksiilans are peacekeepers, and work to end the wars that ravage the galaxy.\n\nDeksill makes good resources and has decent espionage/counter espionage. Their economy and industry both start high. Blue Berets are a decently cheap very high-tech ground unit with decent armor and good damage.";
		}
		if (race.equals("Loric/Lorac"))
		{
			blurb += "Loric/Lorac - 8/20 Victory Points\n-------------------------------------\n\nThe Lorician/Loracians are not altogether evil, but the order of the Doom Priests come from their race alone.\n\nLoric/Lorac has good resource production and the best beginning espionage in the game. They have good economy but a smaller industry. Doom Priests are a ground unit that have good armor and do decent damage.";
		}
		if (race.equals("Fenixa"))
		{
			blurb += "Fenixa - 4/17 Victory Points\n-------------------------------------\n\nFenixa is a desert planet ruled by biker gangs. They are hot-headed and temperamental, the only race created by fiery Tetriadus.\n\nFenixa has decent resource production and low espionage/counter espionage. Their economy is decent as is their industry. Bikers are a cheap ground unit that have low armor but decent damage.";
		}
		if (race.equals("Arsa"))
		{
			blurb += "Arsa - 4/15 Victory Points\n-------------------------------------\n\nThe Arsans are creations of Nobellius, the god of Justice. They stand for Justice and will fight to protect innocents.\n\nArsa has bad resource production but high counter espionage. Their economy is good but they are low on industry. The Arms of Justice are a very cheap ground unit with low armor and approaching decent damage. Arsa begins with the Courts which allows legal action against other players (not yet implemented).";
		}
		if (race.equals("Warchor"))
		{
			blurb += "Warchor - 7/21 Victory Points\n-------------------------------------\n\nWarchons are the last race created by Dagobros, the one he intends to have conquer the galaxy in his name. Evil and ruthless, they are good fighters and have already launched many invasions.\n\nWarchor has decent resource production and the best counter espionage in the game. Their economy is low, but their industry is high. Warchor does not pay maintenance on units that are on the planet Warchor itself. The Taios-Moross is a ground unit with decent armor and good potential damage. Warchor has access to the best fighting ships in the game at all levels of technology.";
		}

		return blurb;
	}

	public void setBlurb(String blur)
	{
		blurb = blur;
	}

	// Calculate this empire's current victory points.
	public void calculateVictoryPoints(Map map)
	{
		victoryPoints = 0;
		int totalIndustry = 0;

	    	for (int i = 0; i < planets.size(); i++)
	    	{

			// 3 VP per planet owned.
			victoryPoints += 3;

			Planet p = (Planet) planets.elementAt(i);

			totalIndustry += p.getIndustry();

		}

		// 1 VP per 10 tech
		victoryPoints += tech/10;

		// 1 VP per 10 industry
		victoryPoints += totalIndustry/10;

		// Add VP for special conditions.
		victoryPoints += specialVictoryCondition(map);

	}


	// Special victory condition, returns a number of victory points for the Empire's special condition.

	public int specialVictoryCondition(Map map)
	{
		int result = 0;

		if (race.equals("Dortal"))
		{
			// Check if there's a barbarian on every planet in the empire.

		    boolean accomplished = true;
		    for (int i = 0; i < planets.size(); i++)
		    {
			Planet plan = (Planet) planets.elementAt(i);

			Location loc = plan.getLocation();

			boolean found = false;

			for (int t = 0; t < groundUnits.size(); t++)
			{
				Unit u = (Unit) groundUnits.elementAt(t);

				if (u.getForces().size() > 0)
				{

					Location loc2 = u.getLocation();

					if (loc.getPositionX() == loc2.getPositionX() && loc.getPositionY() == loc2.getPositionY())
					{
						Vector force = u.getForces();

						for (int z = 0; z < force.size(); z++)
						{
							GroundForce gf = (GroundForce) force.elementAt(z);

							if (gf.getName().equals("Barbarians"))
							{
								found = true;
							}

						}
					}
				}

			}

			accomplished = found;
		    }

		    if (accomplished)
		    {
			result += 2;
		    }

		}

		else if (race.equals("Chalchak"))
		{
			// Dinosaur Fodder: Food production is 8 or greater.

			if (getFoodTurn() > 7)
			{
				result += 2;
			}

		}

		else if (race.equals("Zelzakia"))
		{
			// Galactic Conquerors: +1 for each planet owned beyond 3.

			int total = planets.size() - 3;

			if (total > 0)
			{
				result += total;
			}

		}

		else if (race.equals("Terroria"))
		{
			// Cash Coffers:  Over 100 money in hand.

			if (money > 100)
			{
				result += 3;
			}
		}

		else if (race.equals("Sphere"))
		{
			// Isolationists: Control the belt.

			boolean control = false;

			for (int i = 0; i < spaceUnits.size(); i++)
			{
				Unit u = (Unit) spaceUnits.elementAt(i);

				if (u.getForces().size() > 0)
				{
					Location loc = u.getLocation();

					if (loc.getAsteroids())
					{
						control = true;
					}
				}
			}

			if (control)
			{
				result += 1;
			}
		}

		else if (race.equals("Electra"))
		{
			// Imperial Homeland: Control Electra.

			for (int i = 0; i < planets.size(); i++)
			{
				Planet electra = (Planet) planets.elementAt(i);

				if (electra.getName().equals("Electra"))
				{
					result += 1;
				}
			}
		}

		else if (race.equals("Armoria"))
		{
			// Holy Crusade: Capture Lorac or Warchor.

			for (int i = 0; i < planets.size(); i++)
			{
				Planet test = (Planet) planets.elementAt(i);

				if (test.getName().equals("Lorac"))
				{
					result += 1;
				}
				else if (test.getName().equals("Warchor"))
				{
					result += 2;
				}
			}
		}

		else if (race.equals("Fenixa"))
		{
			// Axis of Oil: Gas production per turn is 10 or greater.

			if (getFoodTurn() > 9)
			{
				result += 2;
			}

		}

		else if (race.equals("Warchor"))
		{
			// Armagai'idon: Each planet owned other than Warchor.

			for (int i = 0; i < planets.size(); i++)
			{
				Planet test = (Planet) planets.elementAt(i);

				if (!test.getName().equals("Warchor"))
				{
					result += 1;
				}
			}
		}

		else if (race.equals("Loric/Lorac"))
		{
			// ShadowCultists: Each 20 points of Espionage.

			result += espionage/20;
		}

		else if (race.equals("Deksiil"))
		{
			// Peace Keepers: No other empire has more than three planets, +2VP...

			boolean accomplished = true;

			for (int i = 0; i < map.getPlayers().size(); i++)
			{
				Player p = (Player) map.getPlayers().elementAt(i);

				// Check all OTHER players...
				if (!p.getEmpire().getName().equals(name))
				{
					if (p.getEmpire().getPlanets().size() > 3)
					{
						accomplished = false;
					}
				}
			}

			if (accomplished)
			{
				result += 2;
			}

		}
		else if (race.equals("Arsa"))
		{
			// Order of the Court: +1 VP per successful court case...
		}



		return result;
	}

	// Returns this empire's special victory condition as a string.

	public String getSpecialVictoryCondition()
	{
		String result = "";

		if (race.equals("Dortal"))
		{
			result = "A Barbarian on every planet in the empire - +2VP";
		}
		else if (race.equals("Chalchak"))
		{
			result = "Food production per turn is 8 or greater - +2VP";
		}

		else if (race.equals("Zelzakia"))
		{
			result = "Each planet owned beyond 3 - +1VP/Planet";
		}

		else if (race.equals("Terroria"))
		{
			result = "Over 100 money currently in hand - +3VP";
		}

		else if (race.equals("Sphere"))
		{
			result = "Control the asteroid belt near Sphere - +1VP";
		}

		else if (race.equals("Electra"))
		{
			result = "Control the Imperial Capital of Electra - +1VP";
		}

		else if (race.equals("Armoria"))
		{
			result = "Capture Lorac (+1VP) and Warchor (+2VP)";
		}

		else if (race.equals("Fenixa"))
		{
			result = "Gas production per turn is 10 or greater - +2VP";
		}

		else if (race.equals("Warchor"))
		{
			result = "Each planet owned aside from Warchor - +1VP/Planet";
		}

		else if (race.equals("Loric/Lorac"))
		{
			result = "Each 20 points of Espionage - +1VP";
		}

		else if (race.equals("Deksiil"))
		{
			result = "No other empire owns more than 3 planets - +2VP";
		}

		else if (race.equals("Arsa"))
		{
			result = "Order of the Court - NOT YET IMPLEMENTED";
		}


		return result;
	}

	// Restores and repairs all units.
	public void restoreRepair()
	{
		for (int i = 0; i < getSpaceUnits().size(); i++)
		{
			Unit u = (Unit) getSpaceUnits().elementAt(i);

			u.repair();
			u.restoreMovement();
		}
		for (int i = 0; i < getGroundUnits().size(); i++)
		{
			Unit u = (Unit) getGroundUnits().elementAt(i);

			u.repair();
		}
	}

	// Calculate and return the max attack of all the space forces belonging to this empire.
	public int getSpaceMaxAttack()
	{
		int result = 0;

		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit unit = (Unit) spaceUnits.elementAt(i);
			result += unit.getTotalMaxAttack();
		}
		return result;
	}

	// Calculate and return the armor of all the space forces belonging to this empire.
	public int getSpaceArmor()
	{
		int result = 0;

		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit unit = (Unit) spaceUnits.elementAt(i);
			result += unit.getTotalArmor();
		}
		return result;
	}

	// Calculate and return the max attack of all the ground forces belonging to this empire.
	public int getGroundMaxAttack()
	{
		int result = 0;

		for (int i = 0; i < groundUnits.size(); i++)
		{
			Unit unit = (Unit) groundUnits.elementAt(i);
			result += unit.getTotalMaxAttack();
		}
		return result;
	}

	// Calculate and return the armor of all the ground forces belonging to this empire.
	public int getGroundArmor()
	{
		int result = 0;

		for (int i = 0; i < groundUnits.size(); i++)
		{
			Unit unit = (Unit) groundUnits.elementAt(i);
			result += unit.getTotalArmor();
		}
		return result;
	}

	// Calculate and return the max attack of all the space forces belonging to this empire at the
	// specific location in question.
	public int getSpaceMaxAttack(Location location)
	{
		int result = 0;

		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit unit = (Unit) spaceUnits.elementAt(i);

			if (!unit.isEmpty() && unit.getLocation().getPositionX() == location.getPositionX() && unit.getLocation().getPositionY() == location.getPositionY())
			{
				result += unit.getTotalMaxAttack();
			}
		}
		return result;
	}

	// Calculate and return the armor of all the space forces belonging to this empire at the specific location in question.
	public int getSpaceArmor(Location location)
	{
		int result = 0;

		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit unit = (Unit) spaceUnits.elementAt(i);
			if (!unit.isEmpty() && unit.getLocation().getPositionX() == location.getPositionX() && unit.getLocation().getPositionY() == location.getPositionY())
			{
				result += unit.getTotalArmor();
			}
		}
		return result;
	}

	// Calculate and return the max attack of all the ground forces belonging to this empire at the specific location.
	public int getGroundMaxAttack(Location location)
	{
		int result = 0;

		for (int i = 0; i < groundUnits.size(); i++)
		{
			Unit unit = (Unit) groundUnits.elementAt(i);
			if (!unit.isEmpty() && unit.getLocation().getPositionX() == location.getPositionX() && unit.getLocation().getPositionY() == location.getPositionY())
			{
				result += unit.getTotalMaxAttack();
			}
		}
		return result;
	}

	// Calculate and return the armor of all the ground forces belonging to this empire at the specific location.
	public int getGroundArmor(Location location)
	{
		int result = 0;

		for (int i = 0; i < groundUnits.size(); i++)
		{
			Unit unit = (Unit) groundUnits.elementAt(i);
			if (!unit.isEmpty() && unit.getLocation().getPositionX() == location.getPositionX() && unit.getLocation().getPositionY() == location.getPositionY())
			{
				result += unit.getTotalArmor();
			}
		}
		return result;
	}


}
