import java.util.*;
import java.io.*;

public abstract class Force implements Cloneable, Serializable, Comparable
{

	protected String name;
	protected String exclusive; // exclusive - leave blank if it is not an exclusive unit, else fill in the race.

	protected int minAttack;
	protected int maxAttack;
	protected int armor;
	protected int currentArmor;
	protected int move;
	protected int currentMove;
	protected boolean mobile;
	protected boolean committed;  // indicates whether a unit is committed in a battle to firing.

	protected int food;
	protected int gas;
	protected int minerals;

	protected int maintenance;
	protected int industry;
	protected int tech;

	public String getExclusive()
	{
		return exclusive;
	}

	public void setName(String nam)
	{
		name = nam;
	}

	public int getArmor()
	{
		return armor;
	}

	public int getCurrentArmor()
	{
		return currentArmor;
	}

	public int getMove()
	{
		return move;
	}

	public int getFood()
	{
		return food;
	}

	public int getGas()
	{
		return gas;
	}

	public int getMinerals()
	{
		return minerals;
	}

	public int getMaintenance()
	{
		return maintenance;
	}

	public int getIndustry()
	{
		return industry;
	}

	public int getTech()
	{
		return tech;
	}

	/*
  	* Determines the "type of dice" by taking the max value and dividing it by the min value
  	* Loops for the number of dice, rolling a random number for each die and adding it to the total
  	*/
 	public int doDamage(Random numbers)
	{
  		int damage = 0;

 		//numbers = new Random(System.currentTimeMillis());

		if (maxAttack > 0)
		{
 			int dice = maxAttack/minAttack;

  			for(int i = 0; i < minAttack; i++)
  			{
   				damage += numbers.nextInt(dice)+1;
   				//System.out.println("On iteration "+i+" the damage was "+damage);
  			}
		}

  		return damage;
 	}


	public abstract String getDescription();
	public abstract String toString();
	public abstract String getInfo();
	public abstract String getBattleInfo();

	public void setCurrentArmor(int arm)
	{
		currentArmor = arm;
	}

	public void restoreArmor()
	{
		currentArmor = armor;
	}

	public void setMove(int moved)
	{
		currentMove = moved;
	}

	public int getCurrentMove()
	{
		return currentMove;
	}

	public void restoreMove()
	{
		currentMove = move;
	}

	// Upgrades max movement by 1 with hyper LSD.
	public void hyperLSD()
	{
		move++;
		restoreMove();
	}

	public String getName()
	{
	    return name;
	}

	public Object clone()
	{
	    try
	    {
			Object result = super.clone();

			if (result.getClass().getName().equals("SpaceForce"))
			{
				((SpaceForce) result).setStorage(new Vector());
			}

			return result;

	    }
	    catch(Exception e)
	    {
			e.printStackTrace();
		}
	    return new Object();
	}

	public int getMinAttack()
	{
		return minAttack;
	}

	public int getMaxAttack()
	{
		return maxAttack;
	}

	public boolean getMobile()
	{
		return mobile;
	}

	public boolean getCommitted()
	{
		return committed;
	}

	public void setCommitted(boolean commit)
	{
		committed = commit;
	}

	public int compareTo(Object o)
	{
		Force f = (Force) o;

		if (f.getMove() == 1)
		{
			return -1;
		}
		else if (this.getMove() == 1)
		{
			return 1;
		}

		if (f.getMaxAttack() + f.getArmor() + (f.getMove()*5)> this.getMaxAttack() + this.getArmor() + (this.getMove() * 5))
		{
			return 1;
		}
		else if (f.getMaxAttack() + f.getArmor() + (f.getMove()*5) == this.getMaxAttack() + this.getArmor() + (this.getMove()*5))
		{
			return 0;
		}
		return -1;
	}
}
