import java.util.*;
import java.io.*;

public class InformationGatheringOperation extends Operation implements Cloneable, Serializable
{
	public InformationGatheringOperation(Player playe, Player targetPlaye, Planet targetPlane)
	{
		player = playe;
		targetPlayer = targetPlaye;
		targetPlanet = targetPlane;

		name = "Information Gathering";
		cost = 2;
		defensive = false;
		type = 1;
	}

	// Determine if operation succeeds or fails, make any changes to the map that are required.
	// Operations will always be performed sequentially, NOT CONCURRENTLY.
	// Adds messages as necessary to indicate the success of the operation.

	public void performOperation(Map currentMap, Vector messages, Vector defensiveOperations, Vector<PaxGenericAI> aiPlayers)
	{
		Random r = new Random();
		r.nextInt(100);

		// indicates whether to report operation to the target based on defensive op.
		boolean report = CheckReport(defensiveOperations);
		// check the success of the operation
		boolean success = CheckSuccess(defensiveOperations);

		if (success)
		{
			StringBuffer results = new StringBuffer("");
			Empire empire = targetPlayer.getEmpire();

			results.append("Your spies have infiltrated " + targetPlanet + ", a planet belonging to " + targetPlayer + "!\n\n");

			results.append("\n\nEmpire Name: " + empire.getName());
			results.append("\nTech Level: " + empire.getTech());
			results.append("\n\n" + targetPlanet.toString());
			results.append("\nEconomy: " + targetPlanet.getEconomy());
			results.append("\nIndustry: " + targetPlanet.getIndustry());

			Location l = targetPlanet.getLocation();

			for (int k = 0; k < empire.getSpaceUnits().size(); k++)
			{
				Unit u = (Unit) empire.getSpaceUnits().elementAt(k);

				if (u.getForces().size() > 0)
				{
					Location l2 = u.getLocation();

					if (l.getPositionX() == l2.getPositionX() && l.getPositionY() == l2.getPositionY())
					{
						results.append("\n\n" + u.toString());

						for (int z = 0; z < u.getForces().size(); z++)
						{
							Force f = (Force) u.getForces().elementAt(z);
							results.append("\n" + f.toString());
						}
					}
				}
			}
			for (int k = 0; k < empire.getGroundUnits().size(); k++)
			{
				Unit u = (Unit) empire.getGroundUnits().elementAt(k);

				if (u.getForces().size() > 0)
				{
					Location l2 = u.getLocation();

					if (l.getPositionX() == l2.getPositionX() && l.getPositionY() == l2.getPositionY())
					{
						results.append("\n\n" + u.toString());
						for (int z = 0; z < u.getForces().size(); z++)
						{
							Force f = (Force) u.getForces().elementAt(z);
							results.append("\n" + f.toString());
						}
					}
				}
			}

			Message message = new Message(results.toString(), false);
			message.addPlayer(player.getName());

			messages.addElement(message);

			// Update the AI's intelligence information.
			for (int i = 0; i < aiPlayers.size(); i++)
			{
				PaxGenericAI aiPlayer = aiPlayers.elementAt(i);

				if (aiPlayer.player.getName().equals(player.getName()))
				{
					aiPlayer.intelligenceFile.add(targetPlanet);
					aiPlayer.intelligenceFile.updateEmpire(targetPlayer.getEmpire());
					break;
				}
			}

			if (report)
			{
				StringBuffer results2 = new StringBuffer("");

				results2.append(player.getName() + "'s spies have infiltrated " + targetPlanet + " and escaped with critical information regarding the state of your empire!\n\n");

				Message message2 = new Message(results2.toString(), false);
				message2.addPlayer(targetPlayer.getName());

				messages.addElement(message2);

				// Update the AI's aggression level.
				for (int i = 0; i < aiPlayers.size(); i++)
				{
					PaxGenericAI aiPlayer = aiPlayers.elementAt(i);

					if (aiPlayer.player.getName().equals(targetPlayer.getName()))
					{
						aiPlayer.intelligenceFile.updateAggression(player.getName(), 0.4);
						break;
					}
				}
			}
		}
		else
		{
			FailureMessage(messages, report, aiPlayers);
		}
	}

}
