import java.util.*;
import java.io.*;
import JSX.*;
import java.awt.*;

public class Loader
{
    public Loader()
    {

    }
  
    // Method to load data from a given XML document containing
    // serialization data.
    // Inputs: String filename - takes the name of the XML document
    public Vector loadFromXML(String filename)
    {
	try
	{
	    // Create a vector to hold the empires
	    Vector v;
	    // Set up object to handle input stream from XML file
	    ObjIn in = new ObjIn(new FileReader(filename));
	    // load data from XML file and cast to vector
	    v = (Vector) in.readObject();	
	    //return the empires vector
	    return v;
	}
	catch(Exception e)
	{
	    Vector err = new Vector();
	    System.out.println("Error in loading XML document "+filename);
	    return err;
	}
    }

    // Method to initialize empires with proper locations for planets
    // and map with proper planets associated with locations
    // and ideally all units added to the proper locations
    public void initLocations(Vector map, Vector empires)
    {
	// load initial locations data
	Vector locationsData = (Vector) loadFromXML("initiallocations.xml");

	// parse locations data and set all invalid locations so
	for(int locationsIter = 0; locationsIter < locationsData.size(); locationsIter++)
	{
	    Initial initial = (Initial) locationsData.elementAt(locationsIter);
	    
	    Vector mapY = (Vector) map.elementAt(Integer.parseInt(initial.getY()));
	    Location mapX = (Location) mapY.elementAt(Integer.parseInt(initial.getX()));

	    mapX.setValid(initial.getValid());

   	    if(((String)initial.getName()).startsWith("Asteroid") == true)
	    {
	    	mapX.setAsteroids(true);
	    }
	}

	//

	try
	{
	    // read each empire
	    for(int empiresIter = 0; empiresIter < empires.size(); empiresIter++)
	    {
		Empire empire = (Empire) empires.elementAt(empiresIter);
		System.out.println("Loading location data for empire "+empire.toString());

		Vector planets = (Vector) empire.getPlanets();
		// look through empire's planet vector to get planets in each empire
		for(int planetsIter = 0; planetsIter < planets.size(); planetsIter++)
		{
		    Planet planet = (Planet) planets.elementAt(planetsIter);
		    System.out.println("  Loading location data for planet "+planet+" in "+empire.toString()+" empire");
		    // look through planet vector for the proper location data
		    for(int locationsDataIter = 0; locationsDataIter < locationsData.size(); locationsDataIter++)
		    {
			// ###
			// Vector locationData = (Vector) locationsData.elementAt(locationsDataIter);
			Initial locationData = (Initial) locationsData.elementAt(locationsDataIter);
			String planetDataName = (String) locationData.getName();
			if(planetDataName.equals(planet.toString()) == true)
			{
			    System.out.println("    Found data for planet "+planetDataName);
			    //	String planetDataX = (String) locationData.elementAt(1);
			    //	String planetDataY = (String) locationData.elementAt(2);

			    String planetDataX = (String) locationData.getX();
			    String planetDataY = (String) locationData.getY();
			    
			    // Now:
			    // locationData - data about location - 3 strings
			    // 	- planetDataName - name of planet
			    // 	- planetDataX	- Planet X coordinates (used to get location from map)
			    // 	- planetDataY	- Planet Y coordinates (used to get location from map)
			    // planet - planet to put at that location
			    //

			    // apparently there was a switch from scanning horizontally across
			    // to scanning vertically down.  This was initially working with mapY being
			    // parsed from planetDataY and so forth, but was changed so that the program
			    // wouldn't crash when data wasn't loaded from the old initiallocations_old.xml
			    // file, which contained a new map.  Instead, data was loaded from the new
			    // initiallocations.xml file which only contained map data for points
			    // which required a draw or invalid square.
			    Vector mapY = (Vector) map.elementAt(Integer.parseInt(planetDataY));
			    Location mapX = (Location) mapY.elementAt(Integer.parseInt(planetDataX));

			    // Now this is the insane part.
			    // Try to put the current planet into the current map location
			    try{
				mapX.addPlanet((Planet) planet);
			    }
			    catch(Exception a){System.out.println("Couldn't add planet "+planet+" to location "+mapX);}

			    // Try to put the current map location into the current planet.
			    try{
				    planet.setLocation(mapX);
			    }
			    catch(Exception a){System.out.println("Couldn't add location "+mapX+" to planet "+planet);}

			    // Ha Ha HA!  Spatial Relationships!
			    
			    // now find unit location information for the current planet
			    System.out.println("      Loading Unit data for planet "+planet);
			    // get SPACE units in the current empire
			    Vector spaceUnits = (Vector) empire.getSpaceUnits();
			    // iterate through all units in the current empire to find the one in orbit
			    // above the planet we're currently spatializing
			    for(int spaceUnitsIter = 0; spaceUnitsIter < spaceUnits.size(); spaceUnitsIter++)
			    {
				Unit spaceUnit = (Unit) spaceUnits.elementAt(spaceUnitsIter);
				String unitName = (String) spaceUnit.toString();
				if(unitName.startsWith(planetDataName) == true)
				{
				    System.out.println("        Found force "+unitName+" on "+planetDataName);
				    // add forces to location
				    mapX.addUnit(spaceUnit);
				    // add location to forces
				    spaceUnit.setLocation(mapX);
				    // Ha Ha HA!  Depth perception!
				    break;
				}
			    }
			
			    // get GROUND units in the current empire
			    Vector groundUnits = (Vector) empire.getGroundUnits();
			    // iterate through all units in the current empire to find the one in orbit
			    // above the planet we're currently spatializing
			    for(int groundUnitsIter = 0; groundUnitsIter < groundUnits.size(); groundUnitsIter++)
			    {
				Unit groundUnit = (Unit) groundUnits.elementAt(groundUnitsIter);
				String unitName = (String) groundUnit.toString();
				if(unitName.startsWith(planetDataName) == true)
				{
				    System.out.println("        Found force "+unitName+" on "+planetDataName);
				    // add forces to location
				    mapX.addUnit(groundUnit);
				    // add location to forces
				    groundUnit.setLocation(mapX);
				    // Ha Ha HA!  Depth perception!
				    break;
				}
			    }
			   // System.out.println("Looking at planet "+planet.toString());
			   // out.writeObject(planet);
			    // go to next planet.  We're done here
    			    break;
    			}
    		    }
		
		}		
	    }
	}
	catch(Exception e)
	{
	    System.out.println("Error initializing Locations");
	}
    }
    
    // Method to load empires and set their initial unit allocations.
    public Vector initEmpires(Map map)
    {
	try
	{
	    // Create vectors to hold the empires and units
	    Vector empires = loadFromXML("empires.xml");
	    Vector initialForces = loadFromXML("initialforces.xml");
	    Vector spaceSources = (Vector)loadFromXML("spaceforces.xml");
	    Vector groundSources = (Vector)loadFromXML("groundforces.xml");

	    // Load the map with the locations specified in locations.xml
	    // 
	    // Removed from use as it was deemed unnecessary, since all
	    // validity data is now located in the updated initiallocations.xml
	    // file
//	    map.setLocations((Vector)loadFromXML("locations.xml"));
	    
	    // iterate through empires
	    for(int empiresIter = 0; empiresIter < empires.size(); empiresIter++)
		{
		    Empire empire = (Empire) empires.elementAt(empiresIter);
		    System.out.println("Loading Empire: "+empire.toString());

		    // Vector for Planets
		    Vector planets = (Vector) empire.getPlanets();

		    // Iterate through all planets in the current empire
		    for(int planetsIter = 0; planetsIter < planets.size(); planetsIter++)
		    {
			// current planet to add forces to
			Planet curPlanet = (Planet)planets.elementAt(planetsIter);
			// current planet being checked
			Planet onPlanet;
			// initial planet data
			Vector initialPlanets;
			System.out.println("  Getting Forces for "+curPlanet.getName());
			
			// Create initial SPACE unit for current planet
			String spaceUnitName = curPlanet.getName() + " Space Fleet 1";
			Unit spaceUnit = new Unit(spaceUnitName, "Neutral");
			
			// Create initial GROUND unit for current planet
			String groundUnitName = curPlanet.getName() + " Ground Army 1";
			Unit groundUnit = new Unit(groundUnitName, "Neutral");

			// Iterate through all planets in initial data list 
			// (find the current planet's initial forces)
			for(int initialIter = 0; initialIter < initialForces.size(); initialIter++)
			{
			    initialPlanets = (Vector) initialForces.elementAt(initialIter);
			    String planetName = (String)initialPlanets.elementAt(0);
			    
			    // planet found in initial data.  Load units appropriately
			    if(planetName.equals(curPlanet.getName()) == true)
			    {
				// load space unit list
				Vector spaceForces = (Vector) initialPlanets.elementAt(1);
				// load ground unit list
				Vector groundForces = (Vector) initialPlanets.elementAt(2);
				// Print Necessary SPACE forces
				System.out.println("    "+spaceForces.toString());
				// clone space forces into unit
				// start by looking through all required forces
				for(int spaceForcesIter = 0; spaceForcesIter < spaceForces.size(); spaceForcesIter++)
				{
				    // get the force name
				    String spaceForce = (String)spaceForces.elementAt(spaceForcesIter);

				    // look through all source info
				    for(int spaceSourcesIter = 0; spaceSourcesIter < spaceSources.size(); spaceSourcesIter++)
				    {
					// get the source
					SpaceForce spaceSource = (SpaceForce) spaceSources.elementAt(spaceSourcesIter);
					spaceSource.setCommitted(true);

					// get the source name
					String sourceName = spaceSource.getName();

					if(sourceName.equals(spaceForce) == true)
					{
					    SpaceForce force = (SpaceForce) spaceSource.clone();
					    System.out.println("        " + force.toString());
					    System.out.println("        "+force.hashCode());

					    // finally add something to something else
					    // add force to spaceUnit (of the current planet)
					    // 
					    spaceUnit.addForce(force);
					    // stop looking.  We found the source data
					    break;
					}
				    }
				}
				
				// Print necessary GROUND forces
				System.out.println("    "+groundForces.toString());
				// clone GROUND forces into unit
				// start by looking through all required forces
				for(int groundForcesIter = 0; groundForcesIter < groundForces.size(); groundForcesIter++)
				{
				    // get the force name
				    String groundForce = (String)groundForces.elementAt(groundForcesIter);

				    // look through all source info
				    for(int groundSourcesIter = 0; groundSourcesIter < groundSources.size(); groundSourcesIter++)
				    {
					// get the source
					GroundForce groundSource = (GroundForce) groundSources.elementAt(groundSourcesIter);
					groundSource.setCommitted(true);

					// get the source name
					String sourceName = groundSource.getName();

					if(sourceName.equals(groundForce) == true)
					{
					    GroundForce force = (GroundForce) groundSource.clone();
					    System.out.println("        " + force.toString());

					    // Add force to groundUnit (of the current planet)
					    groundUnit.addForce(force);
					    // stop looking.  We found the source data for this ground units
					    break;
					}
				    }
				}
				// stop looking.  we're done with this planet
				break;
			    }
			}
			
			// actually add the units to the current empire
			empire.addSpaceUnit(spaceUnit);
			empire.addGroundUnit(groundUnit);			
		    }
		}

	    // spatially locates everything
	    System.out.println("\nLoading Spatial Relationships\n");
	    initLocations(map.getLocations(), empires);

	    // return the loaded empires vector
	    return empires;
	}
	catch(Exception e)
	{
	    Vector err = new Vector();
	    System.out.println("Error in loading Empires Vector");
	    return err;
	}
    }
}
