import java.util.*;
import java.io.*;
import java.lang.Math.*;

public class Map implements Serializable
{

	private Vector players;
	private Vector locations;

	private int turn;
	private int phase;

	private Vector<Location> asteroidAdjacent;
	private Vector<Location> asteroidLocations;

	public Map(Vector player)
	{
    	players = player;

		turn = 1;
		phase = 1;

		locations = new Vector();

		for (int cols = 0; cols < 11; cols++)
		{
			Vector temp = new Vector();

			for (int rows = 0; rows < 9; rows++)
			{
				Location l = new Location(rows, cols, true);
				temp.addElement(l);
			}

			locations.addElement(temp);
		}
	}

	// Returns the closest asteroid location to the one passed in to the function.
	public Location findClosestAsteroidLocation(Location currentLocation)
	{
		Location result = null;
		int minDistance = 10000;

		for (int i = 0; i < getAsteroidLocations().size(); i++)
		{
			Location loc = asteroidLocations.elementAt(i);

			int distance = distance(loc, currentLocation);

			if (distance < minDistance)
			{
				minDistance = distance;
				result = loc;
			}
		}
		return result;
	}

	public Vector<Location> getAsteroidLocations()
	{
		if (asteroidLocations == null)
		{
			findAsteroidLocations();
		}
		return asteroidLocations;
	}

	public void findAsteroidLocations()
	{
		asteroidLocations = new Vector<Location>();
		for (int i = 0; i < locations.size(); i++)
		{
			Vector nextVec = (Vector) locations.elementAt(i);

			for (int t = 0; t < nextVec.size(); t++)
			{
				Location loc = (Location) nextVec.elementAt(t);

				if (loc.getAsteroids() && loc.getValid())
				{
					asteroidLocations.addElement(loc);
					//System.out.println("Asteroid location: " + loc);
				}
			}
		}
	}

	public Vector<Location> getAsteroidAdjacent()
	{
		if (asteroidAdjacent == null)
		{
			findAsteroidAdjacent();
		}
		return asteroidAdjacent;
	}

	public void findAsteroidAdjacent()
	{
		asteroidAdjacent = new Vector<Location>();
		for (int i = 0; i < locations.size(); i++)
		{
			Vector nextVec = (Vector) locations.elementAt(i);

			for (int t = 0; t < nextVec.size(); t++)
			{
				Location loc = (Location) nextVec.elementAt(t);

				if (!loc.getAsteroids() && loc.getValid())
				{
					// Check if any of the adjacent locations have asteroids.
					boolean asteroids = checkAsteroids(i-1,t);

					if (!asteroids)
					{
						asteroids = checkAsteroids(i+1,t);
					}
					if (!asteroids)
					{
						asteroids = checkAsteroids(i,t-1);
					}
					if (!asteroids)
					{
						asteroids = checkAsteroids(i,t+1);
					}

					if (asteroids)
					{
						asteroidAdjacent.addElement(loc);
						//System.out.println("Asteroid Adjacent: " + loc);
					}
				}
			}
		}
	}

	public boolean checkAsteroids(int y, int x)
	{
		boolean asteroids = false;

		Location loc = getLocationAt(y, x);

		if (loc != null)
		{
			asteroids = loc.getAsteroids();
		}

		return asteroids;
	}

	public Location getLocationAt(int y, int x)
	{
		Location result = null;

		if (y < locations.size() && y >= 0)
		{
			Vector col = (Vector) locations.elementAt(y);
			if (x >= 0 &&  x < col.size())
			{
				result = (Location) col.elementAt(x);
			}
		}
		return result;
	}

	public int getTurn()
	{
		return turn;
	}

	public int getPhase()
	{
		return phase;
	}

	public Vector getPlayers()
	{
		return players;
	}

	public void setTurn(int tu)
	{
		turn = tu;
	}

	public void setPhase(int phas)
	{
		phase = phas;
	}

	public void setLocations(Vector l)
	{
	    locations = l;
	}

	public Vector getLocations()
	{
	    return locations;
	}

	public int distance(Location l, Location l2)
	{
		int result = 0;

		if (l2 != null)
		{
			int resultX = l.getPositionX() - l2.getPositionX();
			int resultY = l.getPositionY() - l2.getPositionY();

			result = Math.abs(resultX) + Math.abs(resultY);
			int addition = 0;

			// Take into account locations that are not valid.
			if ((resultX == 0 || resultY == 0) && result > 0)
			{
				int deltaX = resultX/(result);
				int deltaY = resultY/(result);

				for (int i = 0; i < result; i++)
				{
					Location currentLocation = getLocationAt(l2.getPositionY()+i*deltaY, l2.getPositionX()+i*deltaX);

					if (currentLocation != null && !currentLocation.getValid())
					{
						addition += 2;
					}
				}
			}

			result += addition;

		}

		return result;
	}

	public void clean()
	{
		for (int i = 0; i < 11; i++)
		{
			for (int k =0; k < 9; k++)
			{
				Location l = getLocationAt(i, k);
				Vector units = l.getUnits();

				for (int c = 0; c < units.size(); c++)
				{
					if (units.elementAt(c) == null)
					{
						units.removeElementAt(c);
					}
					else
					{
						if (((Unit)units.elementAt(c)).isEmpty())
						{
							units.removeElementAt(c);
						}
					}

				}
			}
		}
	}
}
