import javax.swing.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Timer;
import java.util.TimerTask;


public class MapCanvas extends JPanel
{
	Color c;
	Image starBackground;
	float timer; // controls how the flashing of the active unit.
	static int clickCount = 0; // used to distinguish double clicks from single.
	private Timer clickTimer; // Times the clicks.

	int horizontalIncrement;
	int verticalIncrement;

	int chosenX;
	int chosenY;

	int move;

	Map map;
	Player current;
	Unit currentUnit;
	Unit currentGround;
	int currentIndex;

	Vector V;

	// A vector of battles to be drawn on the map.
	Vector battles;

	// A vector of locations adjacent to the asteroid belt.
	Vector asteroidList;

	// Vector of location labels to be updated.
	Vector spaceLocations;
	Vector groundLocations;

	// Vectors of JLists for unit forces.
	Vector spaceLists;
	Vector groundLists;
	JList activeList;

	// A label for the movement panel.
	JLabel moveLabel;
	// A combo box for space units.
	JComboBox unitsBox;

	public static void main(String[] args) throws Exception
	{
		JFrame test = new JFrame();

	}



	public MapCanvas(Player curren, Map m, Vector spaceLocation, Vector groundLocation, Vector sLists, Vector gLists, JComboBox unitsBo, JList activeLis)
	{

	    	current = curren;
	    	map = m;
		moveLabel = null;
		unitsBox = unitsBo;
		clickCount = 0;

		currentGround = null;

		spaceLocations = spaceLocation;
		groundLocations = groundLocation;//new Vector();
		spaceLists = sLists;
		groundLists = gLists;
		activeList = activeLis;

	    	horizontalIncrement = 90;
		verticalIncrement = 60;

		chosenX = -10;
		chosenY = -10;

		battles = new Vector();
		asteroidList = new Vector();

		setPreferredSize(new Dimension(810, 660));

		starBackground = Toolkit.getDefaultToolkit().getImage("backgroundflare2.0.jpg");
		while (!prepareImage(starBackground,null));

		/* Old background settings
		 * c = new Color(121);
		 * setBackground(Color c);*/

		addMouseListener(new MouseListener(){
 	 		public void mousePressed(MouseEvent e)
			{

			}
		    	public void mouseReleased(MouseEvent e) {}
			public void mouseEntered(MouseEvent e) {}
		    	public void mouseExited(MouseEvent e) {}
    		    	public void mouseClicked(MouseEvent e)
			{
				chosenX = e.getX()/horizontalIncrement;
				chosenY = e.getY()/verticalIncrement;

				if (move > 0 && e.getClickCount() > 1)
				{
					clickCount = 0;

					if (clickTimer != null)
					{
						clickTimer.cancel();
					}


					Location l = currentUnit.getLocation();
					Location l2 = map.getLocationAt(chosenY, chosenX);

					if (l2.getValid())
					{
						// Must move adjacent if not on the belt.
						if (!l.getAsteroids())
						{
							if (map.distance(l, l2) == 1)
							{
								l.removeUnit(currentUnit);
								currentUnit.setLocation(l2);
								l2.addUnit(currentUnit);
								currentUnit.setMinMove(1);
								move--;

								refreshLists();
								setLocationLabels();

								activeList.repaint();

								// Moving on to the asteroid belt.
								if (l2.getAsteroids())
								{
									currentUnit.setMinMove(1);
									move--;
									if (move < 0)
									{
										move = 0;
									}
								}

								if (moveLabel != null)
								{
									moveLabel.setText(" Fleet Movement: " + currentUnit.getMinMove()+"");
								}
							}
						}
						// Can only move adjacent to the belt.
						else
						{
							for (int i = 0; i < asteroidList.size(); i++)
							{
								Location loc = (Location) asteroidList.elementAt(i);

								if (l2.getPositionX() == loc.getPositionX() && l2.getPositionY() == loc.getPositionY())
								{
									l.removeUnit(currentUnit);
									currentUnit.setLocation(l2);
									l2.addUnit(currentUnit);
									currentUnit.setMinMove(1);
									move--;
									i = asteroidList.size();

									refreshLists();
									setLocationLabels();

									if (moveLabel != null)
									{
										moveLabel.setText(" Fleet Movement: " + currentUnit.getMinMove()+"");
									}
								}
							}

						}
					}

				}

				// Select the next unit at that location.
				else if (unitsBox != null  && (move < 1 || e.getClickCount() < 2))
				{
					clickCount++;


					clickTimer = new Timer();
					clickTimer.schedule(new singleClick(unitsBox, currentUnit, current, clickCount, chosenX, chosenY),250);


				}

	//			System.out.println("ChosenX: " + chosenX + " ChosenY: " + chosenY);
				repaint();

			}
		});

		Loader L = new Loader();
	    	V = L.loadFromXML("initiallocations.xml");

		setUpAsteroids();

	}

	// Adds a battle to be drawn on this MapCanvas.
	public void addBattle(Battle b)
	{
		battles.addElement(b);
	}

	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		g.drawImage(starBackground, 0, 0, null);
		Color sunColor = new Color(121);
		//Color asteroidColor = new Color(white);

   		g.setColor(c.WHITE);

		int i = 0;
		int square = 0;
		while (i < 810)
		{
			if (square == 5)
			{
				// Don't draw lines through the suns.

				g.drawLine(i, verticalIncrement*3, i, (660-verticalIncrement*2));

			}
			else
			{
				g.drawLine(i, 0, i, 660);
			}
		    	i = i + horizontalIncrement;
			square++;
		}

		i = 0;
		square = 0;
		while (i < 660)
		{
			if (square == 1 || square == 2)
			{
				g.drawLine(horizontalIncrement, i, horizontalIncrement*4, i);
				g.drawLine(horizontalIncrement*6, i, 810, i);
			}
			else if (square == 10)
			{
				g.drawLine(0, i, horizontalIncrement*4, i);
				g.drawLine(horizontalIncrement*6, i, 810, i);
			}
			else
			{
				g.drawLine(0, i, 810, i);
			}
			i = i + verticalIncrement;
			square++;
		}

		for (int t = 0; t < battles.size(); t++)
		{
			Battle b = (Battle) battles.elementAt(t);
			drawBattle(g, b.getLocation().getPositionX(), b.getLocation().getPositionY());
		}

		// For purposes of this game, the asteroid belt must remain as a single unit.
		// We use this counter to determine which line dividers must be eliminated.
		int asteroids = 0;
		int suns = 0;

		// loop through entire initiallocations file and get all location info
		// to print to the map
		for(int locationsIter = 0; locationsIter < V.size(); locationsIter++)
		{
		    Initial initial = (Initial) V.elementAt(locationsIter);
		    String planetName = (String) initial.getName();
		    g.setColor(initial.getPlanetColor());

		    if(planetName.startsWith("Asteroid") == true)
		    {
			g.setColor(c.lightGray);
			// draw the asteroids name
			g.drawString("Asteroids", horizontalIncrement * Integer.parseInt(initial.getX()) + 20, verticalIncrement * Integer.parseInt(initial.getY()) +20);

			// draw the asteroids
			/*
                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) + 15, verticalIncrement * Integer.parseInt(initial.getY()) + 12, 5, 5);
			//this asteroid covers up the word
                        //g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) + 45, verticalIncrement * Integer.parseInt(initial.getY()) + 12, 5, 5);
                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) + 75, verticalIncrement * Integer.parseInt(initial.getY()) + 12, 5, 5);

			g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) +30, verticalIncrement * Integer.parseInt(initial.getY()) + 24, 5, 5);
                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) +60, verticalIncrement * Integer.parseInt(initial.getY()) + 24, 5, 5);

                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) +15, verticalIncrement * Integer.parseInt(initial.getY()) + 36, 5, 5);
                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) +45, verticalIncrement * Integer.parseInt(initial.getY()) + 36, 5, 5);
                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) +75, verticalIncrement * Integer.parseInt(initial.getY()) + 36, 5, 5);

                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) +30, verticalIncrement * Integer.parseInt(initial.getY()) + 48, 5, 5);
                        g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) +60, verticalIncrement * Integer.parseInt(initial.getY()) + 48, 5, 5);
			*/

			// Black out the specific lines on the map to make the belt one square.
			if (asteroids == 0)
			{
				g.setColor(c.BLACK);
				// Top Horizontal
				g.drawLine(Integer.parseInt(initial.getX())*horizontalIncrement+1, Integer.parseInt(initial.getY())*verticalIncrement, (Integer.parseInt(initial.getX())+1)*horizontalIncrement-1, Integer.parseInt(initial.getY())*verticalIncrement);
				// Right Vertical
				g.drawLine((Integer.parseInt(initial.getX())+1)*horizontalIncrement, Integer.parseInt(initial.getY())*verticalIncrement+1, (Integer.parseInt(initial.getX())+1)*horizontalIncrement, (Integer.parseInt(initial.getY())+1)*verticalIncrement-1);
		    	}
			else if (asteroids == 1)
			{
				g.setColor(c.BLACK);
				// Right Vertical
				g.drawLine((Integer.parseInt(initial.getX())+1)*horizontalIncrement, Integer.parseInt(initial.getY())*verticalIncrement+1, (Integer.parseInt(initial.getX())+1)*horizontalIncrement, (Integer.parseInt(initial.getY())+1)*verticalIncrement-1);
			}
			else if (asteroids == 2)
			{
				g.setColor(c.BLACK);
				// Right Vertical
				g.drawLine((Integer.parseInt(initial.getX())+1)*horizontalIncrement, Integer.parseInt(initial.getY())*verticalIncrement+1, (Integer.parseInt(initial.getX())+1)*horizontalIncrement, (Integer.parseInt(initial.getY())+1)*verticalIncrement-1);
			}
			else if (asteroids == 3)
			{
				g.setColor(c.BLACK);
				// Bottom Horizontal
				g.drawLine(Integer.parseInt(initial.getX())*horizontalIncrement+1, (Integer.parseInt(initial.getY())+1)*verticalIncrement, (Integer.parseInt(initial.getX())+1)*horizontalIncrement-1, (Integer.parseInt(initial.getY())+1)*verticalIncrement);
			}

			asteroids++;

		    }
		    else if(planetName.startsWith("Sun") == true)
		    {
			sunColor = g.getColor();
			// draw the sun name
			// I think this next line makes every sun orange
		     	//g.fillRect(horizontalIncrement * Integer.parseInt(initial.getX())+1, verticalIncrement * Integer.parseInt(initial.getY())+1, horizontalIncrement-1, verticalIncrement-1); // 2, 2, 87, 57

			// Only draw Sun labels on certain squares.
			if (suns == 1 || suns == 3 || suns == 4 || suns == 10   || suns == 7 ) // 5
			{
				g.setColor(initial.getTextColor());
				g.drawString("Sun", horizontalIncrement * Integer.parseInt(initial.getX()) + 20, verticalIncrement * Integer.parseInt(initial.getY()) + 20);
			}
			suns++;
		    }
		    else
		    {
			// draw the planet
			//g.fillOval(horizontalIncrement * Integer.parseInt(initial.getX()) + 30, verticalIncrement * Integer.parseInt(initial.getY()) + 22, 30, 30);

			// draw the planet name
			g.setColor(initial.getTextColor());
			g.drawString(planetName, horizontalIncrement * Integer.parseInt(initial.getX()) + 20, verticalIncrement * Integer.parseInt(initial.getY()) + 20);
		    }
		}

		// Draw lines between sun squares.
		g.setColor(sunColor);

		// Horizontal Lines
		// Position: 1, 0 (y, x)
		//g.drawLine(1, verticalIncrement, horizontalIncrement-1, verticalIncrement);
		// Position: 2, 0
		//g.drawLine(1, verticalIncrement*2, horizontalIncrement-1, verticalIncrement*2);
		// Position: 3, 0
		g.drawLine(1, verticalIncrement*3, horizontalIncrement-1, verticalIncrement*3);
		// Position: 1, 4
		//g.drawLine(1+horizontalIncrement*4, verticalIncrement, horizontalIncrement*5-1, verticalIncrement);
		// Position: 1, 5
		//g.drawLine(1+horizontalIncrement*5, verticalIncrement, horizontalIncrement*6-1, verticalIncrement);
		// Position: 2, 4
		//g.drawLine(1+horizontalIncrement*4, verticalIncrement*2, horizontalIncrement*5-1, verticalIncrement*2);
		// Position: 2, 5
		//g.drawLine(1+horizontalIncrement*5, verticalIncrement*2, horizontalIncrement*6-1, verticalIncrement*2);
		// Position: 3, 4
		g.drawLine(1+horizontalIncrement*4, verticalIncrement*3, horizontalIncrement*5-1, verticalIncrement*3);
		// Position: 3. 5
		g.drawLine(1+horizontalIncrement*5, verticalIncrement*3, horizontalIncrement*6-1, verticalIncrement*3);
		// Position: 6, 2
		g.drawLine(1+horizontalIncrement*2, verticalIncrement*6, horizontalIncrement*3-1, verticalIncrement*6);
		// Position: 7, 2
		g.drawLine(1+horizontalIncrement*2, verticalIncrement*7, horizontalIncrement*3-1, verticalIncrement*7);
		// Position: 9, 4
		g.drawLine(1+horizontalIncrement*4, verticalIncrement*9, horizontalIncrement*5-1, verticalIncrement*9);
		// Position: 9, 5
		g.drawLine(1+horizontalIncrement*5, verticalIncrement*9, horizontalIncrement*6-1, verticalIncrement*9);

		// Position: 10, 4
		//g.drawLine(1+horizontalIncrement*4, verticalIncrement*10, horizontalIncrement*5-1, verticalIncrement*10);
		// Position: 10, 5
		//g.drawLine(1+horizontalIncrement*5, verticalIncrement*10, horizontalIncrement*6-1, verticalIncrement*10);

		// Position: 10, 8
		g.drawLine(1+horizontalIncrement*8, verticalIncrement*10, horizontalIncrement*9-1, verticalIncrement*10);


		// Vertical Lines
		// Position: 0, 1
		g.drawLine(horizontalIncrement*1, 1, horizontalIncrement*1, verticalIncrement);
		// Position: 0, 4
		g.drawLine(horizontalIncrement*4, 1, horizontalIncrement*4, verticalIncrement);
		// Position: 0, 5
		//g.drawLine(horizontalIncrement*5, 1, horizontalIncrement*5, verticalIncrement);
		// Position: 0, 6
		g.drawLine(horizontalIncrement*6, 1, horizontalIncrement*6, verticalIncrement);
		// Position: 1, 1
		g.drawLine(horizontalIncrement*1, verticalIncrement+1, horizontalIncrement*1, verticalIncrement*2);
		// Position: 1, 5
		//g.drawLine(horizontalIncrement*5, verticalIncrement+1, horizontalIncrement*5, verticalIncrement*2);
		// Position: 1, 4
		g.drawLine(horizontalIncrement*4, verticalIncrement+1, horizontalIncrement*4, verticalIncrement*2);
		// Position: 1, 6
		g.drawLine(horizontalIncrement*6, verticalIncrement+1, horizontalIncrement*6, verticalIncrement*2);
		// Position: 2, 1
		g.drawLine(horizontalIncrement*1, verticalIncrement*2+1, horizontalIncrement*1, verticalIncrement*3);
		// Position: 2, 4
		g.drawLine(horizontalIncrement*4, verticalIncrement*2+1, horizontalIncrement*4, verticalIncrement*3);
		// Position: 2, 5
		//g.drawLine(horizontalIncrement*5, verticalIncrement*2+1, horizontalIncrement*5, verticalIncrement*3);
		// Position: 2, 6
		g.drawLine(horizontalIncrement*6, verticalIncrement*2+1, horizontalIncrement*6, verticalIncrement*3);

		// Position: 6, 2
		g.drawLine(horizontalIncrement*2, verticalIncrement*6+1, horizontalIncrement*2, verticalIncrement*7);
		// Position: 6, 3
		g.drawLine(horizontalIncrement*3, verticalIncrement*6+1, horizontalIncrement*3, verticalIncrement*7);


		// Position: 9, 4
		g.drawLine(horizontalIncrement*4, verticalIncrement*9+1, horizontalIncrement*4, verticalIncrement*10);
		// Position: 10, 4
		g.drawLine(horizontalIncrement*4, verticalIncrement*10+1, horizontalIncrement*4, verticalIncrement*11);
		// Position: 9, 5
		//g.drawLine(horizontalIncrement*5, verticalIncrement*9+1, horizontalIncrement*5, verticalIncrement*10);
		// Position: 10, 5
		//g.drawLine(horizontalIncrement*5, verticalIncrement*10+1, horizontalIncrement*5, verticalIncrement*11);
		// Position: 9, 6
		g.drawLine(horizontalIncrement*6, verticalIncrement*9+1, horizontalIncrement*6, verticalIncrement*10);
		// Position: 10, 6
		g.drawLine(horizontalIncrement*6, verticalIncrement*10+1, horizontalIncrement*6, verticalIncrement*11);

		// Position: 10, 8
		g.drawLine(horizontalIncrement*8, verticalIncrement*10+1, horizontalIncrement*8, verticalIncrement*11);



		// Draw red outline around invalid selected square.

		if (chosenX > -1)
		{
			g.setColor(c.RED);
			g.drawLine(horizontalIncrement*chosenX, verticalIncrement*chosenY, horizontalIncrement*(chosenX+1), verticalIncrement*chosenY);
			g.drawLine(horizontalIncrement*chosenX, verticalIncrement*(chosenY+1), horizontalIncrement*(chosenX+1), verticalIncrement*(chosenY+1));
			g.drawLine(horizontalIncrement*chosenX, verticalIncrement*chosenY, horizontalIncrement*chosenX, verticalIncrement*(chosenY+1));
			g.drawLine(horizontalIncrement*(chosenX+1), verticalIncrement*chosenY, horizontalIncrement*(chosenX+1), verticalIncrement*(chosenY+1));
		}


		// Colors for various players.
		Vector colors = new Vector();
		colors.addElement(c.BLUE);
		colors.addElement(c.RED);
		colors.addElement(c.GREEN.darker());
		colors.addElement(c.GRAY);
		colors.addElement(c.MAGENTA);
		colors.addElement(c.ORANGE);
		colors.addElement(c.PINK);
		colors.addElement(c.CYAN.darker());
		colors.addElement(c.RED.darker());
		colors.addElement(c.WHITE.darker());
		colors.addElement(c.GREEN.brighter());
		colors.addElement(c.MAGENTA.darker());


		Vector loc = map.getLocations();

		for (int k = 0; k < loc.size(); k++)
		{
			Vector loc2 = (Vector) loc.elementAt(k);

			for (int z = 0; z < loc2.size(); z++)
			{
				Location l = (Location) loc2.elementAt(z);

				l.setGroundUnitCount(0);
				l.setSpaceUnitCount(0);

			}

		}

		Vector play = map.getPlayers();

		for (int k = 0; k < play.size(); k++)
		{
			Player player = (Player) play.elementAt(k);

			Vector spaces = (Vector) player.getEmpire().getSpaceUnits();

			for (int t = 0; t < spaces.size(); t++)
			{
				Unit u = (Unit) spaces.elementAt(t);
				if (!u.isEmpty())
				{
					Location l = u.getLocation();

					Vector loc2 = (Vector) loc.elementAt(l.getPositionY());

					Location l2 = (Location) loc2.elementAt(l.getPositionX());

					u.setLocation(l2);

					l2.setGroundUnitCount(0);
					l2.setSpaceUnitCount(0);
				}
			}
		}

		for (int k = 0; k < play.size(); k++)
		{
			Player player = (Player) play.elementAt(k);

			Vector grounds = (Vector) player.getEmpire().getGroundUnits();

			for (int t = 0; t < grounds.size(); t++)
			{
				Unit u = (Unit) grounds.elementAt(t);
				if (!u.isEmpty())
				{
					Location l = u.getLocation();

					Vector loc2 = (Vector) loc.elementAt(l.getPositionY());

					Location l2 = (Location) loc2.elementAt(l.getPositionX());

					u.setLocation(l2);

					l.setGroundUnitCount(0);
					l.setSpaceUnitCount(0);
				}
			}
		}


		for (int k = 0; k < play.size(); k++)
		{
			Player player = (Player) play.elementAt(k);

			float diffR = ((Color) colors.elementAt(k)).getRed() - c.YELLOW.brighter().getRed();
			float diffG = ((Color) colors.elementAt(k)).getGreen() - c.YELLOW.brighter().getGreen();
			float diffB = ((Color) colors.elementAt(k)).getBlue() - c.YELLOW.brighter().getBlue();

			int red = ((Color)colors.elementAt(k)).getRed() - (int)(diffR * timer);
			int green = ((Color)colors.elementAt(k)).getGreen() - (int)(diffG * timer);
			int blue = ((Color)colors.elementAt(k)).getBlue() - (int)(diffB * timer);

			//System.out.println(timer + " " + red + ", " + green + ", " + blue);

			Color activeColor = new Color(red, green, blue);

			Vector grounds = (Vector) player.getEmpire().getGroundUnits();

			for (int t = 0; t < grounds.size(); t++)
			{
				Unit u = (Unit) grounds.elementAt(t);
				if (!u.isEmpty())
				{
					Location l = u.getLocation();
					if (!(u.getLoaded() && !u.getOwner().equals(current.toString())))
					{


					g.setColor((Color)colors.elementAt(k));

					boolean activeDraw = false;

					if (u.getOwner().equals(current.toString()) && currentGround != null && u.getName().equals(currentGround.getName()))
					{
						activeDraw = true;
					}

					if (l.getGroundUnitCount() < 1 || activeDraw)
					{

						if (!u.getLoaded())
						{
							g.setColor(c.YELLOW.darker());
						}
						else
						{
							g.setColor(c.MAGENTA.brighter());
						}
						g.fillRect(horizontalIncrement*l.getPositionX()+38, verticalIncrement*l.getPositionY()+30, 14, 14);

						if (activeDraw)
						{
							g.setColor(activeColor);
						}
						else
						{
							g.setColor((Color)colors.elementAt(k));
						}

						g.fillRect(horizontalIncrement*l.getPositionX()+40, verticalIncrement*l.getPositionY()+32, 10, 10);
					}

					g.setColor((Color)colors.elementAt(k));

					g.setFont(new Font("Serif", Font.ITALIC, 10));
					if (u.getOwner().equals(current.toString()))
					{
						g.drawString(u.toString(), horizontalIncrement*l.getPositionX()+54, verticalIncrement*l.getPositionY()+30 + l.getGroundUnitCount()*10);
						l.setGroundUnitCount(l.getGroundUnitCount()+1);
					}

					}
				}

				/*	else
				{
					g.drawString(player.toString(), horizontalIncrement*l.getPositionX()+54, verticalIncrement*l.getPositionY()+30);
				}*/
			}

			Vector spaces = (Vector) player.getEmpire().getSpaceUnits();

			for (int t = 0; t < spaces.size(); t++)
			{
				Unit u = (Unit) spaces.elementAt(t);
				if (!u.isEmpty())
				{
					Location l = u.getLocation();
					g.setColor((Color)colors.elementAt(k));

					boolean activeDraw = false;

					if (u.getOwner().equals(current.toString()) && currentUnit != null && currentUnit.getName().equals(u.getName()))
					{
						//g.setColor(c.YELLOW.brighter());

						g.setColor(activeColor);

						activeDraw = true;
					}

					if (l.getSpaceUnitCount() < 1 || activeDraw)
					{
						g.fillRoundRect(horizontalIncrement*l.getPositionX()+4+3-(k), verticalIncrement*l.getPositionY()+20+2-(k), 14, 7, 30, 10);//20, 10, 30, 10);
					}

					g.setFont(new Font("Serif", 0, 10));

					if (u.getOwner().equals(current.toString()))
					{
						if (currentUnit != null && currentUnit.getName().equals(u.getName()))
						{
							g.setColor(activeColor);//c.YELLOW.brighter());
						}

						g.drawString(u.toString(), horizontalIncrement*l.getPositionX()+1, verticalIncrement*l.getPositionY()+38 + l.getSpaceUnitCount()*10);

						l.setSpaceUnitCount(l.getSpaceUnitCount()+1);
					}
				}
			/*	else
				{
   					g.drawString(player.toString(), horizontalIncrement*l.getPositionX(), verticalIncrement*l.getPositionY()+38);
 				}*/
			}


			// Draw the planet flags.

			for (int t = 0; t < player.getEmpire().getPlanets().size(); t++)
			{
				Planet plan = (Planet) player.getEmpire().getPlanets().elementAt(t);

				Location l = plan.getLocation();

				g.setColor((Color)colors.elementAt(k));

				g.drawLine(horizontalIncrement*l.getPositionX()+70, verticalIncrement*l.getPositionY()+40, horizontalIncrement*l.getPositionX()+70, verticalIncrement*l.getPositionY()+55);
				g.fillRect(horizontalIncrement*l.getPositionX()+70, verticalIncrement*l.getPositionY()+40, 10, 5);
			}
		}


		if (currentUnit != null && !currentUnit.isEmpty())
		{
			Location l = currentUnit.getLocation();

			g.setColor(c.YELLOW.brighter());
			g.drawLine(horizontalIncrement*l.getPositionX(), verticalIncrement*l.getPositionY(), horizontalIncrement*(l.getPositionX()+1), verticalIncrement*l.getPositionY());
			g.drawLine(horizontalIncrement*l.getPositionX(), verticalIncrement*(l.getPositionY()+1), horizontalIncrement*(l.getPositionX()+1), verticalIncrement*(l.getPositionY()+1));
			g.drawLine(horizontalIncrement*l.getPositionX(), verticalIncrement*l.getPositionY(), horizontalIncrement*l.getPositionX(), verticalIncrement*(l.getPositionY()+1));
			g.drawLine(horizontalIncrement*(l.getPositionX()+1), verticalIncrement*l.getPositionY(), horizontalIncrement*(l.getPositionX()+1), verticalIncrement*(l.getPositionY()+1));

			// Moving Highlights

			if (move > 0)
			{
			    	Location l2 = map.getLocationAt(l.getPositionY(), l.getPositionX());

				// Regular Adjacency when not in the Asteroid Belt
				if (!l2.getAsteroids())
				{

				if (l.getPositionY()+1 < 11)
				{
					l2 = map.getLocationAt(l.getPositionY()+1, l.getPositionX());
					if ((l2.getValid()))
					{
						g.setColor(c.GREEN);
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY());
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
					}
				}
				if (l.getPositionY()-1 > -1)
				{
					l2 = map.getLocationAt(l.getPositionY()-1, l.getPositionX());
					if (l2.getValid())
					{
						g.setColor(c.GREEN);
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY());
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
					}
				}
				if (l.getPositionX()+1 < 9)
				{
					l2 = map.getLocationAt(l.getPositionY(), l.getPositionX()+1);
					if (l2.getValid())
					{
						g.setColor(c.GREEN);
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY());
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
					}
				}
				if (l.getPositionX()-1 > -1)
				{
					l2 = map.getLocationAt(l.getPositionY(), l.getPositionX()-1);
					if (l2.getValid())
					{
						g.setColor(c.GREEN);
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY());
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
					}
				}
				}

				// We draw a different adjacency set when in the asteroid belt.
				else if (l2.getAsteroids())
				{
					for (int t = 0; t < asteroidList.size(); t++)
					{
						l2 = (Location) asteroidList.elementAt(t);

						g.setColor(c.GREEN);
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY());
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*l2.getPositionX(), verticalIncrement*l2.getPositionY(), horizontalIncrement*l2.getPositionX(), verticalIncrement*(l2.getPositionY()+1));
						g.drawLine(horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*l2.getPositionY(), horizontalIncrement*(l2.getPositionX()+1), verticalIncrement*(l2.getPositionY()+1));
					}

				}
			}
		}




	//	drawUnits(g);
	}


	public void drawUnits(Graphics g)
	{
	    Location l;
	    int offsetX, offsetY;
	    Vector allUnits;
	    Map m = map;
/*
    	Unit u = new Unit("Test", "Neutral");
	SpaceForce sf = new SpaceForce("Test Force 1", "", 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, true);
	u.addForce(sf);
	Location loc = (Location)m.getLocationAt(0, 1);
	loc.addUnit(u);
*/

	    // get map sizes (Could hard-code this, but why sacrifice modularity?)
	    Vector temp = (Vector) m.getLocations();
	    int mapYSize = temp.size();
	    temp = (Vector)temp.firstElement();
	    int mapXSize = temp.size();

	    // iterate through all locations in the map and draw the units there.
	    for(int mapYIter = 0; mapYIter < mapYSize; mapYIter++)
	    {
		for(int mapXIter = 0; mapXIter < mapXSize; mapXIter++)
		{
		    l = (Location) m.getLocationAt(mapYIter, mapXIter);
		    offsetX = horizontalIncrement*mapXIter;
		    offsetY = verticalIncrement*mapYIter;
		    int spaceIter = 0;
		    int groundIter = 0;
		    // Calculate Space Battles
		    // Start by getting all units.
		    allUnits = (Vector) l.getUnits();
		    for(int unitsIter = 0; unitsIter < allUnits.size(); unitsIter++)
		    {
			g.setFont(new Font("Serif", 0, 10));
			g.setColor(c.RED);
			Unit unit = (Unit) allUnits.elementAt(unitsIter);
			Vector forces = (Vector) unit.getForces();
			// see if we can cast it to a space unit
			try{
			    getPlayerColour(g, m, unit.getOwner());
			    SpaceForce spaceForce = (SpaceForce) forces.firstElement();
			    spaceIter++;
			    drawSpaceUnit(g, offsetX, offsetY, spaceIter, unit.toString());
			}
			catch(Exception e){}

			try{
			    GroundForce groundForce = (GroundForce) forces.firstElement();
			    groundIter++;
			    drawGroundUnit(g, offsetX, offsetY, groundIter, unit.toString());
			}
			catch(Exception e){}

		    }
		}
	    }
	}


	// function to iterate through all players and get the proper player colour
	public void getPlayerColour(Graphics g, Map m, String name)
	{
	    Vector players = (Vector) m.getPlayers();
	    for(int playerIter = 0; playerIter < players.size(); playerIter++)
	    {
		Player p = (Player)players.elementAt(playerIter);
		System.out.println(name);
		if(name.equals(p.getName()) == true)
		{
		    System.out.println(name);
		    break;
		}
	    }
	}

	// function to draw a ground unit on the screen
	public void drawGroundUnit(Graphics g, int offsetX, int offsetY, int num, String name)
	{
	    g.fillRect(offsetX+38, offsetY+30, 14, 14);
	    g.setColor(c.WHITE);
	    g.drawString(name, offsetX+54, offsetY+(5*num)+30);

	}

	// function to draw a space unit on the screen
	public void drawSpaceUnit(Graphics g, int offsetX, int offsetY, int num, String name)
	{
	    g.fillRoundRect(offsetX+2, offsetY+(10*num)+10, 20, 10, 30, 10);
	    g.setColor(c.WHITE);
	    g.drawString(name, offsetX+2, offsetY+(10*num)+17);
	}

	// draws a battle.  Only need to pass in g and the map square (increment * count)
	public void drawBattle(Graphics g, int offsetX, int offsetY)
	{
	    offsetX *= horizontalIncrement;
	    offsetY *= verticalIncrement;
	    offsetX += horizontalIncrement/7 + 1;
	    offsetY += verticalIncrement/7;
	    g.setColor(c.ORANGE.darker());

 	    int[] pointsX = {offsetX+0, offsetX+16, offsetX+32, offsetX+48, offsetX+64,
	       			offsetX+56, offsetX+64,
				offsetX+48, offsetX+32, offsetX+16,
				offsetX+0, offsetX+8, offsetX+0};
	    int[] pointsY = {offsetY+0, offsetY+8, offsetY+0, offsetY+8, offsetY+0,
				offsetY+24, offsetY+48,
				offsetY+40, offsetY+48, offsetY+40,
				offsetY+48, offsetY+24, offsetY+0};
	    g.fillPolygon(pointsX, pointsY, 13);
	    //g.setColor(c.RED);
	    //g.drawString("Battle", offsetX+5, offsetY+15);
	}

	// Put all the locations that are adjacent to the belt in a vector so that we can later
	// make use of them.
	public void setUpAsteroids()
	{
		// Loop through all locations in the map and find those adjacent to the belt.

		Vector asteroidSpaces = new Vector();
		asteroidList = new Vector();

		for (int y = 0; y < 11; y++)
		{
			for (int x = 0; x < 9; x++)
			{

				Location loc = map.getLocationAt(y, x);

				// Add to the vector.
				if (loc.getAsteroids())
				{
					asteroidSpaces.addElement(loc);
				}
			}
		}

		// Loop through asteroid squares and add nearby ones to the adjacency asteroidList.

	       for (int i = 0; i < asteroidSpaces.size(); i++)
	       {

		       Location l = (Location) asteroidSpaces.elementAt(i);

		       int x = l.getPositionX();
		       int y = l.getPositionY();

		       // Get adjacent locations that are within bounds.
		       if ((x-1) > 0)
		       {
				Location adjacent = map.getLocationAt(y, x-1);

				// Only add if it's not asteroids or invalid.

				if (!adjacent.getAsteroids() && adjacent.getValid())
				{
					asteroidList.addElement(adjacent);
				}
		       }

	     	       if ((x+1) < 9)
		       {
				Location adjacent = map.getLocationAt(y, x+1);

				// Only add if it's not asteroids or invalid.

				if (!adjacent.getAsteroids() && adjacent.getValid())
				{
					asteroidList.addElement(adjacent);
				}
		       }
	       	       if ((y-1) > 0)
		       {
				Location adjacent = map.getLocationAt(y-1, x);

				// Only add if it's not asteroids or invalid.

				if (!adjacent.getAsteroids() && adjacent.getValid())
				{
					asteroidList.addElement(adjacent);
				}
		       }
		       if ((y+1) < 11)
		       {
				Location adjacent = map.getLocationAt(y+1, x);

				// Only add if it's not asteroids or invalid.

				if (!adjacent.getAsteroids() && adjacent.getValid())
				{
					asteroidList.addElement(adjacent);
				}
		       }
	       }
	}

	public void setCurrentUnit(Unit u, int currentInde)
	{
		currentUnit = u;
		currentIndex = currentInde;
	}

	public void setMove(int mov)
	{
		move = mov;
	}

	public void setMap(Map ma)
	{
		map = ma;
	}

	public void setSpaceLocations(Vector spaceLocation)
	{
		spaceLocations = spaceLocation;
	}

	public void setGroundLocations(Vector groundLocation)
	{
		groundLocations = groundLocation;
	}

	public void setCurrentGround(Unit currentGroun)
	{
		currentGround = currentGroun;
	}

	public void setLocationLabels()
	{
		for (int i = 0; i < current.getEmpire().getSpaceUnits().size(); i++)
		{
			Unit u = (Unit) current.getEmpire().getSpaceUnits().elementAt(i);

			if (!u.isEmpty())
			{
				// Modify location label here.

				String locationText = "Location (x/y): ";

				Location loc = u.getLocation();
				locationText += loc.getPositionX() + "/" + loc.getPositionY();

				if (loc.getPlanets().size() > 0)
				{
					Planet plan = (Planet) loc.getPlanets().elementAt(0);
					locationText += " ("+plan.getName()+")";
				}

				JLabel spaceLocation = (JLabel) spaceLocations.elementAt(i);
				spaceLocation.setText(locationText);
			}
		}


		for (int i = 0; i < current.getEmpire().getGroundUnits().size(); i++)
		{
			Unit u = (Unit) current.getEmpire().getGroundUnits().elementAt(i);

			if (!u.isEmpty())
			{
				// Modify location label here.

				String locationText = "Location (x/y): ";

				Location loc = u.getLocation();
				locationText += loc.getPositionX() + "/" + loc.getPositionY();

				if (loc.getPlanets().size() > 0)
				{
					Planet plan = (Planet) loc.getPlanets().elementAt(0);
					locationText += " ("+plan.getName()+")";
				}

				if (u.getLoaded())
				{
					locationText += " - SPACE";
				}

				JLabel groundLocation = (JLabel) groundLocations.elementAt(i);
				groundLocation.setText(locationText);
			}
		}
	}


	public void setMoveLabel(JLabel move)
	{
		moveLabel = move;
	}

	public void refreshLists()
	{
		for (int i = 0; i < spaceLists.size(); i++)
		{
			JList list = (JList) spaceLists.elementAt(i);
			list.repaint();
		}
		for (int i = 0; i < groundLists.size(); i++)
		{
			JList list = (JList) groundLists.elementAt(i);
			list.repaint();
		}

	}

	public void setTimer(float time)
	{
		timer = time;
	}


	// Sets the active unit to be one from this square, if possible.
	// Takes the next down the list.
	public void selectNextUnit(int locationX, int locationY)
	{
		Vector spaces = (Vector) current.getEmpire().getSpaceUnits();

		int firstIndex = -1;

		boolean activeLocation = false;

		if (currentUnit.getLocation().getPositionX() == locationX && currentUnit.getLocation().getPositionY() == locationY)
		{
			activeLocation = true;
		}

		for (int t = 0; t < spaces.size(); t++)
		{
			Unit u = (Unit) spaces.elementAt(t);

			if (!u.isEmpty())
			{
				Location l = u.getLocation();

				// Check that the location matches.

				if (l.getPositionX() == locationX && l.getPositionY() == locationY)
				{

					if (!activeLocation)
					{
						unitsBox.setSelectedIndex(t);
						return;
					}
					else
					{
						if (u.getName().equals(currentUnit.getName()))
						{
							activeLocation = false;
						}
						else if (firstIndex < 0)
						{
							firstIndex = t;
						}
					}

				}

			}
		}

		if (firstIndex >= 0)
		{
			unitsBox.setSelectedIndex(firstIndex);
		}
	}

	public void setUnitsBox(JComboBox unitsBo)
	{
		unitsBox = unitsBo;
	}

	public void singleClick(int chosenX, int chosenY)
	{
		if (clickCount > 0)
		{
			System.out.println("Single click!");
			clickCount = 0;
			selectNextUnit(chosenX, chosenY);
		}
	}

}

class singleClick extends TimerTask{
/* (non-Javadoc)
* @see java.util.TimerTask#run()
*/

JComboBox unitsBox;
Unit currentUnit;
Player current;
int clickCount;
int chosenX;
int chosenY;

public singleClick(JComboBox unitBo, Unit currentUni, Player curren, int clickCoun, int choseX, int choseY)
{
	unitsBox = unitBo;
	currentUnit = currentUni;
	current = curren;

	clickCount = clickCoun;
	chosenX = choseX;
	chosenY = choseY;
}

public void run() {
if (clickCount > 0)
{
	clickCount = 0;
	selectNextUnit(unitsBox, currentUnit, current, chosenX, chosenY);
}
}
//this.cancel();

	// Sets the active unit to be one from this square, if possible.
	// Takes the next down the list.
	public void selectNextUnit(JComboBox unitsBox, Unit currentUnit, Player current, int locationX, int locationY)
	{
		Vector spaces = (Vector) current.getEmpire().getSpaceUnits();

		int firstIndex = -1;

		boolean activeLocation = false;

		if (!currentUnit.isEmpty() && currentUnit.getLocation().getPositionX() == locationX && currentUnit.getLocation().getPositionY() == locationY)
		{
			activeLocation = true;
		}

		for (int t = 0; t < spaces.size(); t++)
		{
			Unit u = (Unit) spaces.elementAt(t);

			if (!u.isEmpty())
			{
				Location l = u.getLocation();

				// Check that the location matches.

				if (l.getPositionX() == locationX && l.getPositionY() == locationY)
				{

					if (!activeLocation)
					{
						unitsBox.setSelectedIndex(t);
						return;
					}
					else
					{
						if (u.getName().equals(currentUnit.getName()))
						{
							activeLocation = false;
						}
						else if (firstIndex < 0)
						{
							firstIndex = t;
						}
					}

				}

			}
		}

		if (firstIndex >= 0)
		{
			unitsBox.setSelectedIndex(firstIndex);
		}
	}
}




