import java.util.*;
import java.io.*;

public class Offer implements Serializable
{
	// Represents an offer during the market phase.  
	// 'One' is the name of the player who has put forth the offer.
	// 'Two' starts initially as a blank space, but will later be the person who accepts the offer.


    	private String one; // the name of the person who made the offer.
	private String two; // the name of the person accepting the offer.
	private String id; // a unique ID for the offer.

	static int identity = 0; // used to generate the id.

	// The offer being sent by player 'One'.
	private int oFood;  
	private int oMinerals;
	private int oGas;
	private int oMoney;

	// The offer being requested by player 'One'.
	private int dFood;
	private int dMinerals;
	private int dGas;
	private int dMoney;

	// When a second person has agreed to the deal, it is considered to be 'full', and is waiting for
	// confirmation from the sender.
	private boolean full;

	// When the offer is confirmed, resources are transfered to other players, though they have already
	// been spent to make the offer.
	private boolean confirmed;

	public Offer(String on, int oFoo, int oMineral, int oGa, int oMone, int dFoo, int dMineral, int dGa, int dMone)
	{
		one = on;
		two = " ";

		identity++;
		id = one + identity;

		oFood = oFoo;
		oMinerals = oMineral;
		oGas = oGa;
		oMoney = oMone;

		dFood = dFoo;
		dMinerals = dMineral;
		dGas = dGa;
		dMoney = dMone;
	
		full = false;
		confirmed = false;
	}

	// An offer has been accepted or confirmed.
	
	public boolean accept(String name)
	{
		if (full == true && name.equals(one))
		{
			confirmed = true;
		}
		else if (!full)
		{
			two = name;
			full = true;
		}

		return confirmed;
	}

	// An offer has been rejected by the current offerer, and two is booted from the deal.
	
	public boolean reject(String name)
	{
		boolean authority = false;

		// Checking if they have authority to reject.
		if (one.equals(name))
		{
			two = " ";
			full = false;
			confirmed = false;
			authority = true;
		}

		return authority;
	}

	// Returns this offer in String form to be displayed in the market.

	public String toString()
	{
		String result = one;
	       
		if (!confirmed)
		{
			result += " offers ";// + id + " offers to pay ";
		}
		else
		{
			result += "- ";	
		}

		if (oFood > 0)
		{
			result += oFood + "F";
		}
		if (oGas > 0)
		{
			result += oGas + "G";
		}
		if (oMinerals > 0)
		{
			result += oMinerals + "M";
		}
		if (oMoney > 0)
		{
			result += oMoney + "$";
		}

		result += " for ";

		if (dFood > 0)
		{
			result += dFood + "F";
		}
		if (dGas > 0)
		{
			result += dGas + "G";
		}
		if (dMinerals > 0)
		{
			result += dMinerals + "M";
		}
		if (dMoney > 0)
		{
			result += dMoney + "$";
		}

		if (full)
		{
			result += " from " + two + " ";
		}

		if (confirmed)
		{
			result += "-DONE!";
		}

		while (result.length() < 60)
		{
			result += " ";
		}

		return result;

	}



	// Returns this offer in String form to be displayed in the market.

	public String getDescription()
	{
		String result = "<html><I>"+one+"</I>";
	       
		if (!confirmed)
		{
			result += " will give ";// + id + " offers to pay ";
		}
		else
		{
			result += " gave ";	
		}

		if (oFood > 0)
		{
			result += "<FONT SIZE=+1 COLOR=blue>" + oFood + "F</FONT>";
		}
		if (oGas > 0)
		{
			result += "<FONT SIZE=+1 COLOR=red>" +oGas + "G</FONT>";
		}
		if (oMinerals > 0)
		{
			result += "<FONT SIZE=+1 COLOR=silver>" + oMinerals + "M</FONT>";
		}
		if (oMoney > 0)
		{
			result += "<FONT SIZE=+1 COLOR=green>" +oMoney + "$</FONT>";
		}

		if (!confirmed)
		{
			result += " if <b>you</b> give ";
		}
		else
		{
			result += " to <I>"+two+"</I> for ";
		}

		if (dFood > 0)
		{
			result += "<FONT SIZE=+1 COLOR=blue>" + dFood + "F</FONT>";
		}
		if (dGas > 0)
		{
			result += "<FONT SIZE=+1 COLOR=red>" + dGas + "G</FONT>";
		}
		if (dMinerals > 0)
		{
			result += "<FONT SIZE=+1 COLOR=silver>" + dMinerals + "M</FONT>";
		}
		if (dMoney > 0)
		{
			result += "<FONT SIZE=+1 COLOR=green>" + dMoney + "$</FONT>";
		}

		if (confirmed)
		{
			result += "-DONE!";
		}
		else
		{
			if (full)
			{
				result += " with <I>" + two + "</I> ";
			}
		}

		while (result.length() < 60)
		{
			result += " ";
		}

		result += "</html>";

		return result;

	}


	public String getOne()
	{
		return one;
	}

	public String getTwo()
	{
		return two;
	}

	public String getId()
	{
		return id;
	}

	public boolean getFull()
	{
		return full;
	}

	public boolean getConfirmed()
	{
		return confirmed;
	}

	public int getOFood()
	{
		return oFood;
	}

	public int getOGas()
	{
		return oGas;
	}

	public int getOMinerals()
	{
		return oMinerals;
	}

	public int getOMoney()
	{
		return oMoney;
	}

	public int getDFood()
	{
		return dFood;
	}

	public int getDGas()
	{
		return dGas;
	}

	public int getDMinerals()
	{
		return dMinerals;
	}

	public int getDMoney()
	{
		return dMoney;
	}

}

