import java.util.*;
import java.io.*;

public abstract class Operation implements Cloneable, Serializable
{

	protected Player player; // player performing the operation.
	protected Player targetPlayer; // target player for the operation.
	protected Planet targetPlanet; // target planet for the operation.

	protected String name; // the name of the mission.
	protected int cost; // the espionage cost of the operation.
	protected boolean defensive; // indicates whether this is a defensive operation.
	protected int type; // indicates the type of operation (-2 = c-esp, -1 = reporting, 1 = Info Gather, 2 = Eco, 3 = ind, 4 = tech)

	public abstract void performOperation(Map currentMap, Vector messages, Vector defensiveOperations, Vector<PaxGenericAI> aiPlayers);

	// Returns important information as a single string.

	public String toString()
	{
		String result = name + ", Cost: " + cost;

		if (targetPlayer != null)
		{
			result += ", against " + targetPlayer;
		}
		if (targetPlanet != null)
		{
			result += " on planet " + targetPlanet;
		}

		return result;
	}


	public boolean CheckReport(Vector defensiveOperations)
	{
		boolean report = false;

		for (int t = 0; t < defensiveOperations.size(); t++)
		{
			Operation op = (Operation) defensiveOperations.elementAt(t);

			// check if it's a reporting operation
			if (op.getType() == -1)
			{

				// check if this player is being targetted.
				if (targetPlayer.getName().equals(op.getPlayer().getName()) && op.getTargetPlayer().getName().equals(player.getName()))
				{
					report = true;
				}
			}
		}

		return report;
	}

	public boolean CheckSuccess(Vector defensiveOperations)
	{
		Random r = new Random();
		boolean success = true;

		// Compare against base counter-esp.
		double chance = (r.nextInt(100))/100.0;
		double check = targetPlayer.getEmpire().getCounterEspionage() * 0.05;

		//System.out.println("Chance: " + chance + " against Counter: " + check);

		if (chance < check)
		{
			success = false;
		}

		for (int t = 0; t < defensiveOperations.size(); t++)
		{
			Operation op = (Operation) defensiveOperations.elementAt(t);

			// check if it's a counter esp operation
			if (op.getType() == -2)
			{

				// check if this player is being targetted.
				if (targetPlayer.getName().equals(op.getPlayer().getName()))
				{
					// 40% chance of failure for the target player, 20% for every other player.

					chance = (r.nextInt(100))/100.0;

					//System.out.println("Facing Security: " + chance);

					if (op.getTargetPlayer().getName().equals(player.getName()) && chance < 0.4)
					{
						success = false;
					}
					else if (!op.getTargetPlayer().getName().equals(player.getName()) && chance < 0.2)
					{
						success = false;
					}
				}
			}
		}

		return success;
	}

	public void FailureMessage(Vector messages, boolean report, Vector<PaxGenericAI> aiPlayers)
	{
		StringBuffer results = new StringBuffer("");

		results.append(player.getName() + "'s spies have attempted and failed " + name + " on " + targetPlanet + ", one of " + targetPlayer + "'s planets!\n\n");

		Message message = new Message(results.toString(), false);
		message.addPlayer(player.getName());

		// Only send failure messages to targets if they have a report running.
		if (report)
		{
			message.addPlayer(targetPlayer.getName());

			// Update the AI's aggression level.
			for (int i = 0; i < aiPlayers.size(); i++)
			{
				PaxGenericAI aiPlayer = aiPlayers.elementAt(i);

				if (aiPlayer.player.getName().equals(targetPlayer.getName()))
				{
					aiPlayer.intelligenceFile.updateAggression(player.getName(), 0.4);
					break;
				}
			}
		}

		messages.addElement(message);
	}

	public Object clone()
	{
	    try
	    {
		return super.clone();
	    }
	    catch(Exception e){}
	    return new Object();
	}

	public void setName(String n)
	{
		name = n;
	}

	public String getName()
	{
		return name;
	}

	public void setCost(int c)
	{
		cost = c;
	}

	public int getCost()
	{
		return cost;
	}

	public void setPlayer(Player p)
	{
		player = p;
	}

	public Player getPlayer()
	{
		return player;
	}

	public void setTargetPlayer(Player p)
	{
		targetPlayer = p;
	}

	public Player getTargetPlayer()
	{
		return targetPlayer;
	}

	public void setTargetPlanet(Planet p)
	{
		targetPlanet = p;
	}

	public Planet getTargetPlanet()
	{
		return targetPlanet;
	}

	public void setDefensive(boolean d)
	{
		defensive = d;
	}

	public boolean getDefensive()
	{
		return defensive;
	}

	public void setType(int t)
	{
		type = t;
	}

	public int getType()
	{
		return type;
	}


}
