import java.io.*;
import java.util.*;

//The unchecked warnings come from any method which takes any Vector
@SuppressWarnings("unchecked")


public class PaxGenericAI implements Serializable
{
	public Player player;

	Vector totalSpaceForces;
	Vector totalGroundForces;

	// The number of turns that the AI has played.
	public int movementCount;
	public int improvementCount;

	// The total number of threads that can be used.
	int threadCount;

	// The beam reduction used for the production tree.
	int beamReduction;

	// Ratios used for calculating the 'best' move - can be changed by subclasses.
	float spaceGroundRatio = 1.1f;
	float groundTransportRatio = 2.0f;
	float mobileImmobileRatio = 2.5f;
	float movementWeight = 7.0f;
	float foodWeight = 14.0f;
	float gasWeight = 14.0f;
	float mineralWeight = 14.0f;

	float currentSpaceGroundRatio;
	float currentGroundTransportRatio;
	float currentMobileImmobileRatio;
	float currentMovementWeight;
	float currentFoodWeight;
	float currentGasWeight;
	float currentMineralWeight;

	float economyWeight = 4.2f;
	float industryWeight = 5.0f;
	float techWeight = 5.0f;
	float espionageWeight = 4.2f;
	float moneyWeight = 0.5f;

	float currentEconomyWeight;
	float currentIndustryWeight;
	float currentTechWeight;
	float currentEspionageWeight;
	float currentMoneyWeight;


	// A value between 0.0 and 1.0f indicating the random factor incorporated into the AI's decisions.
	float productionRandomFactor = 0.3f;//0.4f;
	float improvementRandomFactor = 0.3f; //0.4f;
	float totalRandomFactor = 0.03f;

	Random random;

	Map map;
	Vector neutralEmpires;
	private Vector<Player> lastTurnPlayers;

	Vector<AIUnit> aiUnits;

	public AIIntelligenceFile intelligenceFile;

	static final int DEFENSIVE = 0;
	static final int NEUTRAL = 1;
	static final int OFFENSIVE = 2;
	int aiMood = NEUTRAL;

	Location targetLocation;

	public boolean lostPlanet;

	private boolean hyperLSD;
	private boolean orderTransport;
	private boolean orderGround;
	private boolean orderSpace;

	float attackDefenseRatio = 0.6f;
	float offensiveProbability = 0.3f;
	float defensiveProbability = 0.75f;

	float currentAttackDefenseRatio = 0.6f;
	float currentOffensiveProbability = 0.3f;
	float currentDefensiveProbability = 0.75f;


	public PaxGenericAI(Player play, Vector spaceForces, Vector groundForces, int threads, Map ma, Vector empires)
	{
		player = play;
		totalSpaceForces = spaceForces;
		totalGroundForces = groundForces;
		threadCount = threads;

		movementCount = 0;
		improvementCount = 0;
		beamReduction = 0;

		map = ma;
		neutralEmpires = empires;

		aiUnits = new Vector<AIUnit>();
		lastTurnPlayers = new Vector<Player>();

		random = new Random(System.currentTimeMillis());

		hyperLSD = false;
	}

	public void setup()
	{
		// Build intelligence file.
		Vector<Empire> allEmpires = new Vector<Empire>();

		for (int i = 0; i < neutralEmpires.size(); i++)
		{
			allEmpires.addElement((Empire)neutralEmpires.elementAt(i));
		}

		lastTurnPlayers = new Vector<Player>();

		for (int i = 0; i < map.getPlayers().size(); i++)
		{
			Player p = (Player) map.getPlayers().elementAt(i);

			allEmpires.addElement(p.getEmpire());

			lastTurnPlayers.addElement(p.clone());
		}

		intelligenceFile = new AIIntelligenceFile(player.getName(), allEmpires, map);

		//System.out.println(intelligenceFile);
	}

	// A function that draws randomly from among the possible espionage choices.
	public Vector runEspionage(PrintStream ps)
	{
		Vector operations = new Vector();

		ps.println("");
		ps.println("------------------------------");
		ps.println(player.getName() + " - Espionage " + (movementCount+1));
		ps.println("");

		Vector<AIOperation> operationList = buildOperationList(ps);

		int espionage = player.getEmpire().getEspionage();

		while (operationList.size() > 0)
		{
			Operation op = operationList.elementAt(0).operation;
			if (espionage >= op.getCost())
			{
				operations.addElement(op);
				espionage -= op.getCost();

				//System.out.println(op);
				ps.println(op);
			}
			operationList.removeElementAt(0);
		}
		return operations;
	}

	// Craft a list of all possible operations, score and sort them, and return them.
	public Vector<AIOperation> buildOperationList(PrintStream ps)
	{
		Vector<AIOperation> operationList = new Vector<AIOperation>();

		// Build the targetList of human-controlled enemy planets.
		Vector<TargetPlanet> targets = buildTargetList(ps);
		for (int i =0; i < targets.size(); i++)
		{
			if (targets.elementAt(i).target.getOwner().equals("Neutral"))
			{
				targets.removeElementAt(i);
				i--;
			}
		}

		// Build list of operations we would only ever want to do once - Info gathering, counter int operation.
		for (int i = 0; i < targets.size(); i++)
		{
			InformationGatheringOperation op = new InformationGatheringOperation(player, getPlayer(targets.elementAt(i).target.getOwner()), targets.elementAt(i).target);
			AIOperation aiOp = scoreOperation(op, targets.elementAt(i), ps);
			aiOp.priority -= op.getCost() * 2;
			operationList.addElement(aiOp);
		}

		for (int i = 0; i < map.getPlayers().size(); i++)
		{
			Player play = (Player) map.getPlayers().elementAt(i);

			if (!play.getName().equals(player.getName()) && play.getEmpire().getPlanets().size() > 0)
			{
				CounterIntelligenceOperation op = new CounterIntelligenceOperation(player, play, (Planet)player.getEmpire().getPlanets().elementAt(0));
				AIOperation aiOp = scoreOperation(op, null, ps);
				aiOp.priority -= op.getCost() * 2;
				operationList.addElement(aiOp);
			}
		}

		// Build list of repeatable operations, with increasingly depleted weights.
		for (int i = 0; i < map.getPlayers().size(); i++)
		{
			Player play = (Player) map.getPlayers().elementAt(i);

			if (!play.getName().equals(player.getName()) && play.getEmpire().getPlanets().size() > 0)
			{
				for (int t = 0; t < 10000; t++)
				{
					SecurityOperation op = new SecurityOperation(player, play, (Planet)player.getEmpire().getPlanets().elementAt(0));
					AIOperation aiOp = scoreOperation(op, null, ps);
					aiOp.priority = aiOp.priority + (t * (5 + random.nextInt(10)));
					aiOp.priority -= op.getCost() * 2;
					operationList.addElement(aiOp);

					if (t >= player.getEmpire().getEspionage()/op.getCost())
					{
						break;
					}
				}

				for (int t = 0; t < 10000; t++)
				{
					EconomicOperation op = new EconomicOperation(player, play, (Planet)play.getEmpire().getPlanets().elementAt(0));
					AIOperation aiOp = scoreOperation(op, null, ps);
					aiOp.priority = aiOp.priority + (t * (5 + random.nextInt(10)));
					aiOp.priority -= op.getCost() * 2;
					operationList.addElement(aiOp);

					if (t >= player.getEmpire().getEspionage()/op.getCost())
					{
						break;
					}
				}
				for (int t = 0; t < 10000; t++)
				{
					TechnologyOperation op = new TechnologyOperation(player, play, (Planet)play.getEmpire().getPlanets().elementAt(0));
					AIOperation aiOp = scoreOperation(op, null, ps);
					aiOp.priority = aiOp.priority + (t * (5 + random.nextInt(10)));
					aiOp.priority -= op.getCost() * 2;
					operationList.addElement(aiOp);

					if (t >= player.getEmpire().getEspionage()/op.getCost())
					{
						break;
					}
				}
			}
		}

		for (int i = 0; i < targets.size(); i++)
		{
			for (int t = 0; t < 10000; t++)
			{
				IndustryOperation op = new IndustryOperation(player, getPlayer(targets.elementAt(i).target.getOwner()), targets.elementAt(i).target);
				AIOperation aiOp = scoreOperation(op, targets.elementAt(i), ps);
				aiOp.priority = aiOp.priority + (t * (5 + random.nextInt(10)));
				aiOp.priority -= op.getCost() * 2;
				operationList.addElement(aiOp);

				if (t >= player.getEmpire().getEspionage()/op.getCost())
				{
					break;
				}
			}
		}

		Collections.sort(operationList);

		//System.out.println(operationList);
		ps.println(operationList);

		return operationList;
	}

	// Scores a single espionage operation, which may take place against targetPlanet.
	public AIOperation scoreOperation(Operation op, TargetPlanet targetPlanet, PrintStream ps)
	{
		int priority = random.nextInt(40);

		// Get target's aggression, and VP.
		double aggression = 0;
		double vp = 0;

		if (op.getTargetPlayer().getEmpire().getPlanets().size() > 0)
		{
			aggression = intelligenceFile.get(((Planet)op.getTargetPlayer().getEmpire().getPlanets().elementAt(0)).getName()).aggression;
		}

		vp = intelligenceFile.getVictoryPoints(op.getTargetPlayer(), map);

		// CounterIntelligenceOperation
		if (op.getType() == -1)
		{
			priority -= aggression * 10;

		}
		// SecurityOperation
		else if (op.getType() == -2)
		{
			priority -= aggression * 10;
		}
		// InformationGatheringOperation
		else if (op.getType() == 1)
		{
			priority += targetPlanet.priority/10.0;

			priority -= aggression * 10;

			AIPlanetIntelligence intel = intelligenceFile.get(targetPlanet.target.getName());

			priority += Math.max(-40, (intel.reliability - map.getTurn()) * 10);
		}
		// EconomicOperation
		else if (op.getType() == 2)
		{
			priority -= vp * 40;

			priority -= aggression * 5;

			priority -= 5 * currentMoneyWeight;

		}
		// TechnologyOperation
		else if (op.getType() == 4)
		{
			int targetTech = intelligenceFile.getTech(op.getTargetPlayer());
			int techMod = Math.min(5, targetTech - player.getEmpire().getTech());

			if (techMod < 0)
			{
				techMod = 0;
			}

			priority -= techMod * currentTechWeight;

			if (techMod == 0)
			{
				priority += 20;
			}
		}
		// IndustryOperation
		else if (op.getType() == 5)
		{
			priority += targetPlanet.priority/10.0;

			priority -= aggression * 10;

			priority -= vp * 40;

			if (targetLocation != null && targetLocation.getPositionX() == targetPlanet.target.getLocation().getPositionX() && targetLocation.getPositionY() == targetPlanet.target.getLocation().getPositionY())
			{
				priority -= 30;
			}
		}

		return new AIOperation(op, priority);
	}

	public Player getPlayer(String playerName)
	{
		for (int i = 0; i < map.getPlayers().size(); i++)
		{
			Player play = (Player) map.getPlayers().elementAt(i);

			if (play.getName().equals(playerName))
			{
				return play;
			}
		}
		return null;
	}

	public void runMovement(Vector<Player> otherPlayers, PrintStream ps) throws Exception
	{
		ps.println("");
		ps.println("------------------------------");
		ps.println(player.getName() + " - Movement " + (movementCount+1));
		ps.println("");

		setRandomWeightings(ps);

		cleanUnits();
		orderTransport = false;
		orderGround = false;
		orderSpace = false;

		checkMood(otherPlayers, ps);

		for (int i = 0; i < aiUnits.size(); i++)
		{
			aiUnits.elementAt(i).move(map, otherPlayers);
		}

		checkMovement(ps);

		movementCount++;

		//System.out.println(player.getEmpire().getDescription());
		ps.println(player.getEmpire().getDescription());

		//System.out.println(player + " has finished moving!");
		ps.println(player + " has finished moving!");
	}

	public void runImprovementProduction(PrintStream ps) throws Exception
	{
		//System.out.println(player.getEmpire().getDescription());

		intelligenceFile.turnRefresh(map);

		ps.println("");
		ps.println("------------------------------");
		ps.println(player.getName() + " - Improvement/Production " + (improvementCount+1));
		ps.println("");


		// Check if we own the planet that we are targetting.
		if (targetLocation == null || (targetLocation != null && ((Planet)targetLocation.getPlanets().elementAt(0)).getOwner().equals(player.getName())))
		{
			if (targetLocation != null)
			{
				aiMood = NEUTRAL;
			}

			Vector<TargetPlanet> targets = buildTargetList(ps);
			targetLocation = null;

			if (targets.size() > 0)
			{
				targetLocation = targets.elementAt(0).target.getLocation();
			}
		}

		for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
		{
			Planet planet = (Planet) player.getEmpire().getPlanets().elementAt(i);

			AIPlanetIntelligence intel = intelligenceFile.get(planet.getName());

			//System.out.println(planet + " Production Value: " + intel.getProductionImportance(targetLocation, map, intelligenceFile));
			ps.println(planet + " Production Value: " + intel.getProductionImportance(targetLocation, map, intelligenceFile));
		}

		player.getEmpire().calculateMaintenance();

		PaxProductionTree ppt = new PaxProductionTree(player, totalSpaceForces, totalGroundForces, map, beamReduction, ps);
		beamReduction = ppt.beamReduction;

		scoreProductionTree(ppt.productionRoot);

		ppt.findBestProductionNode();

		PaxImprovementTree pit = new PaxImprovementTree(player, ppt, threadCount, ps);

		scoreImprovementTree(pit.root);

		PaxImprovementTreeNode bestImprovementNode = pit.traverseImprovementTree(pit.root, pit.root);

		bestImprovementNode.playerState.playerImprovementDataToPlayer(player);

		for (int i = 0; i < bestImprovementNode.bestProductionNode.forcesBuilt.size(); i++)
		{
			Force force = bestImprovementNode.bestProductionNode.forcesBuilt.elementAt(i);

			force.setCommitted(true);

			if (hyperLSD)
			{
				force.hyperLSD();
			}

			player.addForce(force, ((Planet)player.getEmpire().getPlanets().elementAt(bestImprovementNode.bestProductionNode.planetBuilt.elementAt(i))).getLocation());
		}

		//System.out.println("Best score: " + bestImprovementNode.score);
		//System.out.println(player.getEmpire().getDescription());
		ps.println(player.getEmpire().getDescription());

		player.getEmpire().calculateVictoryPoints(map);

		if (!hyperLSD && player.getEmpire().getTech() > 44)
		{
			hyperLSD = true;

			for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
			{
				Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

				for (int j = 0; j < sUnit.getForces().size(); j++)
				{
					Force force = (Force) sUnit.getForces().elementAt(j);
					force.hyperLSD();
				}
			}
		}

		lastTurnPlayers = new Vector<Player>();

		for (int i = 0; i < map.getPlayers().size(); i++)
		{
			Player p = (Player) map.getPlayers().elementAt(i);
			lastTurnPlayers.addElement(p.clone());
		}

		improvementCount++;
	}


	public void scoreImprovementTree(PaxImprovementTreeNode node)
	{
		if (node.bestProductionNode != null)
		{
			node.score = scoreImprovementNode(node.playerState) + node.bestProductionNode.score;

			int maintenance = 0;

			for (int i = 0; i < node.bestProductionNode.forcesBuilt.size(); i++)
			{
				maintenance += node.bestProductionNode.forcesBuilt.elementAt(i).getMaintenance();
			}

			node.score += (player.getEmpire().getIncome() - maintenance) * 0.5;

			node.score += node.score * (random.nextDouble() * totalRandomFactor * 2 - totalRandomFactor);
		}

		for (int i = 0; i < node.children.size(); i++)
		{
			scoreImprovementTree(node.children.elementAt(i));
		}
	}

	public void scoreProductionTree(PaxProductionTreeNode node)
	{
		node.score = scoreProductionNode(player, node);

		for (int i = 0; i < node.children.size(); i++)
		{
			scoreProductionTree(node.children.elementAt(i));
		}
	}


	// Calculates a score for a player, takes into account their improvement phase
	// states, and uses the production score stored in the production node.
	public int scoreImprovementNode(PlayerImprovementData node)
	{
		double score = 0;

		score += node.tech * currentTechWeight;
		score += node.espionage * currentEspionageWeight;

		for (int i = 0; i < node.economies.size(); i++)
		{
			score += node.economies.elementAt(i) * (currentEconomyWeight + Math.max(0.0, 5 - map.getTurn()));
			score += node.industries.elementAt(i) * currentIndustryWeight;
		}

		score += node.money * currentMoneyWeight;;

		return (int) score;
	}


	// Calculates a score based on the player's set of units.
	// Uses ratios based on space/ground, mobile/immobile, and transports/ground units.
	public int scoreProductionNode(Player player, PaxProductionTreeNode node)
	{
		Vector<Force> forcesBuilt = node.forcesBuilt;

		double score = 0;
		double spaceAttack = 0;
		double spaceArmor = 0;
		double groundAttack = 0;
		double groundArmor = 0;
		double bonus = 0;
		double bonusTrans = 0;
		double bonusGround = 0;
		double bonusSpace = 0;

		int spaceMovement = 0;
		int transportCapacity = 0;
		int mobileGroundForces = 0;
		int mobileGroundOnPlanets = 0;
		int mobileSpaceForces = 0;
		int immobileForces = 0;

		int remainingFood = player.getEmpire().getFood();
		int remainingGas = player.getEmpire().getGas();
		int remainingMinerals = player.getEmpire().getMinerals();

		for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
		{
			Unit unit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

			for (int t = 0; t < unit.getForces().size(); t++)
			{
				Force force = (Force) unit.getForces().elementAt(t);

				spaceArmor += force.getArmor();
				spaceAttack += force.getMaxAttack();
				spaceMovement += force.getCurrentMove();

				if (force.getMobile())
				{
					mobileSpaceForces++;
				}
				else
				{
					immobileForces++;
				}

				if (unit.getLocation().getPlanets().size() > 0 && ((Planet)unit.getLocation().getPlanets().elementAt(0)).getOwner().equals(player.getName()))
				{
					transportCapacity += ((SpaceForce)force).getStorageCapacity();
				}

			}
		}
		for (int i = 0; i < player.getEmpire().getGroundUnits().size(); i++)
		{
			Unit unit = (Unit) player.getEmpire().getGroundUnits().elementAt(i);

			for (int t = 0; t < unit.getForces().size(); t++)
			{
				Force force = (Force) unit.getForces().elementAt(t);

				groundArmor += force.getArmor();
				groundAttack += force.getMaxAttack();

				if (force.getMobile())
				{
					mobileGroundForces++;

					if (!unit.getLoaded())
					{
						mobileGroundOnPlanets++;
					}
				}
				else
				{
					immobileForces++;
				}


			}
		}

		for (int i = 0; i < forcesBuilt.size(); i++)
		{
			Force force = forcesBuilt.elementAt(i);

			Planet planet = (Planet)player.getEmpire().getPlanets().elementAt(node.planetBuilt.elementAt(i));

			AIPlanetIntelligence intel = intelligenceFile.get(planet.getName());
			bonus += (intel.getProductionImportance(targetLocation, map, intelligenceFile) - 1.0) * ( force.getArmor() + force.getMaxAttack() + force.getCurrentMove());

			remainingFood -= force.getFood();
			remainingGas -= force.getGas();
			remainingMinerals -= force.getMinerals();

			double power = 1.0;

			if (force.getArmor() >= 40 && force.getMobile())
			{
				power = 1.2;
			}

			if (force.getClass().getName().equals("SpaceForce"))
			{
				spaceArmor += Math.pow(force.getArmor(), power);
				spaceAttack += Math.pow(force.getMaxAttack(), power);
				spaceMovement += force.getCurrentMove();

				if (force.getCurrentMove() == 1)
				{
					spaceMovement -= 1;
					bonus -= 10;
				}

				if (force.getMobile())
				{
					mobileSpaceForces++;
				}
				else
				{
					immobileForces++;
				}

				transportCapacity += ((SpaceForce)force).getStorageCapacity();

				if (orderSpace && ((SpaceForce)force).getStorageCapacity() == 0)
				{
					bonusSpace = 30;
				}

				if (orderTransport && ((SpaceForce)force).getStorageCapacity() > 0)
				{
					bonusTrans = 30;
				}

			}
			else
			{
				groundArmor += Math.pow(force.getArmor(), power);
				groundAttack += Math.pow(force.getMaxAttack(), power);

				if (force.getMobile())
				{
					mobileGroundForces++;
					mobileGroundOnPlanets++;

					if (orderGround)
					{
						bonusGround = 30;
					}
				}
				else
				{
					immobileForces++;
				}
			}
		}

		score += spaceAttack + spaceArmor + groundAttack + groundArmor + (spaceMovement * currentMovementWeight);

		// Figure out score ratios.
		double penalty = 0;

		double ratio = Math.abs((spaceAttack + spaceArmor)/(groundAttack + groundArmor+0.001) - currentSpaceGroundRatio);

		if (ratio < 0.5)
		{
			ratio = 0;
		}

		ratio = Math.min(1.0, ratio);

		penalty += ratio;

		ratio = ((mobileGroundOnPlanets)/(transportCapacity+0.001) - currentGroundTransportRatio);

		if (ratio < 0)
		{
			ratio = 0;
		}

		ratio = Math.min(1.0, ratio);

		penalty += ratio;


		ratio = ((mobileGroundForces+mobileSpaceForces)/(immobileForces+0.001) - currentMobileImmobileRatio);

		if (ratio > 0)
		{
			ratio = 0;
		}
		else
		{
			ratio = Math.abs(ratio);
		}

		ratio = Math.min(1.0, ratio);

		penalty += ratio;

		score -= (penalty/3.0 * score);

		if (remainingFood > 0)
		{
			score += Math.log(remainingFood+1.8) * currentFoodWeight;
		}

		if (remainingGas > 0)
		{
			score += Math.log(remainingGas+1.8) * currentGasWeight;
		}

		if (remainingMinerals > 0)
		{
			score += Math.log(remainingMinerals+1.8) * currentMineralWeight;
		}

		score += bonus + bonusTrans + bonusGround + bonusSpace;

		return (int) score;
	}

	// Determine the random weightings for this turn.
	public void setRandomWeightings(PrintStream ps)
	{
		currentSpaceGroundRatio = spaceGroundRatio + (spaceGroundRatio * (random.nextFloat() * productionRandomFactor * 2.0f - productionRandomFactor));
		currentGroundTransportRatio = groundTransportRatio + (groundTransportRatio * (random.nextFloat() * productionRandomFactor * 2.0f - productionRandomFactor));
		currentMobileImmobileRatio = mobileImmobileRatio + (mobileImmobileRatio * (random.nextFloat() * productionRandomFactor * 2.0f - productionRandomFactor));
		currentMovementWeight = movementWeight + (movementWeight * (random.nextFloat() * productionRandomFactor * 2.0f - productionRandomFactor));
		//System.out.println("Spc/Grd: " + currentSpaceGroundRatio + ", Grd/Trans: " + currentGroundTransportRatio + ", Mob/Immob: " + currentMobileImmobileRatio + ", Move: " + currentMovementWeight);
		ps.println("Spc/Grd: " + currentSpaceGroundRatio + ", Grd/Trans: " + currentGroundTransportRatio + ", Mob/Immob: " + currentMobileImmobileRatio + ", Move: " + currentMovementWeight);

		currentFoodWeight = foodWeight + (foodWeight * (random.nextFloat() * productionRandomFactor * 2.0f - productionRandomFactor));
		currentGasWeight = gasWeight + (gasWeight * (random.nextFloat() * productionRandomFactor * 2.0f - productionRandomFactor));
		currentMineralWeight = mineralWeight + (mineralWeight * (random.nextFloat() * productionRandomFactor * 2.0f - productionRandomFactor));
		//System.out.println("Food: " + currentFoodWeight + ", Gas: " + currentGasWeight + ", Mineral: " + currentMineralWeight);
		ps.println("Food: " + currentFoodWeight + ", Gas: " + currentGasWeight + ", Mineral: " + currentMineralWeight);

		currentEconomyWeight = economyWeight + (economyWeight * (random.nextFloat() * improvementRandomFactor * 2.0f - improvementRandomFactor));
		currentIndustryWeight = industryWeight + (industryWeight * (random.nextFloat() * improvementRandomFactor * 2.0f - improvementRandomFactor));
		currentTechWeight = techWeight + (techWeight * (random.nextFloat() * improvementRandomFactor * 2.0f - improvementRandomFactor));
		currentEspionageWeight = espionageWeight + (espionageWeight * (random.nextFloat() * improvementRandomFactor * 2.0f - improvementRandomFactor));
		currentMoneyWeight = moneyWeight + (moneyWeight * (random.nextFloat() * improvementRandomFactor * 2.0f - improvementRandomFactor));
		//System.out.println("Eco: " + currentEconomyWeight +", Ind: " + currentIndustryWeight + ", Tech: " +currentTechWeight + ", Esp: " +currentEspionageWeight + ", Money: " + currentMoneyWeight);
		ps.println("Eco: " + currentEconomyWeight +", Ind: " + currentIndustryWeight + ", Tech: " +currentTechWeight + ", Esp: " +currentEspionageWeight + ", Money: " + currentMoneyWeight);
	}


	public void cleanUnits()
	{
		for (int i = 0; i < aiUnits.size(); i++)
		{
			if (aiUnits.elementAt(i).unit.isEmpty() || (aiUnits.elementAt(i).destination == null) || (aiUnits.elementAt(i).unit.getLocation().getPositionX() == aiUnits.elementAt(i).destination.getPositionX() && aiUnits.elementAt(i).unit.getLocation().getPositionY() == aiUnits.elementAt(i).destination.getPositionY()))
			{
				aiUnits.removeElementAt(i);
				i--;
			}
		}
		for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
		{
			if (((Unit)player.getEmpire().getSpaceUnits().elementAt(i)).isEmpty())
			{
				player.getEmpire().getSpaceUnits().removeElementAt(i);
				i--;
			}
		}
		for (int i = 0; i < player.getEmpire().getGroundUnits().size(); i++)
		{
			if (((Unit)player.getEmpire().getGroundUnits().elementAt(i)).isEmpty())
			{
				player.getEmpire().getGroundUnits().removeElementAt(i);
				i--;
			}
		}
	}



	public void checkMood(Vector<Player> otherPlayers, PrintStream ps)
	{
		for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
		{
			Planet planet = (Planet) player.getEmpire().getPlanets().elementAt(i);

			Vector<Unit> closestEnemyUnits = findClosestEnemyUnits(planet, otherPlayers);

			int distance = 9;

			if (closestEnemyUnits.size() > 0)
			{
				distance = map.distance(closestEnemyUnits.elementAt(0).getLocation(), planet.getLocation());
			}

			//System.out.println("Distance: " + distance + " " + planet);

			distance = (distance-1)/2;

			AIPlanetIntelligence intelFile = intelligenceFile.get(planet.getName());
			intelFile.distanceToThreats = distance;
			intelFile.closestEnemyUnits = closestEnemyUnits;
			double maxThreat = intelFile.getMaximumThreateningAggression(map, intelligenceFile);

			maxThreat = Math.min(maxThreat/5, 1.0);
			maxThreat = (maxThreat * 0.3) - 0.05;

			double defenseProbability = (currentDefensiveProbability - Math.pow((distance+1),2.5)/10.0 + maxThreat);
			//System.out.println("Defense check: " + distance + ", " + defenseProbability);
			ps.println("Defense check: " + distance + ", " + defenseProbability);

			if (random.nextDouble() <= defenseProbability)
			{
				//System.out.println(planet + " is threatened, and is switching to a defensive state!");
				ps.println(planet + " is threatened, and is switching to a defensive state!");
				intelFile.mood = intelFile.DEFENSIVE;
			}
			else
			{
				intelFile.mood = intelFile.NEUTRAL;
			}
		}

		// Reset mood to NEUTRAL if we have lost a planet, consider what the next choice will be.
		if (lostPlanet)
		{
			//System.out.println("A planet has been lost, reverting to a neutral state.");
			ps.println("A planet has been lost, reverting to a neutral state.");
			aiMood = NEUTRAL;
			lostPlanet = false;
		}

		if (aiMood == NEUTRAL)
		{
			// Figure out the chance that we switch to OFFENSIVE mode.
			double offenseProbability = (currentOffensiveProbability + Math.pow(player.getEmpire().getMaintenance()/(player.getEmpire().getIncome() + player.getEmpire().getMaintenance() + 1.0),2.0));
			//System.out.println("Offense check: " + player.getEmpire().getMaintenance()/(player.getEmpire().getIncome() + player.getEmpire().getMaintenance() + 1.0) + ", " + offenseProbability);
			ps.println("Offense check: " + player.getEmpire().getMaintenance()/(player.getEmpire().getIncome() + player.getEmpire().getMaintenance() + 1.0) + ", " + offenseProbability);

			if (random.nextDouble() <= offenseProbability)
			{
				//System.out.println("We are ready to attack! Switching to an offensive mood!");
				ps.println("We are ready to attack! Switching to an offensive mood!");
				aiMood = OFFENSIVE;
			}
		}

		boolean liberatingSpace = liberateSpace(ps);

		if (aiMood == NEUTRAL)
		{
			neutralMood();
		}
		else if (aiMood == OFFENSIVE)
		{
			offensiveMood(ps);
		}

		seizeAsteroids(otherPlayers, ps);
		repositionForces(ps);
	}

	// Check that units are not moving into unwinnable battles when they are carrying ground troops, and if so, roll back their
	// movement.
	public void checkMovement(PrintStream ps)
	{
		for (int i = 0; i < aiUnits.size(); i++)
		{
			boolean carryingGround = false;

			Unit sUnit = aiUnits.elementAt(i).unit;

			for (int t = 0; t < sUnit.getForces().size(); t++)
			{
				SpaceForce sf = (SpaceForce) sUnit.getForces().elementAt(t);

				if (sf.getStorage().size() > 0)
				{
					carryingGround = true;
					break;
				}
			}

			if (carryingGround && aiUnits.elementAt(i).destination.getPositionX() == sUnit.getLocation().getPositionX() && aiUnits.elementAt(i).destination.getPositionY() == sUnit.getLocation().getPositionY())
			{
				if (sUnit.getLocation().getPlanets().size() > 0)
				{
					Planet targetPlanet = (Planet) sUnit.getLocation().getPlanets().elementAt(0);

					AIPlanetIntelligence intel = intelligenceFile.get(targetPlanet.getName());

					int targetSpaceAttackTotal = 0;
					int targetSpaceArmorTotal = 0;

					for (int z = 0; z < intel.units.size(); z++)
					{
						Unit u = intel.units.elementAt(z);

						if (u.isSpace())
						{
							targetSpaceAttackTotal += u.getTotalMaxAttack();
							targetSpaceArmorTotal += u.getTotalArmor();
						}
					}

					int spaceAttackTotal = player.getEmpire().getSpaceMaxAttack(targetPlanet.getLocation());
					int spaceArmorTotal = player.getEmpire().getSpaceArmor(targetPlanet.getLocation());

					if ((spaceAttackTotal+spaceArmorTotal) < (targetSpaceAttackTotal+targetSpaceArmorTotal))
					{
						aiUnits.elementAt(i).rollBack();

						//System.out.println(aiUnits.elementAt(i).unit + " is rolling back from " + targetPlanet.getLocation() + " to " + aiUnits.elementAt(i).unit.getLocation());
						ps.println(aiUnits.elementAt(i).unit + " is rolling back from " + targetPlanet.getLocation() + " to " + aiUnits.elementAt(i).unit.getLocation());
					}
				}
			}
		}
	}

	// Determines if the AI should attempt to seize the asteroid belt.
	public void seizeAsteroids(Vector<Player> otherPlayers, PrintStream ps)
	{
		boolean asteroidBelt = false;

		// Check if the player already has ships in the asteroid belt.
		for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
		{
			Unit u = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

			if (!u.isEmpty())
			{
				Location loc = u.getLocation();

				if (loc.getAsteroids())
				{
					asteroidBelt = true;
					break;
				}
			}
		}

		// Consider taking the belt if we don't already own it.
		if (!asteroidBelt)
		{
			Planet closestPlanet = null;
			Location closestAsteroidLocation = null;
			int minDistance = 100000;

			for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
			{
				Planet planet = (Planet) player.getEmpire().getPlanets().elementAt(i);

				if (intelligenceFile.get(planet.getName()).mood != DEFENSIVE)
				{
					Location asteroidLocation = map.findClosestAsteroidLocation(planet.getLocation());

					if (map.distance(asteroidLocation, planet.getLocation()) < minDistance)
					{
						minDistance = map.distance(asteroidLocation, planet.getLocation());
						closestAsteroidLocation = asteroidLocation;
						closestPlanet = planet;
					}
				}
			}

			boolean alreadyAttacking = false;

			for (int i = 0; i < aiUnits.size(); i++)
			{
				if (aiUnits.elementAt(i).destination.getAsteroids())
				{
					alreadyAttacking = true;
				}
			}

			// If we have not already launched an attack force at the belt, and there is a planet that's not defensive from which to launch an attack on the belt...
			if (!alreadyAttacking && closestPlanet != null)
			{
				minDistance = (minDistance-1)/2;
				float score = 3.0f * currentMineralWeight;
				score -= 70 * minDistance;
				float aggression = 0;

				// Check if anyone else has forces in the belt.
				for (int i = 0; i < otherPlayers.size(); i++)
				{
					Player play = otherPlayers.elementAt(i);

					if (!play.getName().equals(player.getName()))
					{
						for (int t = 0; t < play.getEmpire().getSpaceUnits().size(); t++)
						{
							Unit sUnit = (Unit) play.getEmpire().getSpaceUnits().elementAt(t);

							if (!sUnit.isEmpty() && sUnit.getLocation().getAsteroids())
							{
								if (play.getEmpire().getPlanets().size() > 0)
								{
									aggression = intelligenceFile.get(((Planet)play.getEmpire().getPlanets().elementAt(0)).getName()).aggression;
								}
							}
						}
					}
				}

				score += aggression * 30.0f;

				if (aggression > 0.01)
				{
					score -= 60.0f;
					//System.out.println("Enemy forces in the belt!");
					ps.println("Enemy forces in the belt!");
				}
				if (player.getEmpire().getRace().equals("Sphere"))
				{
					score += 120.0f;
				}

				score /= 250.0f;

				if (score > 0 && aiMood != OFFENSIVE)
				{
					score /= 1.5f;
				}

				double offenseProbability = (currentOffensiveProbability + score);
				//System.out.println("Asteroid Invasion Check: " + offenseProbability);
				ps.println("Asteroid Invasion Check: " + offenseProbability);

				if (random.nextDouble() <= offenseProbability)
				{
					//System.out.println("We will attempt to seize the asteroid belt!");
					ps.println("We will attempt to seize the asteroid belt!");

					int possibleSpaceAttack = player.getEmpire().getSpaceMaxAttack();
					int possibleSpaceArmor = player.getEmpire().getSpaceArmor();

					// Build an attack force that will destroy the enemy in question.
					int spaceAttackTotal = 0;
					int spaceArmorTotal = 0;

					Vector<Unit> spaceUnits = new Vector<Unit>();

					for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
					{
						Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

						if (!sUnit.isEmpty() && (sUnit.getLocation().getPlanets().size() == 0 || intelligenceFile.get(((Planet)sUnit.getLocation().getPlanets().elementAt(0)).getName()).mood != DEFENSIVE ))
						{
							spaceUnits.addElement(sUnit);
						}
					}

					Vector<Unit> attackUnits = new Vector<Unit>();

					while (spaceUnits.size() > 0)
					{
						Unit sUnit = findClosestUnit(closestAsteroidLocation, spaceUnits);
						spaceUnits.remove(sUnit);

						Unit spaceAttack = new Unit(player.getName() + " Asteroid Attack " + player.getNewUnitCounter(), player.getName());

						//System.out.println("Asteroid attack from " + sUnit.getName());
						ps.println("Asteroid attack from " + sUnit.getName());

						Vector<SpaceForce> spaceForces = getSortedSpaceForces(sUnit.getLocation());

						for (int i = 0; i < spaceForces.size(); i++)
						{
							SpaceForce sf = spaceForces.elementAt(i);

							if (sf.getMobile() && sf.getMaxAttack() > 0)
							{
								if (((spaceAttackTotal+0.0)/possibleSpaceAttack < currentAttackDefenseRatio/3.0 && (spaceArmorTotal+0.0)/possibleSpaceArmor < currentAttackDefenseRatio/3.0))
								{
									spaceAttack.addForce(sf);
									removeSpaceForce(sf, sUnit.getLocation());

									spaceAttackTotal += sf.getMaxAttack();
									spaceArmorTotal += sf.getArmor();
								}
							}
							if ((spaceAttackTotal+0.0)/possibleSpaceAttack > currentAttackDefenseRatio/3.0 || (spaceArmorTotal+0.0)/possibleSpaceArmor > currentAttackDefenseRatio/3.0)
							{
								spaceUnits.clear();
								break;
							}
						}

						if (!spaceAttack.isEmpty())
						{
							player.getEmpire().addSpaceUnit(spaceAttack);
							spaceAttack.setLocation(sUnit.getLocation());
							spaceAttack.getLocation().addUnit(spaceAttack);
							attackUnits.addElement(spaceAttack);
						}
					}

					//System.out.println("Potential Space: " + possibleSpaceAttack + ", " + possibleSpaceArmor);
					ps.println("Potential Space: " + possibleSpaceAttack + ", " + possibleSpaceArmor);

					//System.out.println("Attack force space: " + spaceAttackTotal + ", " + spaceArmorTotal);
					ps.println("Attack force space: " + spaceAttackTotal + ", " + spaceArmorTotal);

					for (int i = 0; i < attackUnits.size(); i++)
					{
						aiUnits.addElement(new AIUnit(attackUnits.elementAt(i), closestAsteroidLocation));
					}
				}
			}
		}
	}

	// Returns a list of the closest enemy units controlled by the players in otherPlayers to the
	// planet passed into the argument.
	public Vector<Unit> findClosestEnemyUnits(Planet planet, Vector<Player> otherPlayers)
	{
		int minDistance = 10000;
		Vector<Unit> closestEnemyUnits = new Vector<Unit>();

		for (int i = 0; i < otherPlayers.size(); i++)
		{
			Player p = otherPlayers.elementAt(i);

			if (!p.getName().equals(player.getName()))
			{
				for (int j = 0; j < p.getEmpire().getSpaceUnits().size(); j++)
				{
					Unit sUnit = (Unit) p.getEmpire().getSpaceUnits().elementAt(j);

					if (!sUnit.isEmpty())
					{
						int distance = map.distance(planet.getLocation(), sUnit.getLocation());

						distance = (distance-1)/2;

						if (distance == minDistance)
						{
							closestEnemyUnits.addElement(sUnit);
						}
						else if (distance < minDistance)
						{
							minDistance = distance;

							closestEnemyUnits.clear();
							closestEnemyUnits.addElement(sUnit);
						}
					}
				}
			}
		}
		return closestEnemyUnits;
	}

	public void neutralMood()
	{
		currentFoodWeight *= 1.2;
		currentGasWeight *= 1.2;
		currentMineralWeight *= 1.2;

		currentEconomyWeight *= 1.2;
		currentTechWeight *= 1.2;
		currentEspionageWeight *= 1.2;
		currentMoneyWeight *= 1.2;
	}

	// Reposition troops from planets far from the frontline towards those in greater danger.
	public void repositionForces(PrintStream ps)
	{
		// Check each planet against its closest neighbor to determine if it has more forces, and if it is of a lower production importance.
		for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
		{
			Planet currentPlanet = (Planet) player.getEmpire().getPlanets().elementAt(i);

			player.getEmpire().getPlanets().remove(currentPlanet);

			Planet neighbor = findMinimumDistance(currentPlanet.getLocation(), player.getEmpire().getPlanets());

			if (neighbor != null && intelligenceFile.get(neighbor.getName()).getProductionImportance(targetLocation, map, intelligenceFile) >= intelligenceFile.get(currentPlanet.getName()).getProductionImportance(targetLocation, map, intelligenceFile))
			{
				int deltaSpaceMaxAttack = player.getEmpire().getSpaceMaxAttack(currentPlanet.getLocation()) - player.getEmpire().getSpaceMaxAttack(neighbor.getLocation());
				int deltaSpaceArmor = player.getEmpire().getSpaceArmor(currentPlanet.getLocation()) - player.getEmpire().getSpaceArmor(neighbor.getLocation());
				int deltaGroundMaxAttack = player.getEmpire().getGroundMaxAttack(currentPlanet.getLocation()) - player.getEmpire().getGroundMaxAttack(neighbor.getLocation());
				int deltaGroundArmor = player.getEmpire().getGroundArmor(currentPlanet.getLocation()) - player.getEmpire().getGroundArmor(neighbor.getLocation());

				//System.out.println("Trying to reposition from " + currentPlanet + " to " + neighbor);
				//System.out.println("Delta Space: " + deltaSpaceMaxAttack + " / " + deltaSpaceArmor);
				//System.out.println("Delta Ground: " + deltaGroundMaxAttack + " / " + deltaGroundArmor);

				ps.println("Trying to reposition from " + currentPlanet + " to " + neighbor);
				ps.println("Delta Space: " + deltaSpaceMaxAttack + " / " + deltaSpaceArmor);
				ps.println("Delta Ground: " + deltaGroundMaxAttack + " / " + deltaGroundArmor);

				Unit sUnit = new Unit("Space Reposition " + player.getNewUnitCounter(), player.getName());

				Vector<SpaceForce> spaceForces = getSortedSpaceForces(currentPlanet.getLocation());
				Vector<GroundForce> groundForces = getSortedGroundForces(currentPlanet.getLocation());

				int z = 0;
				while (spaceForces.size() > 0 && (deltaSpaceMaxAttack + deltaSpaceArmor + deltaGroundMaxAttack + deltaGroundArmor) >= 0)
				{
					if (spaceForces.size() > 0)
					{
						SpaceForce sf = spaceForces.elementAt(0);
						spaceForces.remove(sf);

						if ((deltaSpaceMaxAttack+deltaSpaceArmor-sf.getMaxAttack()-sf.getArmor()) > 0 || (sf.getStorageCapacity() > 0 && (deltaGroundMaxAttack + deltaGroundArmor) > 0))
						{
							//System.out.println("Adding " + sf.getName() + " to reposition forces!");
							ps.println("Adding " + sf.getName() + " to reposition forces!");

							deltaSpaceMaxAttack -= sf.getMaxAttack();
							deltaSpaceArmor -= sf.getArmor();

							Unit gUnit = new Unit("Ground Reposition " + player.getNewUnitCounter(), player.getName());

							while (groundForces.size() > 0 && gUnit.getForces().size() < sf.getStorageCapacity())// && (deltaGroundMaxAttack+deltaGroundArmor) > 0)
							{
								GroundForce gf = groundForces.elementAt(0);
								groundForces.remove(gf);

								if ((deltaGroundMaxAttack+deltaGroundArmor-gf.getMaxAttack()-gf.getArmor()) > 0)
								{
									deltaGroundMaxAttack -= gf.getMaxAttack();
									deltaGroundArmor -= gf.getArmor();

									//System.out.println("Adding " + gf.getName() + " to reposition forces!");
									ps.println("Adding " + gf.getName() + " to reposition forces!");

									gUnit.addForce(gf);
									removeGroundForce(gf, currentPlanet.getLocation());
								}
							}

							if (!gUnit.isEmpty())
							{
								sf.getStorage().addElement(gUnit);

								player.getEmpire().getGroundUnits().addElement(gUnit);
								gUnit.setLocation(currentPlanet.getLocation());
								currentPlanet.getLocation().addUnit(gUnit);
								gUnit.setLoaded(true);
							}

							sUnit.addForce(sf);
							removeSpaceForce(sf, currentPlanet.getLocation());
						}
					}

					if (++z >= 1000)
					{
						break;
					}
				}

				if (!sUnit.isEmpty())
				{
					player.getEmpire().getSpaceUnits().addElement(sUnit);
					sUnit.setLocation(currentPlanet.getLocation());
					currentPlanet.getLocation().addUnit(sUnit);

					aiUnits.addElement(new AIUnit(sUnit, neighbor.getLocation()));
				}
			}

			player.getEmpire().getPlanets().insertElementAt(currentPlanet, i);
		}

	}


	// Liberate space above one of the worlds.
	public boolean liberateSpace(PrintStream ps)
	{
		boolean launchingForce = false;
		Location targetSpace = null;

		for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
		{
			Planet planet = (Planet) player.getEmpire().getPlanets().elementAt(i);

			if (!planet.isSpaceClear(player, lastTurnPlayers))
			{
				targetSpace = planet.getLocation();
				break;
			}
		}

		if (targetSpace != null)
		{
			int possibleSpaceAttack = player.getEmpire().getSpaceMaxAttack();
			int possibleSpaceArmor = player.getEmpire().getSpaceArmor();

			// Build an attack force that will destroy the enemy in question.
			int spaceAttackTotal = 0;
			int spaceArmorTotal = 0;

			// Calculate composition of current attack force.
			for (int i = 0; i < aiUnits.size(); i++)
			{
				if (aiUnits.elementAt(i).destination.getPositionX() == targetSpace.getPositionX() && aiUnits.elementAt(i).destination.getPositionY() == targetSpace.getPositionY())
				{
					for (int j = 0; j < aiUnits.elementAt(i).unit.getForces().size(); j++)
					{
						SpaceForce sf = (SpaceForce) aiUnits.elementAt(i).unit.getForces().elementAt(j);

						spaceAttackTotal += sf.getMaxAttack();
						spaceArmorTotal += sf.getArmor();

					}
				}
			}

			AIPlanetIntelligence intel = intelligenceFile.get(((Planet)targetSpace.getPlanets().elementAt(0)).getName());

			int targetSpaceAttackTotal = 0;
			int targetSpaceArmorTotal = 0;

			for (int i = 0; i < intel.units.size(); i++)
			{
				Unit u = intel.units.elementAt(i);

				if (u.isSpace() && !u.getOwner().equals(player.getName()))
				{
					targetSpaceAttackTotal += u.getTotalMaxAttack();
					targetSpaceArmorTotal += u.getTotalArmor();
				}
			}

			Vector<Unit> spaceUnits = new Vector<Unit>();

			for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
			{
				Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

				if (!sUnit.isEmpty() && (sUnit.getLocation().getPlanets().size() == 0 || intelligenceFile.get(((Planet)sUnit.getLocation().getPlanets().elementAt(0)).getName()).mood != DEFENSIVE ))
				{
					spaceUnits.addElement(sUnit);
				}
			}

			Vector<Unit> attackUnits = new Vector<Unit>();

			while (spaceUnits.size() > 0)
			{
				Unit sUnit = findClosestUnit(targetSpace, spaceUnits);
				spaceUnits.remove(sUnit);

				Unit spaceAttack = new Unit(player.getName() + " Space Attack " + player.getNewUnitCounter(), player.getName());

				//System.out.println("Liberating space from " + sUnit.getName());
				ps.println("Liberating space from " + sUnit.getName());

				Vector<SpaceForce> spaceForces = getSortedSpaceForces(sUnit.getLocation());

				for (int i = 0; i < spaceForces.size(); i++)
				{
					SpaceForce sf = spaceForces.elementAt(i);

					if (sf.getMobile() && sf.getMaxAttack() > 0)
					{
						if (((spaceAttackTotal+0.0)/possibleSpaceAttack < currentAttackDefenseRatio && (spaceArmorTotal+0.0)/possibleSpaceArmor < currentAttackDefenseRatio))
						{
							spaceAttack.addForce(sf);
							removeSpaceForce(sf, sUnit.getLocation());

							spaceAttackTotal += sf.getMaxAttack();
							spaceArmorTotal += sf.getArmor();
						}
					}
					if ((spaceAttackTotal+0.0)/possibleSpaceAttack > currentAttackDefenseRatio || (spaceArmorTotal+0.0)/possibleSpaceArmor > currentAttackDefenseRatio)
					{
						spaceUnits.clear();
						break;
					}
				}

				if (!spaceAttack.isEmpty())
				{
					player.getEmpire().addSpaceUnit(spaceAttack);
					spaceAttack.setLocation(sUnit.getLocation());
					spaceAttack.getLocation().addUnit(spaceAttack);
					attackUnits.addElement(spaceAttack);
				}
			}

			//System.out.println("Target Space: " + targetSpaceAttackTotal + ", " + targetSpaceArmorTotal);
			ps.println("Target Space: " + targetSpaceAttackTotal + ", " + targetSpaceArmorTotal);

			//System.out.println("Potential Space: " + possibleSpaceAttack + ", " + possibleSpaceArmor);
			ps.println("Potential Space: " + possibleSpaceAttack + ", " + possibleSpaceArmor);

			//System.out.println("Attack force space: " + spaceAttackTotal + ", " + spaceArmorTotal);
			ps.println("Attack force space: " + spaceAttackTotal + ", " + spaceArmorTotal);

			if (spaceAttackTotal > targetSpaceAttackTotal || spaceArmorTotal > targetSpaceArmorTotal)
			{
				for (int i = 0; i < attackUnits.size(); i++)
				{
					aiUnits.addElement(new AIUnit(attackUnits.elementAt(i), targetSpace));
					launchingForce = true;
				}
			}

		}
		return launchingForce;
	}

	// Recalls all ships in deep space to the nearest allied planet.
	public void recallSpace(PrintStream ps)
	{
		for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
		{
			Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

			// Check if this space unit is currently stationed in deep space.
			if (!sUnit.isEmpty() && sUnit.getLocation().getPlanets().size() == 0 && !sUnit.getLocation().getAsteroids())
			{
				Planet destination = findMinimumDistance(sUnit.getLocation(), player.getEmpire().getPlanets());

				for (int t = 0; t < aiUnits.size(); t++)
				{
					if (aiUnits.elementAt(t).unit.getName().equals(sUnit.getName()))
					{
						aiUnits.removeElementAt(t);
						t--;
					}
				}
				aiUnits.addElement(new AIUnit(sUnit, destination.getLocation()));
				//System.out.println("Redeploying offensive unit.");
				ps.println("Redeploying offensive unit.");
			}
		}
	}

	// Switches all attacking forces to target the new target location.
	public void switchTargets(Location newTargetLocation)
	{
		for (int i = 0; i < aiUnits.size(); i++)
		{
			AIUnit sUnit = (AIUnit) aiUnits.elementAt(i);

			if (sUnit.destination.getPositionX() == targetLocation.getPositionX() && sUnit.destination.getPositionY() == targetLocation.getPositionY())
			{
				sUnit.destination = newTargetLocation;
			}
		}
	}

	public void offensiveMood(PrintStream ps)
	{
		returnTransports();

		boolean moreGround = true;
		boolean moreTransportSpace = false;

		int possibleSpaceAttack = player.getEmpire().getSpaceMaxAttack();
		int possibleSpaceArmor = player.getEmpire().getSpaceArmor();
		int possibleGroundAttack = player.getEmpire().getGroundMaxAttack();
		int possibleGroundArmor = player.getEmpire().getGroundArmor();

		// Build an attack force that will destroy the enemy in question.

		int spaceAttackTotal = 0;
		int spaceArmorTotal = 0;

		int groundAttackTotal = 0;
		int groundArmorTotal = 0;

		// Calculate composition of current attack force.
		if (targetLocation != null)
		{
			// Calculate composition of current attack force.
			for (int i = 0; i < aiUnits.size(); i++)
			{
				if (aiUnits.elementAt(i).destination.getPositionX() == targetLocation.getPositionX() && aiUnits.elementAt(i).destination.getPositionY() == targetLocation.getPositionY())
				{
					for (int j = 0; j < aiUnits.elementAt(i).unit.getForces().size(); j++)
					{
						SpaceForce sf = (SpaceForce) aiUnits.elementAt(i).unit.getForces().elementAt(j);

						spaceAttackTotal += sf.getMaxAttack();
						spaceArmorTotal += sf.getArmor();

						for (int z = 0; z < sf.getStorage().size(); z++)
						{
							Unit gUnit = (Unit) sf.getStorage().elementAt(z);

							for (int t = 0; t < gUnit.getForces().size(); t++)
							{
								Force force = (Force) gUnit.getForces().elementAt(t);

								groundAttackTotal += force.getMaxAttack();
								groundArmorTotal += force.getArmor();
							}
						}
					}
				}
			}
		}

		if ((spaceAttackTotal+spaceArmorTotal) == 0)
		{
			Vector<TargetPlanet> targets = buildTargetList(ps);
			targetLocation = null;

			if (targets.size() > 0)
			{
				targetLocation = targets.elementAt(0).target.getLocation();
			}
			else
			{
				return;
			}
		}

		AIPlanetIntelligence intel = intelligenceFile.get(((Planet)targetLocation.getPlanets().elementAt(0)).getName());

		int targetSpaceAttackTotal = 0;
		int targetSpaceArmorTotal = 0;

		int targetGroundAttackTotal = 0;
		int targetGroundArmorTotal = 0;


		for (int i = 0; i < intel.units.size(); i++)
		{
			Unit u = intel.units.elementAt(i);

			if (u.isSpace())
			{
				targetSpaceAttackTotal += u.getTotalMaxAttack();
				targetSpaceArmorTotal += u.getTotalArmor();
			}
			else
			{
				targetGroundAttackTotal += u.getTotalMaxAttack();
				targetGroundArmorTotal += u.getTotalArmor();
			}
		}

		Vector<Planet> offensivePlanets = new Vector<Planet>();

		for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
		{
			Planet plan = (Planet) player.getEmpire().getPlanets().elementAt(i);

			intel = intelligenceFile.get(plan.getName());

			if (intel.mood != intel.DEFENSIVE || (targetLocation.getPlanets().size() > 0 && intel.isSoleAggressor(((Planet)targetLocation.getPlanets().elementAt(0)).getOwner())))
			{
				offensivePlanets.addElement(plan);
			}
		}

		Vector<Unit> spaceUnits = new Vector<Unit>();

		for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
		{
			Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

			if (!sUnit.isEmpty() && (sUnit.getLocation().getPlanets().size() == 0 || !((Planet)sUnit.getLocation().getPlanets().elementAt(0)).getOwner().equals(player.getName()) || offensivePlanets.contains((Planet)sUnit.getLocation().getPlanets().elementAt(0))))
			{
				spaceUnits.addElement(sUnit);
			}
		}

		Vector<Unit> attackUnits = new Vector<Unit>();

		while (spaceUnits.size() > 0)//while (offensivePlanets.size() > 0)
		{
			Unit sUnit = findClosestUnit(targetLocation, spaceUnits);
			spaceUnits.remove(sUnit);

			moreGround = false;
			if (sUnit.getLocation().getPlanets().size() > 0 && offensivePlanets.contains((Planet)sUnit.getLocation().getPlanets().elementAt(0)))
			{
				moreGround = true;
			}

			Unit spaceAttack = new Unit(player.getName() + " Space Attack " + player.getNewUnitCounter(), player.getName());

			//System.out.println("Loading attackers from " + stagingPlanet);
			//System.out.println("Prepping attackers from " + sUnit.getLocation());

			Vector<SpaceForce> spaceForces = getSortedSpaceForces(sUnit.getLocation());

			for (int i = 0; i < spaceForces.size(); i++)
			{
				SpaceForce sf = spaceForces.elementAt(i);

				if (sf.getMobile())
				{
					if (!matterTransporter() && ((groundAttackTotal+0.0)/possibleGroundAttack < currentAttackDefenseRatio && (groundArmorTotal+0.0)/possibleGroundArmor < currentAttackDefenseRatio) && sf.getStorageCapability() > 0 && moreGround)
					{
						moreGround = loadGroundInto(sf, sUnit.getLocation(), groundAttackTotal, groundArmorTotal, possibleGroundAttack, possibleGroundArmor);

						for (int z = 0; z < sf.getStorage().size(); z++)
						{
							Unit gUnit = (Unit) sf.getStorage().elementAt(z);

							for (int t = 0; t < gUnit.getForces().size(); t++)
							{
								GroundForce gf = (GroundForce) gUnit.getForces().elementAt(t);
								groundAttackTotal += gf.getMaxAttack();
								groundArmorTotal += gf.getArmor();

								//System.out.println(player.getName() + " has loaded " + gf.getName() + " into " + sf);
								ps.println(player.getName() + " has loaded " + gf.getName() + " into " + sf);
							}
						}

						if (sf.getStorageCapability() > 0)
						{
							moreTransportSpace = true;
						}
					}

					if (((spaceAttackTotal+0.0)/possibleSpaceAttack < currentAttackDefenseRatio && (spaceArmorTotal+0.0)/possibleSpaceArmor < currentAttackDefenseRatio) || (sf.getStorage().size() > 0) || (matterTransporter() && sf.getStorageCapacity() > 0))// && (groundAttackTotal+0.0)/possibleGroundAttack < currentAttackDefenseRatio && (groundArmorTotal+0.0)/possibleGroundArmor < currentAttackDefenseRatio))
					{
						spaceAttack.addForce(sf);
						removeSpaceForce(sf, sUnit.getLocation());
						spaceForces.removeElementAt(i);
						i--;

						spaceAttackTotal += sf.getMaxAttack();
						spaceArmorTotal += sf.getArmor();
					}
				}
				if (((spaceAttackTotal+0.0)/possibleSpaceAttack > currentAttackDefenseRatio || (spaceArmorTotal+0.0)/possibleSpaceArmor > currentAttackDefenseRatio) && ((groundAttackTotal+0.0)/possibleGroundAttack > currentAttackDefenseRatio || (groundArmorTotal+0.0)/possibleGroundArmor > currentAttackDefenseRatio))
				{
					//offensivePlanets.clear();
					spaceUnits.clear();
					break;
				}
			}

			if (!spaceAttack.isEmpty())
			{
				player.getEmpire().addSpaceUnit(spaceAttack);
				spaceAttack.setLocation(sUnit.getLocation());
				spaceAttack.getLocation().addUnit(spaceAttack);
				attackUnits.addElement(spaceAttack);
			}
		}

		//System.out.println("Target Space: " + targetSpaceAttackTotal + ", " + targetSpaceArmorTotal);
		ps.println("Target Space: " + targetSpaceAttackTotal + ", " + targetSpaceArmorTotal);
		//System.out.println("Target Ground: " + targetGroundAttackTotal + ", " + targetGroundArmorTotal);
		ps.println("Target Ground: " + targetGroundAttackTotal + ", " + targetGroundArmorTotal);

		//System.out.println("Potential Space: " + possibleSpaceAttack + ", " + possibleSpaceArmor);
		ps.println("Potential Space: " + possibleSpaceAttack + ", " + possibleSpaceArmor);
		//System.out.println("Potential Ground: " + possibleGroundAttack + ", " + possibleGroundArmor);
		ps.println("Potential Ground: " + possibleGroundAttack + ", " + possibleGroundArmor);

		//System.out.println("Attack force space: " + spaceAttackTotal + ", " + spaceArmorTotal);
		ps.println("Attack force space: " + spaceAttackTotal + ", " + spaceArmorTotal);
		//System.out.println("Attack force ground: " + groundAttackTotal + ", " + groundArmorTotal);
		ps.println("Attack force ground: " + groundAttackTotal + ", " + groundArmorTotal);

		if (((spaceAttackTotal+spaceArmorTotal) > (targetSpaceAttackTotal+targetSpaceArmorTotal)) && (((1.3 * groundAttackTotal) > targetGroundAttackTotal && (1.3  * groundArmorTotal) > targetGroundArmorTotal) || matterTransporter()) || player.getEmpire().getIncome() <= 2)
		{
			for (int i = 0; i < attackUnits.size(); i++)
			{
				aiUnits.addElement(new AIUnit(attackUnits.elementAt(i), targetLocation));
			}

			useMatterTransporter(groundAttackTotal, groundArmorTotal, possibleGroundAttack, possibleGroundArmor, ps);
		}
		else
		{
			//System.out.println("Insufficient forces to attack!");
			ps.println("Insufficient forces to attack!");

			// Recall the space forces already on route to this world, select another target.
			recallSpace(ps);

			Vector<TargetPlanet> targets = buildTargetList(ps);
			targetLocation = null;

			if (targets.size() > 0)
			{
				targetLocation = targets.elementAt(0).target.getLocation();
			}


			if (spaceAttackTotal < targetSpaceAttackTotal || spaceArmorTotal < targetSpaceArmorTotal)
			{
				currentSpaceGroundRatio *= 1.5;
				currentMobileImmobileRatio *= 1.5;
				orderSpace = true;

				//System.out.println("Need more space forces for the attack!");
				ps.println("Need more space forces for the attack!");
			}
			if ((groundAttackTotal * 1.3) <= targetGroundAttackTotal || (groundArmorTotal * 1.3) <= targetGroundArmorTotal)
			{
				// Check whether we're short on ground troops, or transports.
				if (!moreTransportSpace)
				{
					//currentSpaceGroundRatio *= 1.2;
					currentGroundTransportRatio /= 1.3;
					currentMobileImmobileRatio *= 1.5;
					currentMovementWeight *= 1.5;
					orderTransport = true;

					//System.out.println("Need more transports for the attack!");
					ps.println("Need more transports for the attack!");
				}
				if (!moreGround || (possibleGroundAttack * currentAttackDefenseRatio * 1.3) < targetGroundAttackTotal || (possibleGroundArmor * currentAttackDefenseRatio * 1.3) < targetGroundArmorTotal)
				{
					currentSpaceGroundRatio /= 1.5;
					currentMobileImmobileRatio *= 1.5;
					orderGround = true;

					//System.out.println("Need more ground units for the attack!");
					ps.println("Need more ground units for the attack!");
				}
			}

			// Unload ground troops, so we can bring the best that are built each time.
			for (int i = 0; i < attackUnits.size(); i++)
			{
				Unit u = attackUnits.elementAt(i);

				if (u.isSpace())
				{
					for (int z = 0; z < u.getForces().size(); z++)
					{
						SpaceForce sf = (SpaceForce) u.getForces().elementAt(z);

						for (int t = 0; t < sf.getStorage().size(); t++)
						{
							Unit gUnit = (Unit) sf.getStorage().elementAt(t);
							gUnit.setLoaded(false);
							sf.getStorage().remove(gUnit);
							t--;
						}
					}
				}
			}
		}
	}

	public void returnTransports()
	{
		if (!matterTransporter())
		{
			Vector<Unit> returningTransportUnits = new Vector<Unit>();

			for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
			{
				Unit unit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

				Planet closestPlanet = findMinimumDistance(unit.getLocation(), player.getEmpire().getPlanets());

				Unit returningTransports = new Unit("Returning Transports", player.getName());

				// We may have transports over an enemy planet that should return to pick up more troops.
				if (unit.getLocation().getPlanets().size() > 0 && !((Planet)unit.getLocation().getPlanets().elementAt(0)).getOwner().equals(player.getName()))
				{
					for (int j = 0; j < unit.getForces().size(); j++)
					{
						SpaceForce sf = (SpaceForce) unit.getForces().elementAt(j);

						// Transfer empty transports to a new unit, send them back to the closest planet in the empire..
						if (sf.getStorageCapacity() > 0 && sf.getStorage().size() == 0)
						{
							returningTransports.addForce(sf);//getForces().addElement(sf);
							unit.removeForce(sf);//.getForces().remove(sf);
							j--;

							//System.out.println("Sending " + sf + " back to " + closestPlanet + "!");
						}
					}
				}

				if (!returningTransports.isEmpty())
				{
					returningTransports.setLocation(unit.getLocation());
					returningTransports.getLocation().addUnit(returningTransports);
					returningTransportUnits.addElement(returningTransports);
					aiUnits.addElement(new AIUnit(returningTransports, closestPlanet.getLocation()));
				}
			}

			for (int i = 0; i < returningTransportUnits.size(); i++)
			{
				Unit returningTransports = returningTransportUnits.elementAt(i);
				player.getEmpire().addSpaceUnit(returningTransports);
			}
		}
	}


	// Returns a sorted list of space forces at the current location.
	public Vector<SpaceForce> getSortedSpaceForces(Location location)
	{
		boolean tokenGuard = false;
		Vector<SpaceForce> spaceForces = new Vector<SpaceForce>();

		for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
		{
			Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

			boolean found = false;

			for (int t = 0; t < aiUnits.size(); t++)
			{
				if (aiUnits.elementAt(t).unit.getName().equals(sUnit.getName()))
				{
					found = true;
				}
			}

			if (!found && !sUnit.isEmpty() && sUnit.getLocation().getPositionX() == location.getPositionX() && sUnit.getLocation().getPositionY() == location.getPositionY())
			{
				for (int j = 0; j < sUnit.getForces().size(); j++)
				{
					SpaceForce sf = (SpaceForce) sUnit.getForces().elementAt(j);

					if (sf.getMobile())
					{
						spaceForces.addElement(sf);
					}
					else
					{
						tokenGuard = true;
					}
				}
			}
		}

		Collections.sort(spaceForces);

		// Leave behind at least a token force at this location if it contains a planet.
		if (location.getPlanets().size() > 0 || location.getAsteroids())
		{
			for (int i = spaceForces.size() - 1; i >= 0 && !tokenGuard; i--)
			{
				SpaceForce sf = spaceForces.elementAt(i);
				if (sf.getStorageCapability() < 1)
				{
					spaceForces.remove(sf);
					break;
				}
			}
		}

		return spaceForces;
	}

	// Returns a sorted list of ground forces at the current location.
	public Vector<GroundForce> getSortedGroundForces(Location location)
	{
		boolean tokenGuard = false;
		Vector<GroundForce> groundForces = new Vector<GroundForce>();

		for (int i = 0; i < player.getEmpire().getGroundUnits().size(); i++)
		{
			Unit gUnit = (Unit) player.getEmpire().getGroundUnits().elementAt(i);

			if (!gUnit.isEmpty() && !gUnit.getLoaded() && gUnit.getLocation().getPositionX() == location.getPositionX() && gUnit.getLocation().getPositionY() == location.getPositionY())
			{
				for (int j = 0; j < gUnit.getForces().size(); j++)
				{
					GroundForce gf = (GroundForce) gUnit.getForces().elementAt(j);

					if (gf.getMobile())
					{
						groundForces.addElement(gf);
					}
					else
					{
						tokenGuard = true;
					}
				}
			}
		}

		Collections.sort(groundForces);

		// Leave behind at least a token force at this location.
		if (!tokenGuard && groundForces.size() > 0)
		{
			groundForces.removeElementAt(groundForces.size()-1);
		}

		return groundForces;
	}


	// Removes a specific space force at a specific location.  Find the unit to which it formerly belonged and remove it.
	public void removeSpaceForce(SpaceForce sf, Location location)
	{
		for (int i = 0; i < player.getEmpire().getSpaceUnits().size(); i++)
		{
			Unit sUnit = (Unit) player.getEmpire().getSpaceUnits().elementAt(i);

			if (!sUnit.isEmpty() && sUnit.getLocation().getPositionX() == location.getPositionX() && sUnit.getLocation().getPositionY() == location.getPositionY())
			{
				if (sUnit.getForces().contains(sf))
				{
					if (sUnit.removeForce(sf))
					{
						break;
					}
				}
			}
		}
	}

	// Removes a specific ground force at a specific location.  Find the unit to which it formerly belonged and remove it.
	public void removeGroundForce(GroundForce gf, Location location)
	{
		for (int i = 0; i < player.getEmpire().getGroundUnits().size(); i++)
		{
			Unit gUnit = (Unit) player.getEmpire().getGroundUnits().elementAt(i);

			if (!gUnit.isEmpty() && gUnit.getLocation().getPositionX() == location.getPositionX() && gUnit.getLocation().getPositionY() == location.getPositionY())
			{
				if (gUnit.getForces().contains(gf))
				{
					if (gUnit.removeForce(gf))
					{
						break;
					}
				}
			}
		}
	}

	// Use the matter transporter to beam ground troops onto a transport within striking distance
	// of the current target.
	public void useMatterTransporter(int groundAttackTotal, int groundArmorTotal, int possibleGroundAttack, int possibleGroundArmor, PrintStream ps)
	{
		if (matterTransporter() && targetLocation != null)
		{
			Vector<Planet> offensivePlanets = new Vector<Planet>();

			for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
			{
				Planet plan = (Planet) player.getEmpire().getPlanets().elementAt(i);

				AIPlanetIntelligence intel = intelligenceFile.get(plan.getName());

				if (intel.mood != intel.DEFENSIVE || (targetLocation.getPlanets().size() > 0 && intel.isSoleAggressor(((Planet)targetLocation.getPlanets().elementAt(0)).getOwner())))
				{
					offensivePlanets.addElement(plan);
				}
			}

			for (int i = 0; i < aiUnits.size(); i++)
			{
				if (offensivePlanets.size() > 0)
				{
					AIUnit aiUnit = aiUnits.elementAt(i);
					aiUnit.unit.calculateMinMove();

					if (aiUnit.destination.getPositionX() == targetLocation.getPositionX() && aiUnit.destination.getPositionY() == targetLocation.getPositionY())
					{
						if (map.distance(aiUnit.unit.getLocation(), targetLocation) <= aiUnit.unit.getMinMove())
						{
							for (int j = 0; j < aiUnit.unit.getForces().size(); j++)
							{
								SpaceForce sf = (SpaceForce) aiUnit.unit.getForces().elementAt(j);

								while (offensivePlanets.size() > 0 && sf.getStorageCapability() > 0)
								{
									boolean moreGround = loadGroundInto(sf, offensivePlanets.elementAt(0).getLocation(), groundAttackTotal, groundArmorTotal, possibleGroundAttack, possibleGroundArmor);

									if (!moreGround)
									{
										offensivePlanets.removeElementAt(0);
									}

									for (int z = 0; z < sf.getStorage().size(); z++)
									{
										Unit gUnit = (Unit) sf.getStorage().elementAt(z);

										for (int t = 0; t < gUnit.getForces().size(); t++)
										{
											GroundForce gf = (GroundForce) gUnit.getForces().elementAt(t);
											groundAttackTotal += gf.getMaxAttack();
											groundArmorTotal += gf.getArmor();

											//System.out.println(player.getName() + " has beamed " + gf.getName() + " using matter transporters into " + sf);
											ps.println(player.getName() + " has beamed " + gf.getName() + " using matter transporters into " + sf);
										}

										gUnit.getLocation().removeUnit(gUnit);
										gUnit.setLocation(aiUnit.unit.getLocation());
										gUnit.getLocation().addUnit(gUnit);
									}

									if ((groundAttackTotal+0.0)/possibleGroundAttack > currentAttackDefenseRatio || (groundArmorTotal+0.0)/possibleGroundArmor > currentAttackDefenseRatio)
									{
										return;
									}
								}
							}
						}
					}
				}
			}
		}
	}


	public boolean loadGroundInto(SpaceForce sf, Location location, int groundAttackTotal, int groundArmorTotal, int possibleGroundAttack, int possibleGroundArmor)
	{
		boolean tokenGuard = false;
		int capacity = sf.getStorageCapability();

		Vector<GroundForce> groundForces = new Vector<GroundForce>();

		Unit groundAttack = new Unit(player.getName() + " Ground Attack " + player.getNewUnitCounter(), player.getName());

		for (int i = 0;  i < player.getEmpire().getGroundUnits().size(); i++)//location.getUnits().size(); i++)
		{
			Unit unit = (Unit) player.getEmpire().getGroundUnits().elementAt(i);//location.getUnits().elementAt(i);

			//System.out.println("Checking unit " + unit);

			if (unit.getOwner().equals(player.getName()) && !unit.isEmpty() && !unit.isSpace() && !unit.getLoaded() && unit.getLocation().getPositionX() == location.getPositionX() && unit.getLocation().getPositionY() == location.getPositionY())
			{
				for (int j = 0; j < unit.getForces().size(); j++)
				{

					GroundForce gf = (GroundForce) unit.getForces().elementAt(j);

					//System.out.println("Potential to load " + gf + " into " + sf);

					if (gf.getMobile())
					{
						groundForces.addElement(gf);
					}
					else
					{
						tokenGuard = true;
					}
				}
			}
		}

		Collections.sort(groundForces);

		if (!tokenGuard && groundForces.size() > 0)
		{
			groundForces.removeElementAt(groundForces.size()-1);
		}

		for (int i = 0; i < groundForces.size() && groundAttack.getForces().size() < capacity; i++)
		{
			if ((groundAttackTotal+0.0)/possibleGroundAttack < currentAttackDefenseRatio && (groundArmorTotal+0.0)/possibleGroundArmor < currentAttackDefenseRatio)
			{
				groundAttack.addForce(groundForces.elementAt(i));

				for (int z = 0; z < player.getEmpire().getGroundUnits().size(); z++)//location.getUnits().size(); z++)
				{
					Unit unit = (Unit) player.getEmpire().getGroundUnits().elementAt(z);//location.getUnits().elementAt(z);

					if (unit.getOwner().equals(player.getName()) && unit.getLocation().getPositionX() == location.getPositionX() && unit.getLocation().getPositionY() == location.getPositionY())
					{
						unit.removeForce(groundForces.elementAt(i));//getForces().remove(groundForces.elementAt(i));
					}
				}

				groundAttackTotal += groundForces.elementAt(i).getMaxAttack();
				groundArmorTotal += groundForces.elementAt(i).getArmor();
			}
		}


		if (groundAttack.getForces().size() <= capacity && !groundAttack.isEmpty())
		{
			player.getEmpire().addGroundUnit(groundAttack);

			groundAttack.setLoaded(true);
			sf.getStorage().addElement(groundAttack);

			groundAttack.setLocation(location);
			location.addUnit(groundAttack);

			//System.out.println(aiUnits.elementAt(0).unit.getOwner() + " loading " +groundAttack.getName() + " into " + sf);
		}

		if (groundAttack.getForces().size() < capacity)
		{
			return false;
		}

		return true;
	}




	// Builds a list of targetPlanets, sorted by their priority, which can then be used as a basis on which to make attack and movement
	// decisions.
	public Vector<TargetPlanet> buildTargetList(PrintStream ps)
	{
		Vector players = map.getPlayers();
		Vector<TargetPlanet> targetPlanets = new Vector<TargetPlanet>();

		// Calculate priorities for human targets.
		for (int i = 0; i < players.size(); i++)
		{
			Player currentPlayer = (Player) players.elementAt(i);

			if (!currentPlayer.getName().equals(player.getName()))
			{
				for (int j = 0; j < currentPlayer.getEmpire().getPlanets().size(); j++)
				{
					Planet planet = (Planet) currentPlayer.getEmpire().getPlanets().elementAt(j);

					Vector<Planet> planets = new Vector<Planet>();

					for (int t = 0; t < player.getEmpire().getPlanets().size(); t++)
					{
						Planet plan = (Planet) player.getEmpire().getPlanets().elementAt(t);

						AIPlanetIntelligence intel = intelligenceFile.get(plan.getName());

						if (intel.mood != intel.DEFENSIVE || intel.isSoleAggressor(planet.getOwner()))
						{
							planets.addElement(plan);
						}
					}

					if (planets.size() > 0)
					{
						targetPlanets.addElement(scoreTargetPlanet(planet, planets));
					}
				}
			}
		}

		Vector<Planet> planets = new Vector<Planet>();

		for (int t = 0; t < player.getEmpire().getPlanets().size(); t++)
		{
			Planet plan = (Planet) player.getEmpire().getPlanets().elementAt(t);

			AIPlanetIntelligence intel = intelligenceFile.get(plan.getName());

			if (intel.mood != intel.DEFENSIVE)
			{
				planets.addElement(plan);
			}
		}

		if (planets.size() > 0)
		{
			// Calculate the priorities for neutral targets.
			for (int i = 0; i < neutralEmpires.size(); i++)
			{
				Empire empire = (Empire) neutralEmpires.elementAt(i);

				for (int j = 0; j  < empire.getPlanets().size(); j++)
				{
					Planet planet = (Planet) empire.getPlanets().elementAt(j);
					targetPlanets.addElement(scoreTargetPlanet(planet, planets));
				}
			}
		}

		Collections.sort(targetPlanets);

		//System.out.println("Target List: " + targetPlanets);
		ps.println("Target List: " + targetPlanets);

		return targetPlanets;
	}



	// Score a TargetPlanet based on heuristics such as proximity, resource production, probable defenses, and the
	// nature of the empire controlling it.  Return the result as a TargetPlanet.
	public TargetPlanet scoreTargetPlanet(Planet planet, Vector<Planet> planets)
	{
		int score = random.nextInt(70);

		//score += map.distance(findMinimumDistance(planet.getLocation(), planets).getLocation(), planet.getLocation()) * 70;

		Planet closestPlanet = findMinimumDistance(planet.getLocation(), planets);

		int distance = map.distance(closestPlanet.getLocation(), planet.getLocation());

		// Modifications for the asteroid belt.
		if ((closestPlanet.getName().equals("Sphere") && planet.getName().equals("Terroria")) || (closestPlanet.getName().equals("Terroria") && planet.getName().equals("Sphere")) )
		{
			distance = 3;
		}
		else if ((closestPlanet.getName().equals("Loric") && planet.getName().equals("Terroria")) || (closestPlanet.getName().equals("Terroria") && planet.getName().equals("Loric"))  )
		{
			distance = 4;
		}

		score += distance * 70;

		score -= planet.getFood() * currentFoodWeight;
		score -= planet.getGas() * currentGasWeight;
		score -= planet.getMinerals() * currentMineralWeight;

		AIPlanetIntelligence intel = intelligenceFile.get(planet.getName());

		score -= intel.planet.getEconomy() * currentEconomyWeight;
		score -= intel.planet.getIndustry() * currentIndustryWeight;

		score -= intel.aggression * 30;

		if (!planet.getOwner().equals("Neutral"))
		{
			score += (map.getTurn() - intel.reliability) * 10;
		}

		for (int i = 0; i < intel.units.size(); i++)
		{
			score += (intel.units.elementAt(i).getTotalMaxAttack() * 3);
			score += (intel.units.elementAt(i).getTotalArmor() * 3);
		}

		if (planet.getName().equals("Tortalus"))
		{
			int techMod = 50 - player.getEmpire().getTech();

			if (techMod < 0)
			{
				techMod = 0;
			}

			score -= techMod * currentTechWeight;
		}

		// Take into account how many VP the player has.
		for (int i = 0; i < map.getPlayers().size(); i++)
		{
			Player p = (Player) map.getPlayers().elementAt(i);

			if (p.getName().equals(planet.getOwner()))
			{
				double vp = intelligenceFile.getVictoryPoints(p, map);

				//System.out.println(planet + " VP: " + vp);

				if (vp < 0.5)
				{
					vp = vp * vp;
				}

				score -= vp * 200;

				break;
			}
		}

		TargetPlanet tp = new TargetPlanet(planet, score);
		return tp;
	}

	// Finds the minimum distance from a set of planets to a chosen location.
	public Planet findMinimumDistance(Location targetLocation, Vector planets)
	{
		int minDistance = 10000;
		Planet closestPlanet = null;

		for (int i = 0; i < planets.size(); i++)
		{
			Planet planet = (Planet) planets.elementAt(i);

			int distance = map.distance(planet.getLocation(), targetLocation);

			if (distance < minDistance)
			{
				minDistance = distance;
				closestPlanet = planet;
			}
		}
		return closestPlanet;
	}

	// Finds the minimum distance from a set of units to a chosen location.
	public Unit findClosestUnit(Location targetLocation, Vector<Unit> spaceUnits)
	{
		int minDistance = 10000;
		Unit closestUnit = null;

		for (int i = 0; i < spaceUnits.size(); i++)
		{
			Unit sUnit = (Unit) spaceUnits.elementAt(i);

			int distance = map.distance(targetLocation, sUnit.getLocation());

			if (distance < minDistance)
			{
				minDistance = distance;
				closestUnit = sUnit;

			}
		}
		return closestUnit;
	}


	// Returns a boolean indicating whether this empire has achieved matter transporter technology.
	public boolean matterTransporter()
	{
		if (player.getEmpire().getTech() >= 50)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}
