import java.io.*;
import java.util.*;

//The unchecked warnings come from any method which takes any Vector
@SuppressWarnings("unchecked")

class PaxImprovementTree
{

	Player player;
	public PaxImprovementTreeNode root;
	public PaxProductionTree productionTree;

	Vector<PaxImprovementTreeNode> leaves;

	int threadCount;

	public int nodeCount;

	final int maxNodeCount = 5000;
	int industryReduction;

	boolean tortalus = false;

	PrintStream ps;

	public PaxImprovementTree(Player currentPlayer, PaxProductionTree prodTree, int threads, PrintStream stream)
	{
		player = currentPlayer;
		productionTree = prodTree;
		root = null;

		threadCount = threads;

		industryReduction = 0;

		ps = stream;

		do
		{
			runImprovement();
			industryReduction++;
		} while (nodeCount > maxNodeCount && (industryReduction != player.getEmpire().getPlanets().size()));

		findBestProductionNodeConcurrent();
	}

	public void runImprovement()
	{
		leaves = new Vector<PaxImprovementTreeNode>();
		nodeCount = 0;
		Random random = new Random(System.currentTimeMillis());

		tortalus = false;

		for (int i = 0; i < player.getEmpire().getPlanets().size(); i++)
		{
			Planet p = (Planet) player.getEmpire().getPlanets().elementAt(i);

			if (p.getName().equals("Tortalus") && !p.getCaptured())
			{
				tortalus = true;
			}
		}

		Vector<String> possImps = new Vector<String>();

		possImps.addElement("tech");
		possImps.addElement("espionage");
		//possImps.addElement("counterEspionage");

		if (player.getEmpire().getMoney() <= 70)
		{
			possImps.addElement("economy");
		}

		int industryCount;
		for (industryCount = 0; industryCount < player.getEmpire().getPlanets().size(); industryCount++)
		{
			possImps.addElement("industry " + (industryCount+1));
		}

		int offset = possImps.size() - industryCount;

		for (int z = 0; z < industryReduction; z++)
		{
			int selection = random.nextInt(industryCount) + offset;

			possImps.removeElementAt(selection);

			industryCount--;
		}

		startImprovementTree(possImps);

		for (int i = 0; i < leaves.size(); i++)
		{
			if (leaves.elementAt(i) != root)
			{
				leaves.elementAt(i).parent = root;
				root.children.add(leaves.elementAt(i));
				nodeCount++;
			}
		}

		//System.out.println("Improvement Tree Nodes: "  + nodeCount);
		ps.println("Improvement Tree Nodes: "  + nodeCount);
	}

	public void startImprovementTree(Vector possibleImps)
	{
		root = new PaxImprovementTreeNode(new PlayerImprovementData(player));

		buildImprovementTree(root, possibleImps, 0);
	}

	public void buildImprovementTree(PaxImprovementTreeNode node, Vector possibleImps, int impIndex)
	{
		PaxImprovementTreeNode child;
		PlayerImprovementData childPlayer;
		String tempImp;
		int cost = 5;
		int curMoney = node.playerState.money;

		Random random = new Random(System.currentTimeMillis());

		for(int i=impIndex; i<possibleImps.size(); i++)
		{
			tempImp = (String) possibleImps.elementAt(i);

			if(tempImp.equals("economy"))
			{
				cost = 3;
			}
			else
			{
				cost = 5;
			}

			if(curMoney >= cost)
			{
				childPlayer = node.playerState.clone();
				childPlayer.money = curMoney - cost;

				if(tempImp.equals("tech"))
				{
					childPlayer.tech = childPlayer.tech + 1;

					if (tortalus)
					{
						childPlayer.tech = childPlayer.tech + 1;
					}
				}
				if(tempImp.equals("espionage"))
				{
					childPlayer.espionage = childPlayer.espionage + 1;
				}
				if(tempImp.equals("counterEspionage"))
				{
					//childPlayer.getEmpire().setCounterEspionage(childPlayer.getEmpire().getCounterEspionage()+1);
				}
				if(tempImp.equals("economy"))
				{
					int z = random.nextInt(player.getEmpire().getPlanets().size());

					childPlayer.economies.setElementAt(childPlayer.economies.elementAt(z)+1,z);
				}
				if(tempImp.startsWith("industry"))
				{
					int planetIndex = Integer.parseInt(tempImp.substring(tempImp.indexOf(" ")+1, tempImp.length()));
					planetIndex--;

					childPlayer.industries.setElementAt(childPlayer.industries.elementAt(planetIndex)+1,planetIndex);
				}

				child = new PaxImprovementTreeNode(node, childPlayer);
				node.children.addElement(child);
				buildImprovementTree(child, possibleImps, i);


			}
		}

		if (node.children.size() == 0)
		{
			leaves.addElement(node);
		}

	}


	public void findBestProductionNodeConcurrent()
	{
		Vector<Thread> threads = new Vector();
		Vector<PaxImprovementTreeWorker> workers = new Vector();
		//threadCount = 1;

		for (int i = 1; i < threadCount; i++)
		{
			PaxImprovementTreeWorker worker = new PaxImprovementTreeWorker(this, leaves, i, threadCount);
			workers.addElement(worker);

			Thread thread = new Thread(worker);
			thread.start();
			threads.addElement(thread);

		}


		for (int i = 0; i < leaves.size(); i+=threadCount)
		{
			PaxImprovementTreeNode workNode  = leaves.elementAt(i);

			findBestProductionNode(workNode);
		}


		try
		{
			for (int i = 0; i < threads.size(); i++)
			{
				threads.elementAt(i).join();
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

	}

	public void findBestProductionNode(PaxImprovementTreeNode node)
	{
		PaxProductionTreeNode bestNode = productionTree.bestProductionNode;

		if (((Planet)player.getEmpire().getPlanets().elementAt(0)).getIndustry() < node.playerState.industries.elementAt(0) || player.getEmpire().getTech()/5 < node.playerState.tech/5 || ((Planet)player.getEmpire().getPlanets().elementAt(0)).getEconomy() < node.playerState.economies.elementAt(0))
		{
			Player currentPlayer = player.clone();

			node.playerState.playerImprovementDataToPlayer(currentPlayer);

			bestNode = productionTree.traverseProductionTree(currentPlayer, productionTree.productionRoot);
		}
		node.bestProductionNode = bestNode;
	}


	public PaxImprovementTreeNode traverseImprovementTree(PaxImprovementTreeNode currentNode, PaxImprovementTreeNode bestNode)
	{
		if (currentNode.score > bestNode.score)
		{
			bestNode = currentNode;
		}

		for (int i = 0; i < currentNode.children.size(); i++)
		{
			bestNode = traverseImprovementTree(currentNode.children.elementAt(i), bestNode);
		}

		return bestNode;
	}


}
