import java.io.*;
import java.util.*;

//The unchecked warnings come from any method which takes any Vector
@SuppressWarnings("unchecked")

class PaxProductionTree
{
	public PaxProductionTreeNode productionRoot;

	public Player player;
	public Player potentialPlayer;
	public int nodeCount;

	Vector totalSpaceForces;
	Vector totalGroundForces;

	public PaxProductionTreeNode bestProductionNode;
	Vector<PaxProductionTreeNode> bestProductionFrontier;

	Map map;

	Random random;

	public int beamReduction;
	final int maxNodeCount = 40000;

	PrintStream ps;

	public PaxProductionTree(Player play, Vector spaceForces, Vector groundForces, Map ma, int beamReduce, PrintStream stream)
	{
		player = play;
		totalSpaceForces = spaceForces;
		totalGroundForces = groundForces;

		random = new Random(System.currentTimeMillis());

		map = ma;

		beamReduction = beamReduce;

		ps = stream;

		//System.out.println("Beam reduction: " + beamReduction);
		ps.println("Beam reduction: " + beamReduction);

		do
		{
			bestProductionFrontier = new Vector<PaxProductionTreeNode>();
			runProduction();
			beamReduction++;

		} while (nodeCount > maxNodeCount);

		if (beamReduction > 0)
		{
			beamReduction--;
		}
	}


	public void runProduction()
	{
		Vector possForce = new Vector();
		Force temp;

		potentialPlayer = player.clone();

		int money = potentialPlayer.getEmpire().getMoney();
		int eco = ((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).getEconomy();
		int industry = ((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).getCurrentIndustry();
		int tech = player.getEmpire().getTech();

		potentialPlayer.getEmpire().setTech(potentialPlayer.getEmpire().getTech() + money/5);
		((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).setEconomy(((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).getEconomy() + money/3);
		((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).setCurrentIndustry(((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).getCurrentIndustry() + money/5);
		((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).setIndustry(((Planet)potentialPlayer.getEmpire().getPlanets().elementAt(0)).getIndustry() + money/5);


		// Build a list of all possible forces.
		for(int i=0; i<totalSpaceForces.size(); i++)
		{
			temp = (Force) totalSpaceForces.elementAt(i);
			if(temp.getTech() <= potentialPlayer.getEmpire().getTech())
			{
				if(temp.getExclusive().length() == 0 || temp.getExclusive().equals(player.getEmpire().getRace()))
				{
					possForce.addElement(temp);
				}
			}
		}

		for(int i=0; i<totalGroundForces.size(); i++)
		{
			temp = (Force) totalGroundForces.elementAt(i);
			if(temp.getTech() <= potentialPlayer.getEmpire().getTech())
			{
				if(temp.getExclusive().length() == 0 || temp.getExclusive().equals(player.getEmpire().getRace()))
				{
					possForce.addElement(temp);
				}
			}
		}

		startProductionTree(potentialPlayer, possForce);
	}

	public void startProductionTree(Player potentialPlayer, Vector possibleForces)
	{
		nodeCount = 0;

		productionRoot = new PaxProductionTreeNode();

		buildProductionTree(productionRoot, possibleForces, 0, 0, 0, 0, 0, 0, 0);

		//System.out.println("Production Tree Node Count: " + nodeCount);
		ps.println("Production Tree Node Count: " + nodeCount);
	}

	public void buildProductionTree(PaxProductionTreeNode node, Vector possibleForces, int forceIndex, int planetIndex, int foodUsed, int minUsed, int gasUsed, int ecoUsed, int indUsed)
	{
		PaxProductionTreeNode child;
		Force tempForce;

		if (nodeCount > maxNodeCount)
		{
			return;
		}

		Vector<Integer> beam = beamSearch(possibleForces, possibleForces.size()-beamReduction);
		//System.out.println(beam);

		for (int j = planetIndex; j < player.getEmpire().getPlanets().size(); j++)
		{
			for(int i=forceIndex; i<possibleForces.size(); i++)
			{
				if (beam.contains(i))
				{
					tempForce = (Force)((Force) possibleForces.elementAt(i)).clone();

					if(testIndustry((indUsed + tempForce.getIndustry()), j) &&
							testResources((foodUsed + tempForce.getFood()), (minUsed + tempForce.getMinerals()), (gasUsed + tempForce.getGas())) &&
								testEconomy((ecoUsed + tempForce.getMaintenance())) && (((Planet)player.getEmpire().getPlanets().elementAt(j)).isSpaceClear(player, map.getPlayers()) || tempForce.getClass().getName().equals("GroundForce")))
					{
						child = new PaxProductionTreeNode(node, tempForce, j);
						node.children.addElement(child);

						buildProductionTree(child, possibleForces, i, j,
							(foodUsed + tempForce.getFood()), (minUsed + tempForce.getMinerals()), (gasUsed + tempForce.getGas()),
								(ecoUsed + tempForce.getMaintenance()), (indUsed + tempForce.getIndustry()));

						if (nodeCount > maxNodeCount)
						{
							return;
						}

						nodeCount++;
					}
				}
			}
			forceIndex = 0;
			indUsed = 0;
		}

	}


	public Vector<Integer> beamSearch(Vector possibleForces, int number)
	{
		Vector<Integer> beam = new Vector<Integer>();

		for (int i = 0; i < possibleForces.size(); i++)
		{
			beam.addElement(i);
		}


		while (beam.size() > number)
		{
			int selection = random.nextInt(beam.size());

			beam.removeElementAt(selection);
		}

		return beam;
	}

	public void findBestProductionNode()
	{
		bestProductionFrontier = new Vector<PaxProductionTreeNode>();
		bestProductionNode = productionRoot;

		Vector<PaxProductionTreeNode> work = new Vector<PaxProductionTreeNode>();

		work.addElement(productionRoot);

		while (!work.isEmpty())
		{
			PaxProductionTreeNode node = work.elementAt(0);
			work.removeElementAt(0);

			if (node.score >= bestProductionNode.score)
			{
				bestProductionNode = node;
			}

			for (int i = 0; i < node.children.size(); i++)
			{
				PaxProductionTreeNode child = node.children.elementAt(i);


				// Check the state's industry expenditure against the current player's available industry.
				//int industryUsage = ((Planet)player.getEmpire().getPlanets().elementAt(0)).getIndustry();
				Vector<Integer> industryUsage = new Vector<Integer>();
				for (int z = 0; z < player.getEmpire().getPlanets().size(); z++)
				{
					industryUsage.addElement(((Planet)player.getEmpire().getPlanets().elementAt(z)).getIndustry());
				}

				int techRequired = 0;
				int maintenanceDelta = player.getEmpire().getIncome();

				for(int j=0; j<child.forcesBuilt.size(); j++)
				{
					Force tempForce = child.forcesBuilt.elementAt(j);

					int planetIndex = child.planetBuilt.elementAt(j);

					industryUsage.setElementAt(industryUsage.elementAt(planetIndex) - tempForce.getIndustry(), planetIndex);

					if (tempForce.getTech() > techRequired)
					{
						techRequired = tempForce.getTech();
					}

					maintenanceDelta -= tempForce.getMaintenance();
				}

				boolean sufficientIndustry = true;

				for (int z = 0; z < industryUsage.size(); z++)
				{
					if (industryUsage.elementAt(z) < 0)
					{
						sufficientIndustry = false;
					}
				}

				if (sufficientIndustry && player.getEmpire().getTech() >= techRequired && maintenanceDelta >= 0)
				{
					work.addElement(child);
				}
				else
				{
					bestProductionFrontier.addElement(child);
				}
			}
		}
	}



	public PaxProductionTreeNode traverseProductionTree(Player player, PaxProductionTreeNode root)
	{
		PaxProductionTreeNode bestProductionNode = this.bestProductionNode;//root;

		Vector<PaxProductionTreeNode> work = new Vector<PaxProductionTreeNode>();

		for (int i = 0; i < bestProductionFrontier.size(); i++)
		{
			work.addElement(bestProductionFrontier.elementAt(i));
		}

		while (!work.isEmpty())
		{
			PaxProductionTreeNode node = work.elementAt(0);
			work.removeElementAt(0);


			// Check the state's industry expenditure against the current player's available industry.
			//int industryUsage = ((Planet)player.getEmpire().getPlanets().elementAt(0)).getIndustry();
			Vector<Integer> industryUsage = new Vector<Integer>();
			for (int z = 0; z < player.getEmpire().getPlanets().size(); z++)
			{
				industryUsage.addElement(((Planet)player.getEmpire().getPlanets().elementAt(z)).getIndustry());
			}


			int techRequired = 0;
			int maintenanceDelta = player.getEmpire().getIncome();

			for(int j=0; j<node.forcesBuilt.size(); j++)
			{
				Force tempForce = node.forcesBuilt.elementAt(j);

				//industryUsage -= tempForce.getIndustry();
				int planetIndex = node.planetBuilt.elementAt(j);

				industryUsage.setElementAt(industryUsage.elementAt(planetIndex) - tempForce.getIndustry(), planetIndex);

				if (tempForce.getTech() > techRequired)
				{
					techRequired = tempForce.getTech();
				}

				maintenanceDelta -= tempForce.getMaintenance();
			}

			boolean sufficientIndustry = true;

			for (int z = 0; z < industryUsage.size(); z++)
			{
				if (industryUsage.elementAt(z) < 0)
				{
					sufficientIndustry = false;
				}
			}

			if (sufficientIndustry && player.getEmpire().getTech() >= techRequired && maintenanceDelta >= 0)
			{
				if (node.score >= bestProductionNode.score)
				{
					bestProductionNode = node;
				}

				for (int i = 0; i < node.children.size(); i++)
				{
					PaxProductionTreeNode child = node.children.elementAt(i);

					/*
					// Check the state's industry expenditure against the current player's available industry.
					int industryUsage = ((Planet)player.getEmpire().getPlanets().elementAt(0)).getIndustry();
					int techRequired = 0;
					int maintenanceDelta = player.getEmpire().getIncome();

					for(int j=0; j<child.forcesBuilt.size(); j++)
					{
						Force tempForce = child.forcesBuilt.elementAt(j);

						industryUsage -= tempForce.getIndustry();

						if (tempForce.getTech() > techRequired)
						{
							techRequired = tempForce.getTech();
						}

						maintenanceDelta -= tempForce.getMaintenance();
					}

					if (industryUsage >= 0 && player.getEmpire().getTech() >= techRequired && maintenanceDelta >= 0)
					{*/
						work.addElement(child);
					//}
				}
			}

		}

		return bestProductionNode;

	}



	/* This set of functions test "vital statistics" of the player to ensure things
	 * are either buildable or buyable
	 */

	public boolean testIndustry(int industry, int planetNum)
	{
		if(((Planet) potentialPlayer.getEmpire().getPlanets().elementAt(planetNum)).getCurrentIndustry() - industry >= 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public boolean testEconomy(int maint)
	{
		if(potentialPlayer.getEmpire().getIncome() - maint >= 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public boolean testResources(int food, int minerals, int gas)
	{
		if(food != 0)
		{
			if(potentialPlayer.getEmpire().getFood() - food < 0)
			{
				return false;
			}
		}

		if(minerals != 0)
		{
			if(potentialPlayer.getEmpire().getMinerals() - minerals < 0)
			{
				return false;
			}
		}

		if(gas != 0)
		{
			if(potentialPlayer.getEmpire().getGas() - gas < 0)
			{
				return false;
			}
		}

		return true;
	}



	/*~~~~~~~~~~~~~~~~~~~~~End to vital Statistics functions~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

}
