import java.io.*;
import java.util.*;

//The unchecked warnings come from any method which takes any Vector
@SuppressWarnings("unchecked")

class PaxProductionTreeNode
{
	public Vector<Force> forcesBuilt;
	public int score;
	public Vector<PaxProductionTreeNode> children;
	public PaxProductionTreeNode parent; //At root, parent = NULL

	// A Vector that contains the identifiers for where the corresponding forces were built.
	public Vector<Integer> planetBuilt;

	//Creates root node
	public PaxProductionTreeNode ()
	{
		parent = null;
		children = new Vector<PaxProductionTreeNode>();

		forcesBuilt = new Vector<Force>();
		planetBuilt = new Vector<Integer>();
		score = -1;
	}

	//Creates child node
	public PaxProductionTreeNode (PaxProductionTreeNode parentalUnit, Force newForce, int planetID)
	{
		parent = parentalUnit;
		children = new Vector<PaxProductionTreeNode>();

		forcesBuilt = propogateForce();
		forcesBuilt.addElement(newForce);
		score = -1;
		planetBuilt = propogatePlanetBuilt();
		planetBuilt.addElement(new Integer(planetID));
	}

	private Vector<Force> propogateForce ()
	{
		Vector<Force> clonedForces = new Vector<Force>();

		for(int i = 0; i < parent.forcesBuilt.size(); i++)
		{
			clonedForces.addElement(((Force)parent.forcesBuilt.elementAt(i).clone()));
		}

		return clonedForces;
	}

	private Vector<Integer> propogatePlanetBuilt ()
	{
		Vector<Integer> clonedPlanetBuilt = new Vector<Integer>();

		for (int i = 0; i < parent.planetBuilt.size(); i++)
		{
			clonedPlanetBuilt.addElement(new Integer(parent.planetBuilt.elementAt(i)));
		}

		return clonedPlanetBuilt;
	}




}