import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import java.io.*;

// Name: Jeremy Long
// Student Number: 587 702
// NSID: jsl847
// CMPT436, Project


public class PaxServer extends UnicastRemoteObject implements PaxServerInterface
{
	protected float version;

	protected Map map;
	protected int totalPlayers;
	protected Vector players;
	protected Vector empires; // list of empires - contains only neutrals after players have chosen.
	protected Vector spaceForces;
	protected Vector groundForces;
	protected int progress[];
	protected Vector messages;
	protected Vector sbattles;
	protected Vector gbattles;
	protected int battleCount;

	protected Vector spaceBattles; // a Vector of space battles;
	protected Vector groundBattles; // a Vector of ground battles;
	protected boolean spaceFighting = false; // indicates that space battles are still occuring.
	protected boolean groundFighting = false; // indicates that ground battles are still occuring.
	protected boolean setUp = false; // we are not set up for battles.
	protected int battleReady = 0; // 0 indicates that we need to set up battles, else we're ready.  Reset each turn.
	protected int deploy[]; // similar to progress, so that all players have a chance to defensive redeploy.
	protected int checkPlanetsNumber = 0; // ensures we only do planet checking once per combat phase.

	protected boolean aiMoving = false; // Indicates whether the AIs have started moving.

	protected Vector neutralSpace; // the neutral spaceUnits;
	protected Vector neutralGround; // the neutral groundUnits;

	protected Vector offers; // the offers for market phase, created new each turn.

	protected Vector operations; // a vector of all the offensive operations being performed this round, reset each turn.
	protected Vector defensiveOperations; // a vector of all the defensive operations being performed this round.

	protected Vector tempPlayers;

	protected Vector<PaxGenericAI> aiPlayers;

	// The PrintStream where data from the AIs should be written.
	PrintStream ps;

	Random r;
	boolean firstTime = true;
	Thread aiThread;

	public PaxServer(int totalPlayer, float serverVersion) throws Exception
	{
		super(20000); // port for the remote object.

		totalPlayers = totalPlayer;
		version = serverVersion;

		r = new Random(System.currentTimeMillis());

		players = new Vector();

		map = new Map(players);

		empires = new Vector(); // get results from Brice's XML parser.

		Loader loader = new Loader();
		empires = loader.initEmpires(map);
		spaceForces = loader.loadFromXML("spaceforces.xml");
		groundForces = loader.loadFromXML("groundforces.xml");

		battleCount = 0;

		offers = new Vector();

		ps = new PrintStream(new FileOutputStream(new File("PaxAILog.txt")));

		// Load the game from the AutoSave.
		if (totalPlayers == -1)
		{
			load();
		}
		else
		{
			initializeAIs();
		}

		progress = new int[totalPlayers];
		deploy = new int[totalPlayers];

		for (int i = 0; i < totalPlayers; i++)
		{
			progress[i] = -1;
		}

		// This game consists only of AIs, so play it out without human interaciton/
		if (aiPlayers.size() == totalPlayers)
		{
			ready(0, 0);

			while (!gameOver())
			{

				while (!ready(map.getPhase(), map.getTurn()))
				{
				}

				if (map.getPhase() == 4)
				{
					battleReady = 0;
					setUp = false;
					spaceFighting = false;
					groundFighting = false;


					checkDeploy();
				}
			}
		}
	}

	public void initializeAIs() throws Exception
	{
		int  numberAIs = 0;
		int selection = 0;
		aiPlayers = new Vector<PaxGenericAI>();

		BufferedReader kb = new BufferedReader(new InputStreamReader(System.in));

		while (selection < empires.size() && (numberAIs+totalPlayers) < 12)
		{
			selection = 0;
			System.out.println("Select an AI empire to add, or hit <"+(empires.size()+1)+"> when done adding AIs!");

			for (int j = 0; j < empires.size(); j++)
			{
				System.out.println((j+1) + " - " + empires.elementAt(j));
			}
			System.out.println((empires.size()+1) + " - done adding AIs!");

			while (selection < 1 || selection > (empires.size()+1))
			{
				try
				{
					selection = Integer.parseInt(kb.readLine());

					if (selection < 1 || selection > (empires.size()+1))
					{
						System.out.println("Error - Please enter a valid selection.");
					}

				}
				catch (Exception e)
				{
					System.out.println("Error - Please enter a valid empire index.");
				}
			}

			selection--;

			if (selection < empires.size())
			{
				aiPlayers.addElement(createAI("AI " + ((Empire)empires.elementAt(selection)).getRace(), ((Empire)empires.elementAt(selection)).getRace(), numberAIs, Runtime.getRuntime().availableProcessors()));
				numberAIs++;
			}

		}

		totalPlayers += numberAIs;
	}



	public PaxGenericAI createAI(String name, String race, int index, int processors) throws RemoteException
	{
		connect(name, "localhost");
		chooseEmpire(name, race, race);

		Player player = (Player) players.elementAt(index);

		PaxGenericAI ai = new PaxGenericAI(player, spaceForces, groundForces, processors, map, empires);

		return ai;
	}

	// Returns the version number of the server.
	public float getVersion() throws RemoteException
	{
		return this.version;
	}

	// Saves the state of the current Pax map to a file.  Should be called at the beginning of every
	// turn.
	public void save()
	{
		try
		{

			FileOutputStream fos = new FileOutputStream("PaxAutoSave.tmp");
			ObjectOutputStream output = new ObjectOutputStream(fos);

			// Save the map.
			output.writeObject(map);

			// Save the neutral empires.
			output.writeObject(empires);

			// Save the AIPlayers.
			output.writeObject(aiPlayers);

			output.close();

			System.out.println("State of game after turn " + (map.getTurn()-1) + " has been saved!");
		}
		catch (Exception e)
		{
			System.out.println("\nCan't save the state of the game.\n");
			e.printStackTrace();
		}
	}

	public void load()
	{
		try
		{

			FileInputStream fis = new FileInputStream("PaxAutoSave.tmp");
			ObjectInputStream input = new ObjectInputStream(fis);

			// Load the data.
			map = (Map) input.readObject();

			empires = (Vector) input.readObject();

			aiPlayers = (Vector) input.readObject();

			// Set up variables based on the map.

			players = map.getPlayers();
			totalPlayers = players.size();

			messages = new Vector();

			assignNeutral();

			cleanNeutral();

			input.close();

			System.out.println("State of game after turn " + (map.getTurn()-1) + " has been loaded!");
		}
		catch (Exception e)
		{
			System.out.println("\nCan't load the state of the game.\n");
			e.printStackTrace();
		}
	}

	public boolean isFull() throws RemoteException
	{
		return players.size() >= totalPlayers;
	}

	public boolean connect(String name, String hostName) throws RemoteException
	{
		boolean success = true;

		System.out.println(name + " is attemping to connect to the game of Pax!");

		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			if (p.getName().equals(name) || name.equals("Neutral"))
			{
				success = false;
			}
		}

		if (success)
		{
			Player p = new Player(name);
			p.setHostName(hostName);
			players.addElement(p);

			System.out.println(name + " has joined the "+totalPlayers+" player game of Pax Galaktika!");
		}

		return success;
	}

	// Reloads the current client by confirming name and hostName.
	public boolean reload(String name, String hostName) throws RemoteException
	{
		boolean success = false;

		System.out.println(name + " is attemping to reload the game of Pax!");

		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			if (p.getName().equals(name))// && p.getHostName().equals(hostName))
			{
				success = true;
			}
		}

		return success;
	}

	// Returns available empires - contains only the neutral empires once the game has started.

	public Vector getEmpires()
	{
		return empires;
	}

	// Removes a planet with the name from a neutral empire.

	public synchronized void removeNeutralPlanet(String name)
	{
		//System.out.println("Removing planet: " + name);

		for (int i = 0; i < empires.size(); i++)
		{

			Empire e = (Empire) empires.elementAt(i);

			for (int t = 0; t < e.getPlanets().size(); t++)
			{
				Planet plan = (Planet) e.getPlanets().elementAt(t);

				if (plan.getName().equals(name))
				{
					e.getPlanets().removeElementAt(t);

				}
			}
		}
	}

	public boolean chooseEmpire(String name, String race, String empName)
	{
		boolean success = false;

		System.out.println(name + " tries to create the empire of " + race);

		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			//System.out.println(p.getName());

			if (p.getName().equals(name))
			{
				for (int k = 0; k < empires.size(); k++)
				{
					Empire e = (Empire) empires.elementAt(k);
					//System.out.println("Checking against: " + e.toString());
					if (e.toString().equals(race))
					{
						success = true;
						e.setName(empName);
						p.setEmpire(e);
						empires.removeElement(e);

						/*for (int t = 0; t < e.getPlanets().size(); t++)
						{
							Planet plan = (Planet) e.getPlanets().elementAt(t);
							plan.setOwner(p.toString());
						}*/
						e.setOwner(p.toString());

						Unit newU2 = new Unit("New Unit " + p.getNewUnitCounter(), p.toString());
						e.addSpaceUnit(newU2);

						Unit newU3 = new Unit("New Unit " + p.getNewUnitCounter(), p.toString());
						e.addSpaceUnit(newU3);

						Unit newU = new Unit("New Unit " + p.getNewUnitCounter(), p.toString());
						e.addGroundUnit(newU);

						Unit newU4 = new Unit("New Unit " + p.getNewUnitCounter(), p.toString());
						e.addGroundUnit(newU4);

						System.out.println(name + " has created the empire of " + race);

					}
				}
			}

		}

		return success;
	}

	// Sets all unpicked empires as
	// neutral by adding their units to the neutral force vectors.
	public synchronized void assignNeutral()
	{

		neutralSpace = new Vector();
		neutralGround = new Vector();

		for (int i = 0; i < empires.size(); i++)
		{
			Empire e = (Empire) empires.elementAt(i);

			for (int t = 0; t < e.getSpaceUnits().size(); t++)
			{
				neutralSpace.addElement(e.getSpaceUnits().elementAt(t));
				//System.out.println(e.getSpaceUnits().elementAt(t));
			}
			for (int t = 0; t < e.getGroundUnits().size(); t++)
			{
				neutralGround.addElement(e.getGroundUnits().elementAt(t));
				//System.out.println(e.getGroundUnits().elementAt(t));
			}
		}
	}

	// Removes empty neutral units from the game.
	public void cleanNeutral()
	{
		for (int i = 0; i < neutralSpace.size(); i++)
		{
			Unit u = (Unit) neutralSpace.elementAt(i);

			if (u.getForces().size() == 0)
			{
				neutralSpace.removeElementAt(i);
				i--;
			}
		}
		for (int i = 0; i < neutralGround.size(); i++)
		{
			Unit u = (Unit) neutralGround.elementAt(i);

			if (u.getForces().size() == 0)
			{
				neutralGround.removeElementAt(i);
				i--;
			}
		}

		// Prints out all the neutral forces.
		/*
		for (int i = 0; i < neutralSpace.size(); i++)
		{
			Unit u = (Unit) neutralSpace.elementAt(i);

			System.out.println((i+1) + " - " + u.getName());
		}
		*/
	}

	public boolean isReady(String name)
	{
		boolean ready = false;

		// Find the right player.

		for (int i = 0; i < players.size(); i++)
		{
			Player play = (Player) players.elementAt(i);

			if (play.getName().equals(name))
			{
				if (progress[i] == map.getPhase())
				{
					ready = true;
				}
			}
		}

		return ready;
	}

	public void aiTurn(int phase, int turn)
	{

		if (phase == 2)
		{
			boolean ready = true;

			for (int i = aiPlayers.size(); i < players.size(); i++)
			{
				if (progress[i] != phase)
				{
					ready = false;
				}
			}

			if (ready && !aiMoving)
			{
				aiMoving = true;
				if (aiThread == null)
				{
					aiThread = new Thread(new PaxAIThread(aiPlayers, phase, map, ps));

					try
					{
						aiThread.start();
						return;
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
			}
		}
		else if (phase == 5)
		{
			aiMoving = false;
			for (int i = 0; i < aiPlayers.size(); i++)
			{
				checkPlanets(aiPlayers.elementAt(i).player.getName(), aiPlayers.elementAt(i).player.getEmpire().getPlanets());

				if (checkAIEliminated(aiPlayers.elementAt(i), i))
				{
					i--;
				}
			}
		}

		for (int i = 0; i < aiPlayers.size(); i++)
		{
			if (progress[i] != phase)
			{

				// We will later examine defensive redeploy for AIs.
				deploy[i] = 1;

				if ((phase != 3 || map.getTurn() == aiPlayers.elementAt(i).movementCount) && (phase != 7 || map.getTurn() == aiPlayers.elementAt(i).improvementCount))
				{
					donePhase(aiPlayers.elementAt(i).player.getName(), phase, turn, aiPlayers.elementAt(i).player);
				}

				if (phase == 1)
				{
					aiPlayers.elementAt(i).player.getEmpire().collect();
					aiThread = null;
					acceptOperations(aiPlayers.elementAt(i).runEspionage(ps));
				}
				else if (phase == 4)//3)
				{
					aiThread = null;
				}
				else if (phase == 5)//(phase == 2 || phase == 5)
				{
					if (aiThread == null)
					{
						aiThread = new Thread(new PaxAIThread(aiPlayers, phase, map, ps));

						try
						{
							aiThread.start();
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}
				}
			}
		}
	}

	// Process battles that only involve AIs or neutral players.
	public void processAIBattles(PaxGenericAI aiPlayer, Vector battles)
	{
		//System.out.println("Processing AI Battles for " + aiPlayer.player + ": " + battles.size());

		for (int z = 0; z < battles.size(); z++)
		{

			boolean neutral = false;

			Battle b = (Battle) battles.elementAt(z);

			b = findBattle(b.getLocation());

			//System.out.println("Battle at " + b.getLocation());

			if (b != null && !b.over())
			{
				boolean isAIBattle = true;

				Combatant newC = null;

				for (int j = 0; j < b.getCombatants().size(); j++)
				{
					Combatant c = (Combatant) b.getCombatants().elementAt(j);

					//System.out.println(c + " is a combatant!");

					if (c.getName().equals(aiPlayer.player.getName()))
					{
						newC = c;
					}

					if (c.getName().equals("Neutral"))
					{
						neutral = true;
					}

					if (!isComputerControlled(c))
					{
						isAIBattle = false;
					}
				}

				if (isAIBattle)
				{
					int stillFighting = 2;
					boolean damageDone = true;

					while (stillFighting > 1 && damageDone)
					{
						setReadyForBattle(aiPlayer.player.getName(), b.getLocation(), true, newC);

						stillFighting = 0;  // number of combatants still fighting.
						damageDone = false; // indicates whether damage was done in the last round, if not
											// we end the battle.

						for (int j = 0; j < b.getCombatants().size(); j++)
						{
							Combatant c = (Combatant) b.getCombatants().elementAt(j);

							if (c.getSize() > 0)
							{
								stillFighting++;
							}

							// Checking if anyone did damage in the last round.
							if (c.getDamage() > 0)
							{
								damageDone = true;
							}
						}
					}
					doneBattle(b.getLocation(), b.getSpace());

					if (neutral)
					{
						// Send a message about this neutral space battle.
						Vector temp = new Vector();

						String winner = " The battle was won by " + b.getWinner() + "!";

						if (b.getWinner().length() == 0)
						{
							winner = " No forces survived the battle.";
						}

						String location = "in deep space";

						if (b.getLocation().getPlanets().size() > 0)
						{
							location = "above " + b.getLocation().getPlanets().elementAt(0);
						}


						Message m = new Message("A "+getAdjective(b)+" space battle was fought " + location + " between " + (Combatant)b.getCombatants().elementAt(0) + " and " + (Combatant)b.getCombatants().elementAt(1) + "!" + winner,  true);

						if (!b.getSpace())
						{
							m = new Message("A "+getAdjective(b)+" ground battle was fought on " + (Planet)b.getLocation().getPlanets().elementAt(0) + " between " + (Combatant)b.getCombatants().elementAt(0) + " and " + (Combatant)b.getCombatants().elementAt(1) + "!" + winner,  true);
						}

						temp.addElement(m);
						sendMessages(temp);
						ps.println(m);
					}
				}
			}
		}
	}



	// This method determines if an AI player has been eliminated from the game, in which case they must be removed both
	// from the list of players and from the list of AIs.  This occurs when they have zero planets and space units.
	public boolean checkAIEliminated(PaxGenericAI aiPlayer, int index)
	{
		boolean result = false;

		Player player = aiPlayer.player;

		if (player.getEmpire().getPlanets().size() == 0)
		{
			result = true;
		}

		if (result)
		{
			System.out.println(player + " has been eliminated!");
			ps.println(player + " has been eliminated!");
			try
			{
				aiPlayers.remove(aiPlayer);

				removePlayer(player.getName());
			}
			catch (Exception e)
			{
				System.out.println("Can't get rid of " + player.getName());
			}

		}
		return result;
	}

	public void assignPlanets(Player player)
	{
		for (int t = 0; t <player.getEmpire().getGroundUnits().size(); t++)
		{
			Unit gu = (Unit) player.getEmpire().getGroundUnits().elementAt(t);

			if (!gu.isEmpty() && (!gu.getLoaded() || gu.getGroundInvade()))
			{
				Location l = gu.getLocation();

				for (int k = 0; k < l.getPlanets().size(); k++)
				{
					Planet plan = (Planet) l.getPlanets().elementAt(k);
					//System.out.println("Over planet " + plan);

					// Should we add this planet?
					boolean addPlanet = true;

					for (int z = 0; z < player.getEmpire().getPlanets().size(); z++)
					{
						Planet plan2 = (Planet) player.getEmpire().getPlanets().elementAt(z);
						if (plan.getName().equals(plan2.getName()))
						{
							addPlanet = false;
						}
					}

					gu.setGroundInvade(false);

					if (addPlanet)
					{
						gu.setLoaded(false);

						boolean found = false;

						// Find who had that planet - if we can't it must be neutral, use
						// the one grabbed from that location.

						for (int z = 0; z < map.getPlayers().size(); z++)
						{
							Player p = (Player) map.getPlayers().elementAt(z);

							for (int m = 0; m < p.getEmpire().getPlanets().size(); m++)
							{
								Planet plane = (Planet) p.getEmpire().getPlanets().elementAt(m);
								if (plane.getName().equals(plan.getName()))
								{
									plan = plane;
									m = p.getEmpire().getPlanets().size();
									z = map.getPlayers().size();
									found = true;
								}
							}
						}

						// Must be a neutral planet, find it.

						if (!found)
						{
							//System.out.println("Searching for planet to add from neutral empires!");
							for (int z = 0; z < empires.size(); z++)
							{
								Empire e = (Empire) empires.elementAt(z);

								for (int a = 0; a < e.getPlanets().size(); a++)
								{
									Planet planet = (Planet) e.getPlanets().elementAt(a);
									if (planet.getName().equals(plan.getName()))
									{
										plan = planet;
									}
								}
							}
						}

						// Setting current industry to zero.
						plan.setCurrentIndustry(0);
						plan.setCaptured(true);
						(player.getEmpire().getPlanets()).addElement(plan);
						//System.out.println("Add planet!");

						// Send message regarding planet capture.
						Vector messages = new Vector();
						Message m = new Message("After a vicious ground battle, " + player.getName() + " has wrested control of the planet " + plan.getName() + " from " + plan.getOwner() + "!", true);
						messages.addElement(m);
						sendMessages(messages);

						// Checking if Tech should be increased!
						found = false;
						for (int z = 0; z < map.getPlayers().size(); z++)
						{
							Player p = (Player) map.getPlayers().elementAt(z);

							if (plan.getOwner().equals(p.getName()))
							{
								//System.out.println("Checking tech!");
								found = true;

								if (p.getEmpire().getTech() > player.getEmpire().getTech())
								{
									int newTech = (p.getEmpire().getTech() + player.getEmpire().getTech())/2;
									//System.out.println("Increasing tech to " + newTech);
									player.getEmpire().setTech(newTech);
								}
							}
						}
						// Setting tech for captured neutral planets.
						if (!found)
						{
							//System.out.println("Searching for planet to remove from neutral empires!");

							for (int z = 0; z < empires.size(); z++)
							{
								Empire e = (Empire) empires.elementAt(z);

								for (int a = 0; a < e.getPlanets().size(); a++)
								{
									Planet planet = (Planet) e.getPlanets().elementAt(a);
									if (planet.getName().equals(plan.getName()))
									{
										//System.out.println("Checking tech!");
										if (e.getTech() > player.getEmpire().getTech())
										{
										   int newTech = (e.getTech() + player.getEmpire().getTech())/2;
											//   System.out.println("Increasing tech to " + newTech);
										   player.getEmpire().setTech(newTech);
										}

										removeNeutralPlanet(planet.getName());
									}

								}
							}

						}

						plan.setOwner(player.getName());

						//System.out.println(player.getEmpire().getPlanets());
					}
				}
			}
		}
	}





	public boolean ready(int phase, int turn)
	{
		boolean ready = true;
		int length = progress.length;

		aiTurn(phase, turn);


		if (phase > 0)
		{
			length = players.size();
		}

		//System.out.println("Length: " +length);

		for (int i = 0; i < length; i++)//progress.length; i++)
		{
			//System.out.println("Checking " + ((Player)players.elementAt(i)).getName() + " " i);

			if (progress[i] != phase)
			{
				ready = false;
				//System.out.println(ready);
			}
		}

		if (ready)
		{
			if (phase == 0)
			{
				//System.out.println("Assign neutral units!");
				assignNeutral();

				messages = new Vector();
				operations = new Vector();
				defensiveOperations = new Vector();

				for (int i = 0; i < aiPlayers.size(); i++)
				{
					aiPlayers.elementAt(i).setup();
				}

				//phase = 6;
				//map.setPhase(6);

			}
			if (phase == 1)
			{
				// Prepare deploy for defensive redeploying.

				for (int i =0; i < players.size(); i++)
				{
					// We're not done deploying yet.
					deploy[i] = 0;
				}

			}

			// Pre-combat
			if (phase == 3)
			{
				// Getting battleReady!
				battleReady++;

			    	// Only do this the first time this part is called.
			    	if (battleReady == 1 && !spaceFighting)
				{
					setUpSpaceBattles();
				}
				else
				{

					// Waiting until we're done setting up the battles.
					while (!groundFighting && !setUp)
					{

					}

				}

				offers = new Vector();
				firstTime = true;
				operations = new Vector();
				defensiveOperations = new Vector();

			}

			if (phase == 4)
			{
				if (firstTime)
				{
					firstTime = false;

					for (int i = 0; i < aiPlayers.size(); i++)
					{
						Vector battles = getNeutralSpaceBattles(aiPlayers.elementAt(i).player.getName());

						processAIBattles(aiPlayers.elementAt(i), battles);
					}

					for (int i = 0; i < aiPlayers.size(); i++)
					{
						Vector battles = getNeutralGroundBattles(aiPlayers.elementAt(i).player.getName());

						processAIBattles(aiPlayers.elementAt(i), battles);
					}

					for (int i = 0; i < aiPlayers.size(); i++)
					{
						assignPlanets(aiPlayers.elementAt(i).player);
					}
				}
			}

			// Set forces as committed;
			if (phase == 5)
			{
				committAll();
			}


			if (phase == 7)
			{
				// Removes empty neutral units.
				cleanNeutral();
				map.setTurn(turn + 1);
				map.setPhase(1);
				setUp = false;  // Getting ready to set up space & ground battles.
				battleReady = 0; // we're back to needing to set up battles.
				save();
			}
			else
			{
				map.setPhase(phase+1);
			}
		}

		return ready;
	}

	public void donePhase(String name, int phase, int turn, Player player)
	{
		// Make sure there is no spoofing of the client.
		if (phase == 0 || (phase == map.getPhase() && turn == map.getTurn()))
		{

			System.out.println(name + " is done phase " + phase + ", turn " + turn + "!");
			ps.println(name + " is done phase " + phase + ", turn " + turn + "!");

			for (int i = 0; i < players.size(); i++)
			{
				Player p = (Player) players.elementAt(i);

				if (p.toString().equals(name))
				{
					if (phase > 0)
					{
						// For espionage bombing...
						if (phase != 2 && phase != 3)
						{
							players.setElementAt(player, i);

							p.getEmpire().setMaxEspionage(p.getEmpire().getEspionage());
						}
						else if (phase == 3)
						{
							// Put the player in the tempVector until we're all ready.

							tempPlayers.addElement(player);

							if (tempPlayers.size() == totalPlayers)
							{
								for (int t = 0; t < tempPlayers.size(); t++)
								{
									Player tempP = (Player) tempPlayers.elementAt(t);

									for (int z = 0; z < players.size(); z++)
									{

										Player playing = (Player) players.elementAt(z);
										if (playing.getName().equals(tempP.getName()))
										{
											players.setElementAt(tempP, z);
										}
									}
								}
							}

						}
						// In espionage, we should still allow for creation of units.
						else
						{
							// Set the units.
							p.getEmpire().setSpaceUnits(player.getEmpire().getSpaceUnits());
							p.getEmpire().setGroundUnits(player.getEmpire().getGroundUnits());

							p.getEmpire().setEspionage(p.getEmpire().getMaxEspionage());

							tempPlayers = new Vector();
						}

						if (phase != 2 && phase != 4)
						{
							messages = new Vector();
						}


						if (phase == 2)
						{
							int countPlayers = 0;

							for (int z = 0; z < players.size(); z++)
							{
								if (z != i)
								{
									if (progress[z] == phase)
									{
										countPlayers++;
									}
								}

							}

							// This is the last player to finish, so perform operations.
							if (countPlayers == players.size() - 1)
							{
								performOperations();
							}

						}

						//p.getEmpire().setMaxEspionage(p.getEmpire().getEspionage());
					}
					progress[i] = phase;
				}
			}

		}
	}

	// This method checks if the defensive redeploy stage is done, and returns the result as a boolean.

	public boolean checkDeploy()
	{
		boolean ready = true;
		int length = players.size();

		for (int i = 0; i < length; i++)
		{
			if (deploy[i] != 1)
			{
				ready = false;
			}
		}

		// We are ready to do battles again.
		if (ready)
		{
			// Getting battleReady!
			battleReady++;

		    	// Only do this the first time this part is called.
		    	if (battleReady == 1 && !spaceFighting)
			{
				setUpSpaceBattles();


				for (int i = 0; i < aiPlayers.size(); i++)
				{
					Vector battles = getSpaceBattles(aiPlayers.elementAt(i).player.getName());

					processAIBattles(aiPlayers.elementAt(i), battles);
				}

			}
			else
			{

				// Waiting until we're done setting up the battles.
				while (!groundFighting && !setUp)
				{

				}
			}
		}
		// Not sure if this will really be fast enough for Donna's blazingly fast computer.
		else if (!ready)
		{
			/*
			if (spaceFighting)
			{
				battleReady = 0;
				setUp = false;
				spaceFighting = false;
				groundFighting = false;
			}
			*/
		}

		return ready;
	}

	// This method is called when a player is done defensive redeploying.

	public void doneDeploy(String name, Player player)
	{
		System.out.println(name + " is done defensive redeploy!");

		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			if (p.getName().equals(name))
			{
				players.setElementAt(player, i);
				//System.out.println("Setting player for " + name + " at " + i+ "!");

				// Preparing for the redeploy set up of battles.
				battleReady = 0;
				setUp = false;
				spaceFighting = false;
				groundFighting = false;

				deploy[i] = 1;
			}
		}
	}

	public boolean isDoneDeploy(String name)
	{
		boolean done = false;

		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			if (p.getName().equals(name))
			{
				if (deploy[i] == 1)
				{
					done = true;
				}
			}
		}

		return done;
	}


	public Map getMap()
	{
		//messages = new Vector();
		return map;
	}

	public void sendMessages(Vector message)
	{
		for (int i = 0; i < message.size(); i++)
		{
			Message s = (Message) message.elementAt(i);
			System.out.println(s);
			messages.addElement(s);
		}
	}

	public Vector getMessages(String name)
	{
		Vector results = new Vector();

		for (int i =0; i < messages.size(); i++)
		{
			Message m = (Message) messages.elementAt(i);

			if (m.hasPermission(name))
			{
				results.addElement(m);
			}
		}

		return results;
	}


	public Vector getSpaceForces()
	{
		return spaceForces;
	}

	public Vector getGroundForces()
	{
		return groundForces;
	}

	// A single method that gathers all of the space battles for every location.  This gives a consistant ordering
	// for when players choose to retrieve their space battles.  This fills the vector space battles.

   	public void setUpSpaceBattles()
    	{
	    	spaceFighting = true;
		spaceBattles = new Vector();
		groundBattles = new Vector();
		sbattles = new Vector(); // Used for neutral battles.
		gbattles = new Vector(); // Used for neutral ground battles.

		// One asteroidBattle per round, all units on asteroids included.
		Battle asteroidBattle = new Battle(map.getLocationAt(9, 2), true);
		boolean first = true;

		Vector locations = map.getLocations();

		//System.out.println("Setting up all space battles!");

		for (int i = 0; i < locations.size(); i++)
		    {
			Vector locations2 = (Vector) locations.elementAt(i);

			for (int i2 = 0; i2 < locations2.size(); i2++)
			{
				Location l = (Location) locations2.elementAt(i2);

				if (l.getValid())
				    {
					int x = l.getPositionX();
					int y = l.getPositionY();

					Battle b = new Battle(l, true);

					for (int p = 0; p < players.size(); p++)
					    {

						Player player = (Player) players.elementAt(p);

						Vector units = player.getEmpire().getSpaceUnits();

						for (int u = 0; u < units.size(); u++)
						    {

							Unit unit = (Unit) units.elementAt(u);

							if (unit.getForces().size() > 0)
							{

								Location l2 = unit.getLocation();

								// Asteroid squares are considered the same.
								if (l.getAsteroids() && l2.getAsteroids() && first)
								{
									asteroidBattle.addUnit(unit);
								}

								else if (x == l2.getPositionX() && y == l2.getPositionY() && !l2.getAsteroids())
								{
									b.addUnit(unit);
									//System.out.println(unit.getName() + " at " + unit.getLocation());
								}

							}
						    }
				 	   }

					if (b.getCombatants().size() > 1)
					{
						spaceBattles.addElement(b);
					}
				    }

				if (l.getAsteroids())
				{
					first = false;
				}

				}
		    }


		// Check if there's a battle in the belt.
		if (asteroidBattle.getCombatants().size() > 1)
		{
			spaceBattles.addElement(asteroidBattle);
		}

		System.out.println("There will be " + spaceBattles.size() + " space battles!");
		ps.println("\nThere will be " + spaceBattles.size() + " space battles!");

		if (spaceBattles.size() == 0)
		{
		    	setUpGroundBattles();
			spaceFighting = false;
		}

		// Need to get enough time to escape from this loop.
		groundFighting = true;
		setUp = true;

		if (spaceBattles.size() == 0 && groundBattles.size() == 0)
		{
		    //	System.out.println("No PvP battles!\n");
			groundFighting = false;
		}


	    }

	// This method uses a player's name to return a vector of all the space battles in which they're involved, as
	// determined in the setUpSpaceBattles() method.

	public Vector getSpaceBattles(String name)
	{
		Vector result = new Vector();

		for (int i = 0; i < spaceBattles.size(); i++)
		{
			Battle b = (Battle) spaceBattles.elementAt(i);

			Vector combatants = b.getCombatants();

			for (int c = 0; c < combatants.size(); c++)
		    	{
				Combatant combatant = (Combatant) combatants.elementAt(c);

				if (combatant.getName().equals(name))
				{
					result.addElement(b);
				}
			}
		}

		return result;
	}

	// Returns a Vector of this player's space battles with neutral forces.  The neutral forces
	// are considered to be owned by a player named 'Neutral'.  This is to be done after all other
	// battles have been resolved.

	public synchronized Vector getNeutralSpaceBattles(String name)
	{
		// This player's neutralBattles.
		Vector neutralBattles = new Vector();
		Player currentPlayer;
		Vector units = new Vector();
		//battles = new Vector();

		// Wait until all the ground fighting is completed.
		while (groundFighting)
		{
			sleep();
		}

		//System.out.println("Checking neutral space battles! " + name);
		ps.println("Checking neutral space battles! " + name);

		for (int i = 0; i < players.size(); i++)
		{
			if (((Player)players.elementAt(i)).getName().equals(name))
			{
				currentPlayer = (Player) players.elementAt(i);

				units = currentPlayer.getEmpire().getSpaceUnits();
			}

		}

		// currentPlayer is now the player who called this method.

		// Compare locations of units with all neutral space fleets.

		for (int i = 0; i < units.size(); i++)
		{
			Unit u = (Unit) units.elementAt(i);

			Location l = u.getLocation();

			// Check if this unit is empty.

			if (u.getForces().size() > 0)
			{

				// Check if there is already a battle at this location, in which case we add
				// the unit.

				Battle check = findBattle(l);

				if (check != null && neutralBattles.contains(check)) // WARNING - && may be a danger.
				{
					//System.out.println("Adding " + u.getName() + " to a space battle!");
					check.addUnit(u);
				}

				else
				{


					for (int k = 0; k < neutralSpace.size(); k++)
					{
						Unit u2 = (Unit) neutralSpace.elementAt(k);
						u2.setOwner("Neutral");

						Location l2 = u2.getLocation();

						// If locations match, then we have a battle.

						if (!u2.isEmpty() && l.getPositionX() == l2.getPositionX() && l.getPositionY() == l2.getPositionY())
						{

							check = findBattle(l);

							if (check != null && neutralBattles.contains(check)) // WARNING - && may be a danger.
							{
								//System.out.println("Adding " + u.getName() + " to a space battle!");
								check.addUnit(u2);
							}

							else
							{


								Battle b = new Battle(l, true);
								b.addUnit(u);
								b.addUnit(u2);

								if (!emptyBattle(b)) // WARNING - new stuff. NOT TOTALLY TESTED.
								{
									neutralBattles.addElement(b);
									sbattles.addElement(b);
									System.out.println("Neutral battle between " + u + " and " + u2 + "("+k+")");
								}
							}
						}

					}

				}
			}

		}

		return neutralBattles;
	}


	// A single method that gathers all of the ground battles for every location.  This gives a consistant ordering
	// for when players choose to retrieve their ground battles.  This fills the vector ground battles.

   	public void setUpGroundBattles()
    {
		groundBattles = new Vector();

		Vector locations = map.getLocations();

		//System.out.println("Setting up all ground battles!");

		for (int i = 0; i < locations.size(); i++)
		{
			Vector locations2 = (Vector) locations.elementAt(i);

			for (int i2 = 0; i2 < locations2.size(); i2++)
			{
				Location l = (Location) locations2.elementAt(i2);

				if (l.getValid())
				{
					int x = l.getPositionX();
					int y = l.getPositionY();

					Battle b = new Battle(l, false);

					for (int p = 0; p < players.size(); p++)
					{

						Player player = (Player) players.elementAt(p);

						Vector units = player.getEmpire().getGroundUnits();

						for (int u = 0; u < units.size(); u++)
						{

							Unit unit = (Unit) units.elementAt(u);

							// Check if unit is empty.
							if (unit.getForces().size() > 0)
							{
								// Check if unit is unloaded or invading.

								if (!unit.getLoaded() || unit.getGroundInvade())
								{
									Location l2 = unit.getLocation();

									if (x == l2.getPositionX() && y == l2.getPositionY())
									{
										b.addUnit(unit);
									}
								}
							}
						}
				   }

					if (b.getCombatants().size() > 1)
					{
						groundBattles.addElement(b);

						// Unloading ground troops.
						Vector un = b.getUnits();

						for (int check = 0; check < un.size(); check++)
						{
							Unit unit2 = (Unit) un.elementAt(check);

							String play = unit2.getOwner();

							for (int check2 = 0; check2 < players.size(); check2++)
							{
								Player p = (Player) players.elementAt(check2);
								if (p.getName().equals(play))
								{
									unLoad(p, unit2);
								}
							}
						}
					}
				}
			}
		}

		System.out.println("There will be " + groundBattles.size() + " ground battles!");
		ps.println("\nThere will be " + groundBattles.size() + " ground battles!");

		if (groundBattles.size() == 0)
		{
			groundFighting = false;
		}
		else
		{
			spaceFighting = false;

			// TODO: Needs to be TESTED.
			groundFighting = true;

			// Do the AI vs. AI ground battles.
			for (int i = 0; i < aiPlayers.size(); i++)
			{
				Vector battles = getGroundBattles(aiPlayers.elementAt(i).player.getName());
				processAIBattles(aiPlayers.elementAt(i), battles);
			}
		}
	}

	// This method uses a player's name to return a vector of all the ground battles in which they're involved, as
	// determined in the setUpGroundBattles() method.

	public Vector getGroundBattles(String name)
	{

	    // Wait until all space fighting is done!
	    while (spaceFighting)
		{
			sleep();
		}

		Vector result = new Vector();

		for (int i = 0; i < groundBattles.size(); i++)
		{
			Battle b = (Battle) groundBattles.elementAt(i);

			Vector combatants = b.getCombatants();

			for (int c = 0; c < combatants.size(); c++)
		    {
				Combatant combatant = (Combatant) combatants.elementAt(c);

				if (combatant.getName().equals(name))
				{
					result.addElement(b);
				}
			}
		}

		return result;
	}

	// Returns a Vector of this player's space battles with neutral forces.  The neutral forces
	// are considered to be owned by a player named 'Neutral'.  This is to be done after all other
	// battles have been resolved.

	public synchronized Vector getNeutralGroundBattles(String name)
	{
		// This player's neutralBattles.
		Vector neutralBattles = new Vector();
		Player currentPlayer = null;
		Vector units = new Vector();
		//battles = new Vector();

		//System.out.println("Checking neutral ground battles! " + name);
		ps.println("Checking neutral ground battles! " + name);

		for (int i = 0; i < players.size(); i++)
		{
			if (((Player)players.elementAt(i)).getName().equals(name))
			{
				currentPlayer = (Player) players.elementAt(i);

				units = currentPlayer.getEmpire().getGroundUnits();
			}

		}

		// currentPlayer is now the player who called this method.

		// Compare locations of units with all neutral ground forces.

		for (int i = 0; i < units.size(); i++)
		{
			Unit u = (Unit) units.elementAt(i);

			Location l = u.getLocation();

			// Check if this unit is empty.

			if (u.getForces().size() > 0)
			{
				if (!u.getLoaded() || u.getGroundInvade())
				{

				// Check if there is already a battle at this location, in which case we add
				// the unit.

				Battle check = findBattle(l);

				if (check != null && !check.getSpace() && neutralBattles.contains(check)) // WARNING - && may be a danger.)
				{
					unLoad(currentPlayer, u);
					check.addUnit(u);
				}

				else
				{

					for (int k = 0; k < neutralGround.size(); k++)
					{
						Unit u2 = (Unit) neutralGround.elementAt(k);
						u2.setOwner("Neutral");

						Location l2 = u2.getLocation();

						// If locations match, then we have a battle.

						if (!u2.isEmpty() && l.getPositionX() == l2.getPositionX() && l.getPositionY() == l2.getPositionY())
						{

							check = findBattle(l);

							if (check != null && neutralBattles.contains(check)) // WARNING - && may be a danger.
							{
								//System.out.println("Adding " + u.getName() + " to a space battle!");
								check.addUnit(u2);
							}

							else
							{

								// Need to set it up for multiple player units to be involved.

								Battle b = new Battle(l, false);
								unLoad(currentPlayer, u);
								b.addUnit(u);
								b.addUnit(u2);

								if (!emptyBattle(b)) // WARNING - new stuff! NOT FULLY TESTED.
								{
									neutralBattles.addElement(b);
									gbattles.addElement(b);
									//System.out.println("Neutral battle between " + u + " and " + u2);
								}
							}
						}

					}
				}
				}
			}

		}

		return neutralBattles;
	}


	// Unloads a ground unit in preparation for battle.

	public void unLoad(Player p, Unit u)
	{
		if (u.getLoaded())
		{

			Vector fleets = p.getEmpire().getSpaceUnits();

			for (int i = 0; i < fleets.size(); i++)
			{
				Unit u2 = (Unit) fleets.elementAt(i);

				for (int t = 0; t < u2.getForces().size(); t++)
				{
					SpaceForce sf = (SpaceForce) u2.getForces().elementAt(t);

					for (int k = 0; k < sf.getStorage().size(); k++)
					{
						Unit utemp = (Unit) sf.getStorage().elementAt(k);

						if (utemp.getName().equals(u.getName()))
						{
							sf.getStorage().removeElementAt(k);
							u.setLoaded(false);
						}
					}
				}

			}

		}

	}

	public boolean isDuplicated(Vector battles, Battle b)
	{
		boolean result = false;

		Location l = b.getLocation();
		for (int i = 0; i < battles.size(); i++)
		{
			Battle b2 = (Battle) battles.elementAt(i);

			// Restoring all currentArmor to maximum!
			for (int k = 0; k < b2.getCombatants().size(); k++)
			{
				Combatant c = (Combatant) b2.getCombatants().elementAt(k);
				c.restoreAllForces();
			}

			if (b2.getLocation().equals(l))
			{
				result = true;
			}
		}

		return result;
	}

	// This method finds the battles in the relevant vectors that can then be resolved.

	public Battle findBattle(Location l)
	{
	    		Battle b = null;

			if (spaceFighting)
			{

				// Space Battles.
				for (int i = 0; i < spaceBattles.size(); i++)
				{
					Battle b2 = (Battle) spaceBattles.elementAt(i);
					if (b2.getLocation().getPositionY() == l.getPositionY() && b2.getLocation().getPositionX() == l.getPositionX())
					{
						b = b2;
					}
					// Asteroid Belt battle.
					else if (b2.getLocation().getAsteroids() && l.getAsteroids())
					{
						b = b2;
					}
				}
			}

			else if (groundFighting)
			{

				// Ground Battles.
				for (int i = 0; i < groundBattles.size(); i++)
				{
					Battle b2 = (Battle) groundBattles.elementAt(i);
					if (b2.getLocation().equals(l) || (b2.getLocation().getPositionY() == l.getPositionY() && b2.getLocation().getPositionX() == l.getPositionX()))
					{
						b = b2;
					}
				}
			}


			// Neutral Battles.
			//
			// Space
			for (int i = 0; i < sbattles.size(); i++)
			{
				Battle b2 = (Battle) sbattles.elementAt(i);
				if (b2.getLocation().equals(l) || (b2.getLocation().getPositionY() == l.getPositionY() && b2.getLocation().getPositionX() == l.getPositionX()))
				{
					b = b2;
				}
			}

			// Ground

			for (int i = 0; i < gbattles.size(); i++)
			{
				Battle b2 = (Battle) gbattles.elementAt(i);
				if (b2.getLocation().equals(l) || (b2.getLocation().getPositionY() == l.getPositionY() && b2.getLocation().getPositionX() == l.getPositionX()))
				{
					b = b2;
				}
			}

		return b;
	}

	public boolean emptyBattle(Battle b)
	{
		boolean empty = true;
		int combatants = 0;

		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			if (c.getSize() > 0)
			{
				combatants++;
			}
		}

		if (combatants > 1)
		{
			empty = false;
		}

		return empty;
	}


	// Returns a boolean indicating whether this current combatant is either neutral forces, or controlled by one of the
	// AI players.  TODO: More sophisticated behavior for committing and uncommitting forces.
	public boolean isComputerControlled(Combatant c)
	{
		if (c.getName().equals("Neutral"))
		{
			return true;
		}

		for (int i = 0; i < aiPlayers.size(); i++)
		{
			if (aiPlayers.elementAt(i).player.getName().equals(c.getName()))
			{
				return true;
			}
		}

		return false;
	}


	// setReadyForBattle - shooting indicates whether a person is opening fire.
	public void setReadyForBattle(String name, Location location, boolean shooting, Combatant newC)
	{
		Battle b = findBattle(location);

		int neutrals = 0; // the number of neutral combatants in the battle.

		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			if (isComputerControlled(c))//c.getName().equals("Neutral"))
			{
				neutrals++;
				c.setDamage(-1);
			}
		}

		b.incrementBattleCount();

		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			if (c.getName().equals(name))
			{
				// Set the committed values...
				for (int t = 0; t < c.getAllForces().size(); t++)
				{
					Force f = (Force) c.getAllForces().elementAt(t);

					Force f2 = (Force) newC.getAllForces().elementAt(t);

					f.setCommitted(f2.getCommitted());
				}

				c.getTargets().clear();

				for (int t = 0; t < newC.getTargets().size(); t++)
				{
					c.getTargets().addElement(newC.getTargets().elementAt(t));
					//System.out.println(c + " will target " + newC.getTargets().elementAt(t));
				}

				i = b.getCombatants().size();
			}
		}

		//System.out.println("\nBattle at " + location + " , Battle Count - " + b.getBattleCount());
		ps.println("\nBattle at " + location + " , Battle Count - " + b.getBattleCount());

		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);
			if (c.getName().equals(name))
			{
				Vector forces = c.getAllForces();

				int totalDamage = 0;

				if (shooting)
				{

					for (int k = 0; k < forces.size(); k++)
					{
						Force f = (Force) forces.elementAt(k);
						int damage = 0;

						// AI players will uncommit their transports that are carrying troops.

						if (isComputerControlled(c) && f.getClass().getName().equals("SpaceForce") && ((SpaceForce)f).getStorage().size() > 0 && f.getMaxAttack() < 1)
						{
							f.setCommitted(false);
						}

						if (f.getCommitted() || !b.getSpace())
						{
							damage = f.doDamage(r);
						}

						//System.out.println(c + "'s " + f.getName() + " inflicted " + damage + " damage!");
						ps.println(c + "'s " + f.getName() + " inflicted " + damage + " damage!");

						totalDamage += damage;
					}
				}

				c.setDamage(totalDamage);
				i = b.getCombatants().size();
			}
		}

		if (b.getBattleCount() >= (b.getCombatants().size() - neutrals))//== (b.getCombatants().size() - neutrals))
		{

			// Do neutral force damage, where player name is "Neutral"
			for (int i = 0; i < b.getCombatants().size(); i++)
			{
				Combatant c = (Combatant) b.getCombatants().elementAt(i);
				if (isComputerControlled(c) && c.getDamage() == -1)//c.getName().equals("Neutral"))
				{
					Vector forces = c.getAllForces();

					int totalDamage = 0;

					for (int k = 0; k < forces.size(); k++)
					{
						Force f = (Force) forces.elementAt(k);
						int damage = f.doDamage(r);

						// AI players will uncommit their transports that are carrying troops.

						if (isComputerControlled(c) && f.getClass().getName().equals("SpaceForce") && ((SpaceForce)f).getStorage().size() > 0 && f.getMaxAttack() < 1)
						{
							f.setCommitted(false);
							damage = 0;
						}

						//System.out.println(c + "'s " + f.getName() + " inflicted " + damage + " damage!");
						ps.println(c + "'s " + f.getName() + " inflicted " + damage + " damage!");

						totalDamage += damage;
					}

					c.setDamage(totalDamage);
					//i = b.getCombatants().size();
				}
			}

			dealDamage(b);
		}

	}

	public boolean isReadyForBattle(Location l)
	{
		boolean result = false;

		Battle b = findBattle(l);


		int neutrals = 0; // the number of neutral combatants in the battle.

		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);
			if (isComputerControlled(c))//c.getName().equals("Neutral"))
			{
				neutrals++;
			}
		}

/*
		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);
			if (c.getDamage() < 0)
			{
				result = false;
			}
		}
*/
		if (b.getBattleCount() == b.getCombatants().size()+1 - neutrals)
		{
			result = true;
		}

/*
		if (lastPast)
		{
			result = true;
			lastPast = false;
		}
*/

		return result;
	}



	// Combatant damage has already been calculated, and now it is dealt out to all forces in the battle.

	public void dealDamage(Battle b)
	{

		//System.out.println("Dealing damage!\n");
		ps.println("Dealing damage!\n");

		/* New Algorithm:
		-Loop through all the combatants' primary targets and assign damage to them.
		-If any of them are overkilled, take the excess damage and apply it to their next target.  This is a little
		tricky because we must try to fairly distribute the damage from those targeting that combatant (ie. if two people
		target the same foe, they should in theory each do 50% of the damage against them, and any remainder is overkill.  However,
		if one side does less than 50%, the other attacker does more.  If they don't do enough to overkill primary targets, then
		the algorithm simply finishes.
		*/
		Random random = new Random(System.currentTimeMillis());

		// For testing purposes, set each combatant up to target the next one in the combatant list.
		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			if (isComputerControlled(c))
			{
				c.getTargets().clear();

				int selection = random.nextInt(2);

				int mod = 1;
				if (selection == 0)
				{
					mod = -1;
				}

				for (int t = 1; t < b.getCombatants().size(); t++)
				{
					c.getTargets().addElement(((Combatant)b.getCombatants().elementAt((i+(mod * t) + b.getCombatants().size())%b.getCombatants().size())));
				}
			}
		}

		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			c.setDamageToAllocate(c.getDamage());
			c.setDamageTaken(0);

			//System.out.println(c + " inflicts " + c.getDamageToAllocate() + " damage!");
			ps.println(c + " inflicts " + c.getDamageToAllocate() + " damage!");
		}

		//System.out.println();
		ps.println();


		// Keep looping until there is no more damage to allocate, or there is only at most one active combatant.
		while (b.moreDamageToAllocate()  && b.stillContested())
		{
			for (int i = 0; i < b.getCombatants().size(); i++)
			{
				Combatant c = (Combatant) b.getCombatants().elementAt(i);

				//System.out.println(c + " has " + c.getDamageToAllocate() + " damage remaining to allocate!");

				// Check the top target for this combatant.
				if (c.getTargets().size() > 0)
				{
					Combatant target = c.getTargets().elementAt(0);

					for (int z = 0; z < b.getCombatants().size(); z++)
					{
						Combatant targ = (Combatant) b.getCombatants().elementAt(z);

						if (targ.getName().equals(target.getName()))
						{
							target = targ;
							break;
						}
					}

					if (c.getDamageToAllocate() > 0)
					{
						//System.out.println(c + " targets " + target + " with " + c.getDamageToAllocate() + " damage!");
						ps.println(c + " targets " + target + " with " + c.getDamageToAllocate() + " damage!");

						target.setDamageTaken(target.getDamageTaken() + c.getDamageToAllocate());
					}
				}
				else
				{
					c.setDamageToAllocate(0);
				}
			}

			for (int i =0; i < b.getCombatants().size(); i++)
			{
				Combatant c = (Combatant) b.getCombatants().elementAt(i);

				// If there is no overkill...
				if (c.getDamageTaken() <= c.getArmor())
				{
					// If there is no overkill, set the remaining damage to be allocated for all players targeting this person to zero.
					for (int t = 0; t < b.getCombatants().size(); t++)
					{
						Combatant attacker = (Combatant) b.getCombatants().elementAt(t);

						if (attacker.getTargets().size() > 0 && attacker.getTargets().elementAt(0).getName().equals(c.getName()))
						{
							attacker.setDamageToAllocate(0);
						}
					}
				}
			}

			for (int i = 0; i < b.getCombatants().size(); i++)
			{
				Combatant c = (Combatant) b.getCombatants().elementAt(i);

				// We only need to reprocess if we have overkill.
				if (c.getDamageTaken() > c.getArmor())
				{
					//System.out.println(c + " is overkilled, determining spillover damage!");
					ps.println(c + " is overkilled, determining spillover damage!");

					// We have overkilled this combatant, so we need to redistribute some of the damage done to them.

					// Grab a list of all combatants who are currently targeting this combatant.
					Vector<Combatant> attackers = new Vector<Combatant>();
					for (int t = 0; t < b.getCombatants().size(); t++)
					{
						Combatant attacker = (Combatant) b.getCombatants().elementAt(t);

						if (attacker.getTargets().size() > 0 && attacker.getTargets().elementAt(0).getName().equals(c.getName()))
						{
							attackers.addElement(attacker);
							attacker.getTargets().removeElementAt(0);
						}
					}

					int damageToDeal = c.getArmor();

					while (damageToDeal > 0 && attackers.size() > 0)
					{
						// Figure out how much percentage damage each attacker should do.
						int dividedDamage = (int)((damageToDeal+0.0)/attackers.size() + 0.99);

						for (int t = 0; t < attackers.size(); t++)
						{
							if (attackers.elementAt(t).getDamageToAllocate() >= dividedDamage)
							{
								damageToDeal -= dividedDamage;
								attackers.elementAt(t).setDamageToAllocate(attackers.elementAt(t).getDamageToAllocate()-dividedDamage);

								//System.out.println(attackers.elementAt(t) + " inflicts " + dividedDamage);
								ps.println(attackers.elementAt(t) + " inflicts " + dividedDamage);
							}
							else
							{
								damageToDeal -= attackers.elementAt(t).getDamageToAllocate();
								//System.out.println(attackers.elementAt(t) + " inflicts " + attackers.elementAt(t).getDamageToAllocate());
								ps.println(attackers.elementAt(t) + " inflicts " + attackers.elementAt(t).getDamageToAllocate());
								attackers.elementAt(t).setDamageToAllocate(0);
								attackers.removeElementAt(t);
								t--;
							}
						}
					}
				}
			}
		}

		// Now we actually deal the damage.
		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			int damage = c.getDamageTaken();

			Vector forces = c.getAllForces();

			while (damage > 0 && forces.size() > 0)
			{
				int target = r.nextInt(forces.size());

				Force f = (Force) forces.elementAt(target);

				// We've picked a target, but we now check if they are safe being uncommitted.
				if (!f.getCommitted() && c.remainingCommitted())
				{
					//System.out.println(f.getName() + " is uncommitted and can't be targeted! Trying again!");
				}
				else
				{
					damage = damage - f.getCurrentArmor();
					if (damage < 0)
					{
						//System.out.println(c.getName()+"'s "+f.getName() + " has been damaged, but it still has " + (-1*damage) + " armor!");
						ps.println(c.getName()+"'s "+f.getName() + " has been damaged, but it still has " + (-1*damage) + " armor!");

						f.setCurrentArmor(Math.abs(damage));
					}
					else
					{
						c.removeForce(f);
						forces.removeElement(f);
						if (b.getSpace())
						{
								SpaceForce sf = (SpaceForce) f;
							for (int k = 0; k < sf.getStorage().size(); k++)
							{
								Unit storedUnit = (Unit) sf.getStorage().elementAt(k);
								Vector temp = storedUnit.getForces();
								temp.removeAllElements();

								for (int z = 0; z < players.size(); z++)
								{
									Player play = (Player) players.elementAt(z);
									if (play.toString().equals(c.getName()))
									{

										Location l2 = storedUnit.getLocation();
										l2.removeUnit(storedUnit);
										storedUnit.setLocation(null);
									}
								}

								//System.out.println(c + "'s ground unit " + storedUnit + " has been destroyed!");
								ps.println(c + "'s ground unit " + storedUnit + " has been destroyed!");
							}
						}
						//System.out.println(c.getName() + "'s " + f.getName() + " has been destroyed with " + f.getCurrentArmor()+" damage!");
						ps.println(c.getName() + "'s " + f.getName() + " has been destroyed with " + f.getCurrentArmor()+" damage!");
					}
				}
			}
		}

		/*
		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			// The combatant attacking.
			int source = ((i+1)%(b.getCombatants().size()));
			Combatant c2 = (Combatant) b.getCombatants().elementAt(source);

			// The combatant being targeted.
			int tar = i;

			int damage = ((Combatant)b.getCombatants().elementAt(source)).getDamage(); // should get next combatant's damage!

			// Change who the damage goes to if the combatant's forces are killed.

			Vector forces = c.getAllForces();

			System.out.println(c2.getName() + " targets " + c.getName() + " with " + damage + " damage!");
			ps.println(c2.getName() + " targets " + c.getName() + " with " + damage + " damage!");

			while (damage > 0 && tar != source)//((i-1)%(b.getCombatants().size()))
			{

			if (forces.size() > 0)
			{
				//System.out.println("Inflicting damage to " + c.getName());

				int target = r.nextInt(forces.size());

				Force f = (Force) forces.elementAt(target);

				// We've picked a target, but we now check if they are safe being uncommitted.

				if (!f.getCommitted() && c.remainingCommitted())
				{
					System.out.println(f.getName() + " is uncommitted and can't be targeted! Trying again!");
				}
				else
				{

					damage = damage - f.getCurrentArmor(); // current armor seems to get set to 0 -- not sure why.
					if (damage < 0)
					{
						System.out.println(c2.getName() + " damaged "+c.getName()+"'s "+f.getName() + ", but it still has " + (-1*damage) + " armor!");
						ps.println(c2.getName() + " damaged "+c.getName()+"'s "+f.getName() + ", but it still has " + (-1*damage) + " armor!");

						f.setCurrentArmor(Math.abs(damage));
					}
					else
					{
						c.removeForce(f);
						forces.removeElement(f);
						if (b.getSpace())
						{
						    	SpaceForce sf = (SpaceForce) f;
							for (int k = 0; k < sf.getStorage().size(); k++)
							{
								Unit storedUnit = (Unit) sf.getStorage().elementAt(k);
								Vector temp = storedUnit.getForces();
								temp.removeAllElements();

								for (int z = 0; z < players.size(); z++)
								{
									Player play = (Player) players.elementAt(z);
									if (play.toString().equals(c.getName()))
									{

										Location l2 = storedUnit.getLocation();
										l2.removeUnit(storedUnit);
										storedUnit.setLocation(null);

									}
								}

								System.out.println(c + "'s ground unit " + storedUnit + " has been destroyed!");
								ps.println(c + "'s ground unit " + storedUnit + " has been destroyed!");
							}
						}
						System.out.println(c2.getName() + " has destroyed " +c.getName() + "'s " + f.getName() + " with " + f.getCurrentArmor()+" damage!");
						ps.println(c2.getName() + " has destroyed " +c.getName() + "'s " + f.getName() + " with " + f.getCurrentArmor()+" damage!");
					}
				}
			}
			else
			{
				// Switch to damaging a new combatant, because current has no forces.
 				//tar = ((i+1)%(b.getCombatants().size()));
				tar--;
				if (tar < 0)
				{
					tar = b.getCombatants().size() - 1;
				}
				c = (Combatant) b.getCombatants().elementAt(tar);
 				forces = c.getAllForces();
				System.out.println("OVERKILL! Carrying over " + damage + " damage to the next target, " + c.getName()+"!");
				ps.println("OVERKILL! Carrying over " + damage + " damage to the next target, " + c.getName()+"!");
			}
			//System.out.println("Survivors: " + forces);


			}

		}
		*/

		b.incrementBattleCount();

	}


	public void doneBattle(Location l, boolean space)
	{
	    //System.out.println("A battle is finished at " + l.getPositionX() + ", " +l.getPositionY()+"!");
	    ps.println("A battle is finished at " + l.getPositionX() + ", " +l.getPositionY()+"!");

		Battle b = findBattle(l);

		// For PvP battles...
		if (spaceFighting && space)
		{
			// Check if all space battles are over.
			//b = findBattle(l);

			spaceBattles.removeElement(b);

			if (spaceBattles.size() == 0)
			{
				// if (yes)
				setUpGroundBattles();
				spaceFighting = false;
			}

			String winner = " The battle was won by " + b.getWinner() + "!";

			if (b.getWinner().length() == 0)
			{
				winner = " No forces survived the battle.";
			}

			String location = "in deep space";

			if (l.getPlanets().size() > 0)
			{
				location = "above " + l.getPlanets().elementAt(0);
			}

			// Adding a message about each finished space battle.
			Message m = new Message("A "+getAdjective(b)+" space battle was fought " + location + " between " + (Combatant)b.getCombatants().elementAt(0) + " and " + (Combatant)b.getCombatants().elementAt(1) + "!" + winner,  true);

			if (b.getCombatants().size() > 2)
			{
				String mess = "A "+getAdjective(b)+" space battle was fought " + location + " between ";
				for (int t = 0; t < b.getCombatants().size(); t++)
				{
					mess += "" + (Combatant)b.getCombatants().elementAt(t);

					if (t == b.getCombatants().size()-2)
					{
						mess += " and ";
					}
					else
					{
						if (t != b.getCombatants().size()-1)
						{
							mess += ", ";
						}
					}
				}
				mess += "!" + winner;
				m = new Message(mess, true);
			}

			//System.out.println(m);
			ps.println(m);

			messages.insertElementAt(m, 0);
		}
		else if (groundFighting && !space)
		{
			//b = findBattle(l);

			if (groundBattles.size() > 0)
			{
				groundBattles.removeElement(b);
			}

			if (groundBattles.size() == 0)
			{
				// if (yes)
				groundFighting = false;
			}

			String winner = " The battle was won by " + b.getWinner() + "!";

			if (b.getWinner().length() == 0)
			{
				winner = " No forces survived the battle.";
			}

			// Adding a message about each finished ground battle.
			Message m = new Message("A "+getAdjective(b)+" ground battle was fought on " + (Planet)l.getPlanets().elementAt(0) + " between " + (Combatant)b.getCombatants().elementAt(0) + " and " + (Combatant)b.getCombatants().elementAt(1) + "!" + winner,  true);

			if (b.getCombatants().size() > 2)
			{
				String mess = "A "+getAdjective(b)+" ground battle was fought on " + (Planet)l.getPlanets().elementAt(0) + " between ";
				for (int t = 0; t < b.getCombatants().size(); t++)
				{
					mess += "" + (Combatant)b.getCombatants().elementAt(t);

					if (t == b.getCombatants().size()-2)
					{
						mess += " and ";
					}
					else
					{
						if (t != b.getCombatants().size()-1)
						{
							mess += ", ";
						}
					}
				}
				mess += "!" + winner;
				m = new Message(mess, true);
			}

			//System.out.println(m);
			ps.println(m);

			messages.addElement(m);
		}
		if (b != null)
		{
			// Update AI intelligence info.
			for (int i = 0; i < b.getCombatants().size(); i++)
			{
				Combatant c = (Combatant) b.getCombatants().elementAt(i);

				for (int t = 0; t < aiPlayers.size(); t++)
				{
					if (c.getName().equals(aiPlayers.elementAt(t).player.getName()))
					{
						if (map.getLocationAt(l.getPositionY(), l.getPositionX()).getPlanets().size() > 0)
						{
							Planet planet = (Planet) map.getLocationAt(l.getPositionY(), l.getPositionX()).getPlanets().elementAt(0);
							aiPlayers.elementAt(t).intelligenceFile.updateUnits(planet, map.getTurn(), map, neutralSpace, neutralGround);

							//System.out.println(aiPlayers.elementAt(t).intelligenceFile.get(planet.getName()));
						}

						for (int z = 0; z < b.getCombatants().size(); z++)
						{
							aiPlayers.elementAt(t).intelligenceFile.updateAggression(((Combatant)b.getCombatants().elementAt(z)).getName(), 1.0);
							//System.out.println("Adjusting " + aiPlayers.elementAt(t).player + "'s aggression against " + ((Combatant)b.getCombatants().elementAt(z)).getName());
						}

						break;
					}
				}
			}
		}
	}


	// Checks all planets for a specific player to see if they still own them.

	public void checkPlanets(String name, Vector planets)
	{

		for (int t = 0; t < planets.size(); t++)
		{
			Planet pl = (Planet) planets.elementAt(t);

			Location loc = pl.getLocation();

			for (int i = 0; i < players.size(); i++)
			{
				Player play = (Player) players.elementAt(i);

				if (!play.getName().equals(name))
				{

					for (int k = 0; k < play.getEmpire().getGroundUnits().size(); k++)
					{
						Unit u = (Unit) play.getEmpire().getGroundUnits().elementAt(k);

						if (u.getForces().size() > 0 && !u.getLoaded()) // Ignore loaded condition for now.
						{
							Location loc2 = u.getLocation();

							if (loc.getPositionX() == loc2.getPositionX() && loc.getPositionY() == loc2.getPositionY())
							{
								planets.removeElementAt(t);

								for (int z = 0; z < aiPlayers.size(); z++)
								{
									if (aiPlayers.elementAt(z).player.getName().equals(name))
									{
										aiPlayers.elementAt(z).lostPlanet = true;
									}
								}

								//System.out.println("Removing " + pl + " from " + name);
								t--;
								k = play.getEmpire().getGroundUnits().size();
								i = players.size();
							}

						}
					}
				}
			}

		}

		//return planets;

		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			if (p.getName().equals(name))
			{
				p.getEmpire().setPlanets(planets);
			}
		}
	}



	// This method is called at the end of every round to determine if the game is over.  If not,
	// the game moves on to the next round.

	public boolean gameOver()
	{
		boolean	over = false;

		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			Empire e = p.getEmpire();

			if (e.getVictoryPoints() >= e.getMaxVictoryPoints())
			{
				over = true;
				System.out.println("Player " + p + " has won this game of Pax!");
				ps.println("Player " + p + " has won this game of Pax!");
			}
		}

		if (over)
		{
			System.out.println("Final scores: ");
			ps.println("Final scores: ");

			for (int i = 0; i < players.size(); i++)
			{
				Player p = (Player) players.elementAt(i);
				Empire e = p.getEmpire();
				System.out.println(p + ": " + ((e.getVictoryPoints()+0.0)/e.getMaxVictoryPoints()));
				ps.println(p + ": " + ((e.getVictoryPoints()+0.0)/e.getMaxVictoryPoints()));

			}
		}

		return over;
	}

	// This method removes a player from the game, assuming that they have been eliminated.

	public void removePlayer(String name)
	{
		Player player = null;

		for (int i = 0; i < players.size(); i++)
		{
			player = (Player) players.elementAt(i);

			if (player.getName().equals(name))
			{

				players.remove(player);

				for (int z = i; z < players.size(); z++)
				{
					progress[z] = progress[z+1];
				}

				totalPlayers = players.size();

				System.out.println(name + " has been removed from the game!");
				break;
			}

		}
	}

	// This method drops a player from the game, turning their empire into an AI.

	public void dropPlayer(String name)
	{
		Player player = null;

		for (int i = 0; i < players.size(); i++)
		{
			player = (Player) players.elementAt(i);

			if (player.getName().equals(name))
			{
				player.setName("AI " + player.getEmpire().getRace());

				aiPlayers.addElement(new PaxGenericAI(player, spaceForces, groundForces, Runtime.getRuntime().availableProcessors(), map, empires));

				// Change ownership of units, planets.

				Empire e = player.getEmpire();

				for (int t = 0; t < e.getSpaceUnits().size(); t++)
				{
					Unit u = (Unit) e.getSpaceUnits().elementAt(t);

					u.setOwner(player.getName());
				}
				for (int t = 0; t < e.getGroundUnits().size(); t++)
				{
					Unit u = (Unit) e.getGroundUnits().elementAt(t);

					u.setOwner(player.getName());
				}

				for (int p = 0; p < e.getPlanets().size(); p++)
				{
					Planet plan = (Planet) e.getPlanets().elementAt(p);
					plan.setOwner(player.getName());
				}

				aiPlayers.elementAt(aiPlayers.size() - 1).setup();

				int prog = progress[i];

				for (int t = i; t > 0; t--)
				{
					progress[t] = progress[t-1];
					deploy[t] = deploy[t-1];
				}

				progress[0] = prog;

				donePhase(player.getName(), map.getPhase(), map.getTurn(), player);

				break;
			}
		}
		System.out.println(name + " has dropped from the game!");
	}


	// This method is used to sleep waiting clients while battles are being set up, needed to
	// prevent mischief from Donna's blazingly fast laptop.

	public void sleep()
	{
		try
		{
			Thread.sleep(2000);
		}
		catch (Exception e)
		{
			System.out.println("A waiting client can't sleep! " + e.toString());
		}
	}

	// Returns a player based on their name.

	public Player getPlayer(String name)
	{
		Player player = null;

		for (int i = 0; i < players.size(); i++)
		{
			player = (Player) players.elementAt(i);

			if (player.getName().equals(name))
			{
				i = players.size();
			}
		}

		return player;
	}

	// getOffers() is a survey method that the clients ping every few seconds to
	// refresh the list of offers in the market.

	public Vector getOffers()
	{
		return offers;
	}

	// addOffer adds a new offer to the Vector of offers and subtract's name's offered resources.

	public void addOffer(String name, Offer offer)
	{
		offers.addElement(offer);

		Player one = getPlayer(offer.getOne());

		one.getEmpire().setFood(one.getEmpire().getFood() - offer.getOFood());
		one.getEmpire().setGas(one.getEmpire().getGas() - offer.getOGas());
		one.getEmpire().setMinerals(one.getEmpire().getMinerals() - offer.getOMinerals());
		one.getEmpire().setMoney(one.getEmpire().getMoney() - offer.getOMoney());
	}

	// Player 'name' accepts offer 'id'.

	public void acceptOffer(String name, String id)
	{
		// We grab the correct offer.
		Offer o = getOffer(id, false);

		if (o != null)
		{
			boolean confirmed = o.accept(name);

			// If confirmed, we pay out the items to both players.

			Player one = getPlayer(o.getOne());
			Player two = getPlayer(o.getTwo());

			if (confirmed)
			{

				// Trading Resources

				one.getEmpire().setFood(one.getEmpire().getFood() + o.getDFood());
				one.getEmpire().setGas(one.getEmpire().getGas() + o.getDGas());
				one.getEmpire().setMinerals(one.getEmpire().getMinerals() + o.getDMinerals());
				one.getEmpire().setMoney(one.getEmpire().getMoney() + o.getDMoney());

				two.getEmpire().setFood(two.getEmpire().getFood() + o.getOFood());
				two.getEmpire().setGas(two.getEmpire().getGas() + o.getOGas());
				two.getEmpire().setMinerals(two.getEmpire().getMinerals() + o.getOMinerals());
				two.getEmpire().setMoney(two.getEmpire().getMoney() + o.getOMoney());

			}
			// Not confirmed, so this is just the second player accepting, and we subtract their
			// resources.
			else
			{
				two.getEmpire().setFood(two.getEmpire().getFood() - o.getDFood());
				two.getEmpire().setGas(two.getEmpire().getGas() - o.getDGas());
				two.getEmpire().setMinerals(two.getEmpire().getMinerals() - o.getDMinerals());
				two.getEmpire().setMoney(two.getEmpire().getMoney() - o.getDMoney());
			}
		}
	}

	// rejectOffer order is given by participant one to boot participant two from the offer, and it restores
	// their resources.

	public void rejectOffer(String name, String id)
	{
		// We grab the correct offer.
		Offer o = getOffer(id, false);

		if (o != null)
		{
			Player two = getPlayer(o.getTwo());

			boolean confirmed = o.reject(name);

			// Refund player two's resources.
			if (confirmed)
			{
				two.getEmpire().setFood(two.getEmpire().getFood() + o.getDFood());
				two.getEmpire().setGas(two.getEmpire().getGas() + o.getDGas());
				two.getEmpire().setMinerals(two.getEmpire().getMinerals() + o.getDMinerals());
				two.getEmpire().setMoney(two.getEmpire().getMoney() + o.getDMoney());
			}
		}
	}

	// withDraws an offer.  Must be initiated by the sender.  Removes from the vector and refunds all
	// associated resources.

	public void withdrawOffer(String name, String id)
	{
		Offer o = getOffer(id, true);

		if (!o.getOne().equals(name))
		{
			offers.addElement(o);
		}
		else if (o != null)
		{
			Player one = getPlayer(o.getOne());

			// Refunding player one's resources.
			one.getEmpire().setFood(one.getEmpire().getFood() + o.getOFood());
			one.getEmpire().setGas(one.getEmpire().getGas() + o.getOGas());
			one.getEmpire().setMinerals(one.getEmpire().getMinerals() + o.getOMinerals());
			one.getEmpire().setMoney(one.getEmpire().getMoney() + o.getOMoney());

			// Refunding player two's resources.
			if (o.getFull())
			{
				Player two = getPlayer(o.getTwo());

				two.getEmpire().setFood(two.getEmpire().getFood() + o.getDFood());
				two.getEmpire().setGas(two.getEmpire().getGas() + o.getDGas());
				two.getEmpire().setMinerals(two.getEmpire().getMinerals() + o.getDMinerals());
				two.getEmpire().setMoney(two.getEmpire().getMoney() + o.getDMoney());
			}


		}
	}


	// A synchronized method for getting or removing offers from the Vector of offers.
	// Returns the offer being deleted, or null if the offer doesn't exist.

	public synchronized Offer getOffer(String id, boolean del)
	{
		Offer o = null;

		for (int i = 0; i < offers.size(); i++)
		{
			Offer offer = (Offer) offers.elementAt(i);

			if (offer.getId().equals(id))
			{
				o = offer;
				if (del)
				{
					offers.removeElementAt(i);
				}
				i = offers.size();
			}
		}

		return o;
	}

	// A method that returns the progress array so that the market frame can display who
	// is still on the market.

	public int[] getProgress()
	{
		return progress;
	}



	// Espionage methods - ecoBomb inflicts damage to planet's economy.

	public void ecoBomb(Planet planet, int damage)
	{

		// Get the player who owns this planet.

		String owner = planet.getOwner();

		Empire empire = null;

		// If !neutral planet, then we look through player vector.
		if (!owner.equals("Neutral"))
		{
			Player play = getPlayer(owner);
			empire = play.getEmpire();

			// Construct message to send to damaged player.

			String string = "Your empire has been infiltrated by enemy spies! \n\n They've destroyed key economic infrastructure on " +planet.getName()+", reducing its economy by "+damage+"!";

			Message m = new Message(string, false);
			m.addPlayer(play.getName());
			messages.addElement(m);
		}
		// Look through neutral vector for empire.
		else
		{
			for (int i = 0; i < empires.size(); i++)
			{
				empire = (Empire) empires.elementAt(i);

				for (int t = 0; t < empire.getPlanets().size(); t++)
				{
					Planet p = (Planet) empire.getPlanets().elementAt(t);

					if (p.getName().equals(planet.getName()))
					{
						i = empires.size();
					}
				}

			}
		}

		// We have the right empire, find the planet.

		for (int i =0; i < empire.getPlanets().size(); i++)
		{
			Planet plan = (Planet) empire.getPlanets().elementAt(i);

			if (plan.getName().equals(planet.getName()))
			{
				// We've found the right planet, subtract eco damage.

				plan.setEconomy(plan.getEconomy() - damage);

				if (plan.getEconomy() < 0)
				{
					plan.setEconomy(0);
				}
			}
		}
	}

	// Espionage methods - stealMoney steals money from a player and returns the amount of money that remains.

	public int stealMoney(Planet planet, int damage, String recipient)
	{

		// Get the player who owns this planet.

		String owner = planet.getOwner();

		Empire empire = null;
		Player play = null;

		// If !neutral planet, then we look through player vector.
		if (!owner.equals("Neutral"))
		{
			play = getPlayer(owner);
			empire = play.getEmpire();


		}
		// Look through neutral vector for empire.
		else
		{
			for (int i = 0; i < empires.size(); i++)
			{
				empire = (Empire) empires.elementAt(i);

				for (int t = 0; t < empire.getPlanets().size(); t++)
				{
					Planet p = (Planet) empire.getPlanets().elementAt(t);

					if (p.getName().equals(planet.getName()))
					{
						i = empires.size();
					}
				}

			}
		}

		// We have the empire - subtract their money.

		if (empire.getMoney() < damage)
		{
			damage = empire.getMoney();
		}

		empire.setMoney(empire.getMoney()-damage);

		// Give money to the user.

		Player user = getPlayer(recipient);
		user.getEmpire().setMoney(user.getEmpire().getMoney() + damage);

		// Construct message to send to damaged player.
		if (!owner.equals("Neutral"))
		{

			String string = "Your empire has been infiltrated by enemy spies! \n\n They've stolen " +damage+" money from your empire!";

			Message m = new Message(string, false);
			m.addPlayer(play.getName());
			messages.addElement(m);
		}


		return damage;
	}


	// Espionage methods - industryBomb inflicts damage to planet's industry.

	public void industryBomb(Planet planet, int damage)
	{

		// Get the player who owns this planet.

		String owner = planet.getOwner();

		Empire empire = null;

		// If !neutral planet, then we look through player vector.
		if (!owner.equals("Neutral"))
		{
			Player play = getPlayer(owner);
			empire = play.getEmpire();

			// Construct message to send to damaged player.

			String string = "Your empire has been infiltrated by enemy spies! \n\n They've destroyed key industrial infrastructure on " +planet.getName()+", reducing its industry by "+damage+"!";

			Message m = new Message(string, false);
			m.addPlayer(play.getName());
			messages.addElement(m);
		}
		// Look through neutral vector for empire.
		else
		{
			for (int i = 0; i < empires.size(); i++)
			{
				empire = (Empire) empires.elementAt(i);

				for (int t = 0; t < empire.getPlanets().size(); t++)
				{
					Planet p = (Planet) empire.getPlanets().elementAt(t);

					if (p.getName().equals(planet.getName()))
					{
						i = empires.size();
					}
				}

			}
		}

		// We have the right empire, find the planet.

		for (int i =0; i < empire.getPlanets().size(); i++)
		{
			Planet plan = (Planet) empire.getPlanets().elementAt(i);

			if (plan.getName().equals(planet.getName()))
			{
				// We've found the right planet, subtract industry damage.

				plan.setIndustry(plan.getIndustry() - damage);

				if (plan.getIndustry() < 0)
				{
					plan.setIndustry(0);
				}

				plan.setCurrentIndustry(plan.getIndustry());
			}
		}
	}

	// Allows a player to gain tech.
	public void techSteal(String name, int newTech)
	{
		Player player = getPlayer(name);

		player.getEmpire().setTech(newTech);

	}


	// Committs all the forces for every player.
	public void committAll()
	{
		for (int i = 0; i < players.size(); i++)
		{
			Player p = (Player) players.elementAt(i);

			for (int t = 0; t < p.getEmpire().getSpaceUnits().size(); t++)
			{
				Unit u = (Unit) p.getEmpire().getSpaceUnits().elementAt(t);

				for (int z = 0; z < u.getForces().size(); z++)
				{
					Force f = (Force) u.getForces().elementAt(z);
					f.setCommitted(true);
				}
			}

			for (int t = 0; t < p.getEmpire().getGroundUnits().size(); t++)
			{
				Unit u = (Unit) p.getEmpire().getGroundUnits().elementAt(t);

				for (int z = 0; z < u.getForces().size(); z++)
				{
					Force f = (Force) u.getForces().elementAt(z);
					f.setCommitted(true);
				}
			}
		}
	}

	public void useEspionage(String name, int amount)
	{
		Player play = getPlayer(name);

		play.getEmpire().setEspionage(play.getEmpire().getEspionage() - amount);
	}


	// This method selects an appropriate adjective to describe a battle.
	public String getAdjective(Battle b)
	{
		String result = "small";

		int totalDamage = 0;

		for (int i = 0; i < b.getCombatants().size(); i++)
		{
			Combatant c = (Combatant) b.getCombatants().elementAt(i);

			totalDamage += c.getTotalDamage();
		}

		result = "pathetic";

		if (totalDamage > 40)
		{
			result = "mediocre";
		}
		if (totalDamage > 70)
		{
			result = "vigorous";
		}
		if (totalDamage > 100)
		{
			result = "large";
		}
		if (totalDamage > 130)
		{
			result = "massive";
		}
		if (totalDamage > 160)
		{
			result = "titanic";
		}

		return result;
	}

	public Vector getPlayers()
	{
		return players;
	}



	// acceptOperations takes a list of operations from the client and adds them to the operations or the
	// defensive operations vector.

	public void acceptOperations(Vector ops)
	{
		for (int t = 0; t < ops.size(); t++)
		{
			Operation op = (Operation) ops.elementAt(t);

			if (op.getDefensive())
			{
				defensiveOperations.addElement(op);
			}
			else
			{
				operations.addElement(op);
			}
		}
	}

	// performOperations processes all offensive operations.

	public void performOperations()
	{
		messages = new Vector();

		//System.out.println("Performing operations.");

		for (int t = 0; t < operations.size(); t++)
		{

			Operation op = (Operation) operations.elementAt(t);

			//System.out.println(op.getPlayer().getName() + " uses " +op.toString()+ " on " + op.getTargetPlanet());
			ps.println(op.getPlayer().getName() + " uses " +op.toString()+ " on " + op.getTargetPlanet());

			op.performOperation(map, messages, defensiveOperations, aiPlayers);

		}
	}



}
