import java.util.*;
import java.io.*;

public class Planet implements Serializable
{

	private String name;
	private String owner;

	private int economy;
	private int industry;
	private int currentIndustry;
	private int food;
	private int gas;
	private int minerals;

	private Location location;

	// indicates whether a planet was captured this turn, needed to prevent industry restore.
	private boolean captured;
	// indicates whether industry has been bombed using espionage.
	private boolean industryBombed;


	public Planet(String nam, int eco, int indus, int foo, int ga, int mineral)
	{
		name = nam;
		economy = eco;
		industry = indus;
		currentIndustry = industry;
		food = foo;
		gas = ga;
		minerals = mineral;

		captured = false;
		industryBombed = false;

		owner = "Neutral";
	}

	// Clones this planet and returns the clone.
	public Planet clone()
	{
		Planet clone = new Planet(this.name, this.economy, this.industry, this.food, this.gas, this.minerals);
		clone.setCurrentIndustry(this.currentIndustry);
		clone.setCaptured(this.captured);
		clone.setIndustryBombed(this.industryBombed);
		clone.setOwner(this.owner);

		clone.setLocation(this.location);

		return clone;
	}

	public String getName()
	{
		return name;
	}

	public String getOwner()
	{
		return owner;
	}

	public int getEconomy()
	{
		return economy;
	}

	public int getIndustry()
	{
		return industry;
	}

	public int getCurrentIndustry()
	{
		return currentIndustry;
	}

	public int getFood()
	{
		return food;
	}

	public int getGas()
	{
		return gas;
	}

	public int getMinerals()
	{
		return minerals;
	}

	public void setEconomy(int eco)
	{
		economy = eco;
	}

	public void setIndustry(int ind)
	{
		industry = ind;
	}

	public void setCurrentIndustry(int ind)
	{
		currentIndustry = ind;
	}

	public void setOwner(String own)
	{
		owner = own;
	}

	public void setLocation(Location l)
	{
		location = l;
	}

	public void restoreIndustry()
	{
		if (!captured && !industryBombed)
		{
			currentIndustry = industry;
		}
	}

	public String toString()
	{
		return name;
	}

	public Location getLocation()
	{
		return location;
	}

	public boolean getCaptured()
	{
		return captured;
	}

	public void setCaptured(boolean cap)
	{
		captured = cap;

		if (cap == false)
		{
			industryBombed = false;
		}
	}

	public void setIndustryBombed(boolean bomb)
	{
		industryBombed = bomb;
	}

	// Check if the space above this planet is clear, and thus we are ready to build.
	public boolean isSpaceClear(Player player, Vector players)
	{
		boolean clear = true;

		Location loc = getLocation();

		for (int check = 0; check < players.size(); check++)
		{
			Player play = (Player) players.elementAt(check);

			if (!play.getName().equals(player.getName()))
			{

				for (int check2 = 0; check2 < play.getEmpire().getSpaceUnits().size(); check2++)
				{
					Unit un = (Unit) play.getEmpire().getSpaceUnits().elementAt(check2);

					if (un.getForces().size() > 0)
					{

						Location loc2 = un.getLocation();

						if (loc.getPositionX() == loc2.getPositionX() && loc.getPositionY() == loc2.getPositionY())
						{
							clear = false;
						}

					}

				}
			}
		}

		return clear;
	}


}
