import java.util.*;
import java.io.*;

public class Player implements Serializable
{

	private String name;
	private String hostName;
	private Empire empire;
	private int newUnitCounter;

	public Player(String nam)
	{
		name = nam;
		newUnitCounter = 0;
	}

	public void setEmpire(Empire e)
	{
		empire = e;
	}



	public String toString()
	{
	    	String result = name;
		if (name.length() > 15)
		{
			result = result.substring(0, 15) + "..";
		}
		return result;
	}

	public Empire getEmpire()
	{
		return empire;
	}

	public String getName()
	{
	    return name;
	}

	public void setName(String newName)
	{
		name = newName;
	}

	public int getNewUnitCounter()
	{
		newUnitCounter++;
		return newUnitCounter;
	}

	public void setNewUnitCounter(int newUnitCount)
	{
		this.newUnitCounter = newUnitCount;
	}

	public String getHostName()
	{
		return hostName;
	}

	public void setHostName(String hostNam)
	{
		hostName = hostNam;
	}


	// Adds a force at a specific location - sticking it into a unit of an appropriate type, or creating one if there
	// is not.
	public void addForce(Force force, Location location)
	{
		boolean created = false;

		boolean spaceForce = true;

		try
		{
			SpaceForce sf = (SpaceForce) force;
		}
		catch (Exception e)
		{
			spaceForce = false;
		}

		if (spaceForce)
		{
			// Add movement for Hyper-LSD;
			if (empire.getTech() > 44 && force.getMobile())
			{
				force.hyperLSD();
			}

			for (int k = 0; k < empire.getSpaceUnits().size(); k++)
			{
				Unit u = (Unit) empire.getSpaceUnits().elementAt(k);
				if ((u.isEmpty() || (u.getLocation() != null && u.getLocation().getPositionX() == location.getPositionX() && u.getLocation().getPositionY() == location.getPositionY())) && !created)
				{
					created = true;
					u.addForce(force);
					u.setLocation(location);
					//System.out.println("Adding " + force + " to " + u + "!");
				}
			}

			if (!created)
			{
				Unit newU = new Unit("New Unit " + getNewUnitCounter(), toString());
				newU.addForce(force);
				newU.setLocation(location);
				location.addUnit(newU);
				empire.addSpaceUnit(newU);
			}
		}
		else
		{
			for (int k = 0; k < empire.getGroundUnits().size(); k++)
			{
				Unit u = (Unit) empire.getGroundUnits().elementAt(k);
				if ((u.isEmpty() || (u.getLocation() != null && u.getLocation().getPositionX() == location.getPositionX() && u.getLocation().getPositionY() == location.getPositionY() && !u.getLoaded())) && !created)// && !u.getLoaded())
				{
					created = true;
					u.addForce(force);
					u.setLocation(location);
					u.setLoaded(false);

				}
			}
			if (!created)
			{
				Unit newU = new Unit("New Unit " + getNewUnitCounter(), toString());
				newU.addForce(force);
				newU.setLocation(location);
				location.addUnit(newU);
				empire.addGroundUnit(newU);
			}
		}

		empire.setFood(empire.getFood() - force.getFood());
		empire.setGas(empire.getGas() - force.getGas());
		empire.setMinerals(empire.getMinerals() - force.getMinerals());

		// Find the right planet, subtract the industry.
		for (int i = 0; i < empire.getPlanets().size(); i++)
		{
			Planet planet = (Planet) empire.getPlanets().elementAt(i);
			if (planet.getLocation().getPositionX() == location.getPositionX() && planet.getLocation().getPositionY() == location.getPositionY())
			{
				planet.setCurrentIndustry(planet.getCurrentIndustry() - force.getIndustry());
				break;
			}
		}

		empire.calculateMaintenance();
	}


	// Returns a clone of this Player object.
	public Player clone()
	{
		Player clone = new Player(this.name);
		clone.setNewUnitCounter(this.newUnitCounter);
		clone.setHostName(this.getHostName());
		clone.setEmpire(this.empire.clone());
		return clone;
	}

}
