import java.util.*;
import java.io.*;

public class Unit implements Serializable
{

	private String name;
	private String owner;

	private Vector forces;
	private Location location;
	private int minMove;
	private boolean loaded;
	private boolean groundInvade;

	public Unit(String nam, String owne)
	{
		name = nam;
		owner = owne;
		minMove = 7;
		forces = new Vector();
		loaded = false;
		groundInvade = false;
	}

	// Clones this Unit, returns the clone.
	public Unit clone()
	{
		Unit clone = new Unit(this.name, this.owner);
		clone.setLoaded(this.loaded);
		clone.setGroundInvade(this.groundInvade);

		for (int i = 0; i < forces.size(); i++)
		{
			Force f = (Force) forces.elementAt(i);
			clone.addForce((Force)f.clone());
		}

		// Now setting the Location within the clone method.
		clone.setLocation(this.getLocation());

		return clone;
	}

	public void addForce(Force temp)
	{
		forces.addElement(temp);
		if (temp.getMove() < minMove)
		{
			minMove = temp.getCurrentMove();
		}
	}

	public boolean removeForce(Force temp)
	{
		boolean result = false;

		if (forces.contains(temp))
		{
			forces.remove(temp);
			calculateMinMove();
			result = true;
		}

		return result;
	}

	public void calculateMinMove()
	{
	    	minMove = 7;
		for (int i = 0; i < forces.size(); i++)
		{
			Force temp = (Force) forces.elementAt(i);
			if (temp.getMove() < minMove)
			{
				minMove = temp.getCurrentMove();
			}
		}
	}

	public Vector getForces()
	{
		return forces;
	}

	public String toString()
	{
	    	String result = name;
		if (result.length() > 14)
		{
			result = result.substring(0, 14) + "..";
		}
		return result;
	}

	public Location getLocation()
	{
		return location;
	}

	public void setLocation(Location l)
	{
		location = l;

		try
		{
			for (int i = 0; i < forces.size(); i++)
			{
				SpaceForce sf = (SpaceForce) forces.elementAt(i);
				Vector temp = sf.getStorage();

				for (int k = 0; k < temp.size(); k++)
				{
					Unit u = (Unit) temp.elementAt(k);
					u.setLocation(l);
				}
			}
		}
		catch (Exception e) {}
	}

	public int getMinMove()
	{
		return minMove;
	}

	public String getOwner()
	{
		return owner;
	}

	public String getName()
	{
		return name;
	}

	public void setOwner(String owne)
	{
		owner = owne;
	}

	public void setName(String nam)
	{
		name = nam;
	}

	public boolean isEmpty()
	{
		return (forces.size() == 0);
	}

	public void setMinMove(int newMin)
	{
		for (int i = 0; i < forces.size(); i++)
		{
			SpaceForce sf = (SpaceForce) forces.elementAt(i);
			sf.setMove(sf.getCurrentMove() - newMin);//1);

			if (sf.getMove() < 0)
			{
				sf.setMove(0);
			}
		}
		minMove = minMove - newMin;
		if (minMove < 0)
		{
			minMove = 0;
		}
	}

	public void restoreMovement()
	{
		for (int i = 0; i < forces.size(); i++)
		{
			SpaceForce sf = (SpaceForce) forces.elementAt(i);
			sf.restoreMove();
		}
		calculateMinMove();
	}

	public boolean getLoaded()
	{
		return loaded;
	}

	public void setLoaded(boolean load)
	{
		loaded = load;
		groundInvade = false;
	}


	public void repair()
	{
		for (int i = 0; i < forces.size(); i++)
		{
			Force f = (Force) forces.elementAt(i);

			f.restoreArmor();
		}
	}

	public boolean getGroundInvade()
	{
		return groundInvade;
	}

	public void setGroundInvade(boolean invade)
	{
		groundInvade = invade;
	}

	public boolean isSpace()
	{
		boolean result = true;

		if (!this.isEmpty() && this.forces.elementAt(0).getClass().getName().equals("GroundForce"))
		{
			result = false;
		}

		return result;
	}

	public int getTotalMaxAttack()
	{
		int result = 0;

		for (int i = 0; i < forces.size(); i++)
		{
			result += ((Force) forces.elementAt(i)).getMaxAttack();
		}
		return result;
	}

	public int getTotalArmor()
	{
		int result = 0;

		for (int i = 0; i < forces.size(); i++)
		{
			result += ((Force) forces.elementAt(i)).getArmor();
		}
		return result;
	}

}
